@@ -1,20 +1,23 @@
 /*
  * load-library.c
  *
- * 
- * Author: 
+ *
+ * Authors:
  *  Richard Hult <rhult@hem.passagen.se>
- * 
- *  http://www.dtek.chalmers.se/~d4hult/oregano/ 
- * 
- * Copyright (C) 1999,2000  Richard Hult 
- * 
- * This program is free software; you can redistribute it and/or 
- * modify it under the terms of the GNU General Public License as 
- * published by the Free Software Foundation; either version 2 of the 
- * License, or (at your option) any later version. 
- * 
- * This program is distributed in the hope that it will be useful, 
+ *  Ricardo Markiewicz <rmarkie@fi.uba.ar>
+ *  Andres de Barbara <adebarbara@fi.uba.ar>
+ *
+ * Web page: http://arrakis.lug.fi.uba.ar/
+ *
+ * Copyright (C) 1999-2001  Richard Hult
+ * Copyright (C) 2003,2004  LUGFI
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
@@ -26,6 +29,7 @@
  */
 #include "config.h"
 #include <gnome.h>
+#include <string.h>
 #include "xml-compat.h"
 #include "main.h"
 #include "xml-helper.h"
@@ -45,7 +49,7 @@ typedef enum {
 	PARSE_SYMBOL_OBJECTS,
 	PARSE_SYMBOL_LINE,
 	PARSE_SYMBOL_ARC,
-        PARSE_SYMBOL_TEXT,
+	PARSE_SYMBOL_TEXT,
 	PARSE_SYMBOL_CONNECTIONS,
 	PARSE_SYMBOL_CONNECTION,
 	PARSE_PARTS,
@@ -86,42 +90,43 @@ typedef struct {
 	SymbolObject *object;
 } ParseState;
 
-static xmlEntityPtr get_entity (void *user_data, const CHAR *name);
+static xmlEntityPtr get_entity (void *user_data, const xmlChar *name);
 static void start_document (ParseState *state);
 static void end_document (ParseState *state);
-static void start_element (ParseState *state, const CHAR *name, const CHAR **attrs);
-static void end_element (ParseState *state, const CHAR *name);
+static void start_element (ParseState *state, const xmlChar *name,
+						   const xmlChar **attrs);
+static void end_element (ParseState *state, const xmlChar *name);
 
-static void characters (ParseState *state, const CHAR *chars, int len);
+static void my_characters (ParseState *state, const xmlChar *chars, int len);
 static void my_warning (void *user_data, const char *msg, ...);
 static void my_error (void *user_data, const char *msg, ...);
 static void my_fatal_error (void *user_data, const char *msg, ...);
 
 static xmlSAXHandler oreganoSAXParser = {
-    0, /* internalSubset */
-    0, /* isStandalone */
-    0, /* hasInternalSubset */
-    0, /* hasExternalSubset */
-    0, /* resolveEntity */
-    (getEntitySAXFunc) get_entity, /* getEntity */
-    0, /* entityDecl */
-    0, /* notationDecl */
-    0, /* attributeDecl */
-    0, /* elementDecl */
-    0, /* unparsedEntityDecl */
-    0, /* setDocumentLocator */
-    (startDocumentSAXFunc) start_document, /* startDocument */
-    (endDocumentSAXFunc) end_document, /* endDocument */
-    (startElementSAXFunc)start_element, /* startElement */
-    (endElementSAXFunc)end_element, /* endElement */
-    0, /* reference */
-    (charactersSAXFunc) characters, /* characters */
-    0, /* ignorableWhitespace */
-    0, /* processingInstruction */
-    0, /*(commentSAXFunc)0,  comment */
-    (warningSAXFunc)my_warning, /* warning */
-    (errorSAXFunc)my_error, /* error */
-    (fatalErrorSAXFunc)my_fatal_error, /* fatalError */
+	0, /* internalSubset */
+	0, /* isStandalone */
+	0, /* hasInternalSubset */
+	0, /* hasExternalSubset */
+	0, /* resolveEntity */
+	(getEntitySAXFunc) get_entity, /* getEntity */
+	0, /* entityDecl */
+	0, /* notationDecl */
+	0, /* attributeDecl */
+	0, /* elementDecl */
+	0, /* unparsedEntityDecl */
+	0, /* setDocumentLocator */
+	(startDocumentSAXFunc) start_document, /* startDocument */
+	(endDocumentSAXFunc) end_document, /* endDocument */
+	(startElementSAXFunc)start_element, /* startElement */
+	(endElementSAXFunc)end_element, /* endElement */
+	0, /* reference */
+	(charactersSAXFunc) my_characters, /* characters */
+	0, /* ignorableWhitespace */
+	0, /* processingInstruction */
+	0, /*(commentSAXFunc)0,	 comment */
+	(warningSAXFunc)my_warning, /* warning */
+	(errorSAXFunc)my_error, /* error */
+	(fatalErrorSAXFunc)my_fatal_error, /* fatalError */
 };
 
 LibrarySymbol *
@@ -141,8 +146,8 @@ library_get_symbol (const gchar *symbol_name)
 		   break;
 	}
 
- 	if (symbol == NULL) {
- 		g_message (_("Could not find the requested symbol: %s\n"), symbol_name);
+	if (symbol == NULL) {
+		g_message (_("Could not find the requested symbol: %s\n"), symbol_name);
 	}
 
 	return symbol;
@@ -167,7 +172,6 @@ Library *
 library_parse_xml_file (const gchar *filename)
 {
 	Library *library;
-	gint ret;
 	ParseState state;
 
 	if (oreganoXmlSAXParseFile (&oreganoSAXParser, &state, filename) < 0) {
@@ -189,8 +193,8 @@ start_document (ParseState *state)
 	state->state = PARSE_START;
 	state->unknown_depth = 0;
 	state->prev_state = PARSE_UNKNOWN;
-	
-        state->content = g_string_sized_new (128);
+
+	state->content = g_string_sized_new (128);
 	state->part = NULL;
 	state->symbol = NULL;
 	state->connection = NULL;
@@ -209,13 +213,12 @@ start_document (ParseState *state)
 static void
 end_document (ParseState *state)
 {
-
 	if (state->unknown_depth != 0)
-                g_warning ("unknown_depth != 0 (%d)", state->unknown_depth);
+		g_warning ("unknown_depth != 0 (%d)", state->unknown_depth);
 }
 
 static void
-start_element (ParseState *state, const CHAR *name, const CHAR **attrs)
+start_element (ParseState *state, const xmlChar *name, const xmlChar **attrs)
 {
 	switch (state->state) {
 	case PARSE_START:
@@ -227,15 +230,15 @@ start_element (ParseState *state, const CHAR *name, const CHAR **attrs)
 		break;
 
 	case PARSE_LIBRARY:
-                if (!strcmp (name, "ogo:author")) {
+		if (!strcmp (name, "ogo:author")) {
 			state->state = PARSE_AUTHOR;
 			g_string_truncate (state->content, 0);
-                } else if (!strcmp (name, "ogo:name")) {
+		} else if (!strcmp (name, "ogo:name")) {
 			state->state = PARSE_NAME;
 			g_string_truncate (state->content, 0);
-                } else if (!strcmp (name, "ogo:symbols")) {
+		} else if (!strcmp (name, "ogo:symbols")) {
 			state->state = PARSE_SYMBOLS;
-                } else if (!strcmp (name, "ogo:parts")) {
+		} else if (!strcmp (name, "ogo:parts")) {
 			state->state = PARSE_PARTS;
 		} else {
 			state->prev_state = state->state;
@@ -245,7 +248,7 @@ start_element (ParseState *state, const CHAR *name, const CHAR **attrs)
 		break;
 
 	case PARSE_SYMBOLS:
-                if (!strcmp (name, "ogo:symbol")) {
+		if (!strcmp (name, "ogo:symbol")) {
 			state->state = PARSE_SYMBOL;
 			state->symbol = g_new0 (LibrarySymbol, 1);
 		} else {
@@ -256,12 +259,12 @@ start_element (ParseState *state, const CHAR *name, const CHAR **attrs)
 		break;
 
 	case PARSE_SYMBOL:
-                if (!strcmp (name, "ogo:name")) {
+		if (!strcmp (name, "ogo:name")) {
 			state->state = PARSE_SYMBOL_NAME;
 			g_string_truncate (state->content, 0);
-                } else if (!strcmp (name, "ogo:objects")) {
+		} else if (!strcmp (name, "ogo:objects")) {
 			state->state = PARSE_SYMBOL_OBJECTS;
-                } else if (!strcmp (name, "ogo:connections")) {
+		} else if (!strcmp (name, "ogo:connections")) {
 			state->state = PARSE_SYMBOL_CONNECTIONS;
 		} else {
 			state->prev_state = state->state;
@@ -269,24 +272,22 @@ start_element (ParseState *state, const CHAR *name, const CHAR **attrs)
 			state->unknown_depth++;
 		}
 		break;
-
 	case PARSE_SYMBOL_OBJECTS:
-                if (!strcmp (name, "ogo:line")) {
+		if (!strcmp (name, "ogo:line")) {
 			state->object = g_new0 (SymbolObject, 1);
 			state->object->type = SYMBOL_OBJECT_LINE;
 			state->state = PARSE_SYMBOL_LINE;
 			g_string_truncate (state->content, 0);
-                } else if (!strcmp (name, "ogo:arc")) {
+		} else if (!strcmp (name, "ogo:arc")) {
 			state->object = g_new0 (SymbolObject, 1);
 			state->object->type = SYMBOL_OBJECT_ARC;
 			state->state = PARSE_SYMBOL_ARC;
 			g_string_truncate (state->content, 0);
-
-                } else if (!strcmp (name, "ogo:text")) {
-                        state->object = g_new0 (SymbolObject, 1);
-                        state->object->type = SYMBOL_OBJECT_TEXT;
-                        state->state = PARSE_SYMBOL_TEXT;
-                        g_string_truncate (state->content, 0);
+		} else if (!strcmp (name, "ogo:text")) {
+			state->object = g_new0 (SymbolObject, 1);
+			state->object->type = SYMBOL_OBJECT_TEXT;
+			state->state = PARSE_SYMBOL_TEXT;
+			g_string_truncate (state->content, 0);
 		} else {
 			state->prev_state = state->state;
 			state->state = PARSE_UNKNOWN;
@@ -295,7 +296,7 @@ start_element (ParseState *state, const CHAR *name, const CHAR **attrs)
 		break;
 
 	case PARSE_SYMBOL_CONNECTIONS:
-                if (!strcmp (name, "ogo:connection")) {
+		if (!strcmp (name, "ogo:connection")) {
 			state->state = PARSE_SYMBOL_CONNECTION;
 			state->connection = g_new0 (Connection, 1);
 			g_string_truncate (state->content, 0);
@@ -305,10 +306,10 @@ start_element (ParseState *state, const CHAR *name, const CHAR **attrs)
 			state->unknown_depth++;
 		}
 		break;
-		
+
 
 	case PARSE_PARTS:
-                if (!strcmp (name, "ogo:part")) {
+		if (!strcmp (name, "ogo:part")) {
 			state->state = PARSE_PART;
 			state->part = g_new0 (LibraryPart, 1);
 			state->part->library = state->library;
@@ -319,18 +320,18 @@ start_element (ParseState *state, const CHAR *name, const CHAR **attrs)
 		}
 		break;
 	case PARSE_PART:
-                if (!strcmp (name, "ogo:name")) {
+		if (!strcmp (name, "ogo:name")) {
 			state->state = PARSE_PART_NAME;
 			g_string_truncate (state->content, 0);
-                } else if (!strcmp (name, "ogo:description")) {
+		} else if (!strcmp (name, "ogo:description")) {
 			state->state = PARSE_PART_DESCRIPTION;
 			g_string_truncate (state->content, 0);
-                } else if (!strcmp (name, "ogo:symbol")) {
+		} else if (!strcmp (name, "ogo:symbol")) {
 			state->state = PARSE_PART_USESYMBOL;
 			g_string_truncate (state->content, 0);
-                } else if (!strcmp (name, "ogo:labels")) {
+		} else if (!strcmp (name, "ogo:labels")) {
 			state->state = PARSE_PART_LABELS;
-                } else if (!strcmp (name, "ogo:properties")) {
+		} else if (!strcmp (name, "ogo:properties")) {
 			state->state = PARSE_PART_PROPERTIES;
 		} else {
 			state->prev_state = state->state;
@@ -339,7 +340,7 @@ start_element (ParseState *state, const CHAR *name, const CHAR **attrs)
 		}
 		break;
 	case PARSE_PART_LABELS:
-                if (!strcmp (name, "ogo:label")) {
+		if (!strcmp (name, "ogo:label")) {
 			state->state = PARSE_PART_LABEL;
 			state->label = g_new0 (PartLabel, 1);
 		} else {
@@ -349,14 +350,14 @@ start_element (ParseState *state, const CHAR *name, const CHAR **attrs)
 		}
 		break;
 	case PARSE_PART_LABEL:
-                if (!strcmp (name, "ogo:name")) {
+		if (!strcmp (name, "ogo:name")) {
 			state->state = PARSE_PART_LABEL_NAME;
 			g_string_truncate (state->content, 0);
-                } else if (!strcmp (name, "ogo:text")) {
-		        state->state = PARSE_PART_LABEL_TEXT;
+		} else if (!strcmp (name, "ogo:text")) {
+			state->state = PARSE_PART_LABEL_TEXT;
 			g_string_truncate (state->content, 0);
-                } else if (!strcmp (name, "ogo:position")) {
-		        state->state = PARSE_PART_LABEL_POS;
+		} else if (!strcmp (name, "ogo:position")) {
+			state->state = PARSE_PART_LABEL_POS;
 			g_string_truncate (state->content, 0);
 		} else {
 			state->prev_state = state->state;
@@ -364,9 +365,9 @@ start_element (ParseState *state, const CHAR *name, const CHAR **attrs)
 			state->unknown_depth++;
 		}
 		break;
-		
+
 	case PARSE_PART_PROPERTIES:
-                if (!strcmp (name, "ogo:property")) {
+		if (!strcmp (name, "ogo:property")) {
 			state->state = PARSE_PART_PROPERTY;
 			state->property = g_new0 (Property, 1);
 		} else {
@@ -376,11 +377,11 @@ start_element (ParseState *state, const CHAR *name, const CHAR **attrs)
 		}
 		break;
 	case PARSE_PART_PROPERTY:
-                if (!strcmp (name, "ogo:name")) {
+		if (!strcmp (name, "ogo:name")) {
 			state->state = PARSE_PART_PROPERTY_NAME;
 			g_string_truncate (state->content, 0);
-                } else if (!strcmp (name, "ogo:value")) {
-		        state->state = PARSE_PART_PROPERTY_VALUE;
+		} else if (!strcmp (name, "ogo:value")) {
+			state->state = PARSE_PART_PROPERTY_VALUE;
 			g_string_truncate (state->content, 0);
 		} else {
 			state->prev_state = state->state;
@@ -388,11 +389,11 @@ start_element (ParseState *state, const CHAR *name, const CHAR **attrs)
 			state->unknown_depth++;
 		}
 		break;
-		
+
 	case PARSE_SYMBOL_NAME:
 	case PARSE_SYMBOL_LINE:
 	case PARSE_SYMBOL_ARC:
-        case PARSE_SYMBOL_TEXT:
+	case PARSE_SYMBOL_TEXT:
 	case PARSE_SYMBOL_CONNECTION:
 	case PARSE_PART_NAME:
 	case PARSE_PART_DESCRIPTION:
@@ -405,7 +406,7 @@ start_element (ParseState *state, const CHAR *name, const CHAR **attrs)
 	case PARSE_NAME:
 	case PARSE_AUTHOR:
 		/* there should be no tags inside these types of tags */
-                g_message ("*** '%s' tag found", name);
+		g_message ("*** '%s' tag found", name);
 		state->prev_state = state->state;
 		state->state = PARSE_UNKNOWN;
 		state->unknown_depth++;
@@ -418,14 +419,14 @@ start_element (ParseState *state, const CHAR *name, const CHAR **attrs)
 		break;
 	case PARSE_FINISH:
 		/* should not start new elements in this state */
-                g_assert_not_reached ();
+		g_assert_not_reached ();
 		break;
 	}
 	/*g_message("Start element %s (state %s)", name, states[state->state]);*/
 }
 
 static void
-end_element (ParseState *state, const CHAR *name)
+end_element (ParseState *state, const xmlChar *name)
 {
 	switch (state->state) {
 	case PARSE_UNKNOWN:
@@ -445,7 +446,8 @@ end_element (ParseState *state, const CHAR *name)
 		state->state = PARSE_LIBRARY;
 		break;
 	case PARSE_SYMBOL:
-		g_hash_table_insert (state->library->symbol_hash, state->symbol->name, state->symbol);
+		g_hash_table_insert (state->library->symbol_hash, state->symbol->name,
+							 state->symbol);
 		state->state = PARSE_SYMBOLS;
 		break;
 	case PARSE_SYMBOL_NAME:
@@ -455,22 +457,22 @@ end_element (ParseState *state, const CHAR *name)
 	case PARSE_SYMBOL_OBJECTS:
 		state->state = PARSE_SYMBOL;
 		break;
-	case PARSE_SYMBOL_LINE: 
+	case PARSE_SYMBOL_LINE:
 	{
 		int i, j;
-                gchar *ptr;
-                gchar **points;
+		gchar *ptr;
+		gchar **points;
 
-                i= sscanf (state->content->str,"%d %n",
-                          &state->object->u.uline.spline,&j);
-                if ( i )
-                        ptr = state->content->str+j;
-                else {
-                        state->object->u.uline.spline = FALSE;
-                        ptr = state->content->str;
-                }
+		i= sscanf (state->content->str,"%d %n",
+				   &state->object->u.uline.spline,&j);
+		if ( i )
+			ptr = state->content->str+j;
+		else {
+			state->object->u.uline.spline = FALSE;
+			ptr = state->content->str;
+		}
 
-                points = g_strsplit (ptr, "(", 0);
+		points = g_strsplit (ptr, "(", 0);
 
 		i = 0;
 		/* Count the points. */
@@ -480,56 +482,58 @@ end_element (ParseState *state, const CHAR *name)
 
 		/* Do not count the first string, which simply is a (. */
 		i--;
-		
+
 		/* Construct gnome canvas points. */
-                state->object->u.uline.line = gnome_canvas_points_new (i);
+		state->object->u.uline.line = gnome_canvas_points_new (i);
 		for (j = 0; j < i; j++) {
 			double x, y;
 
 			sscanf (points[j+1], "%lf %lf)", &x, &y);
 
-                        state->object->u.uline.line->coords[2 * j] = x;
-                        state->object->u.uline.line->coords[2 * j + 1] = y;
+			state->object->u.uline.line->coords[2 * j] = x;
+			state->object->u.uline.line->coords[2 * j + 1] = y;
 		}
 
 		g_strfreev (points);
 		state->symbol->symbol_objects = g_slist_prepend (state->symbol->symbol_objects, state->object);
 		state->state = PARSE_SYMBOL_OBJECTS;
 	}
-	break; 
-	case PARSE_SYMBOL_ARC: 
-		sscanf (state->content->str, "(%lf %lf)(%lf %lf)", 
+	break;
+	case PARSE_SYMBOL_ARC:
+		sscanf (state->content->str, "(%lf %lf)(%lf %lf)",
 			&state->object->u.arc.x1, &state->object->u.arc.y1,
 			&state->object->u.arc.x2, &state->object->u.arc.y2);
 		state->symbol->symbol_objects = g_slist_prepend (state->symbol->symbol_objects, state->object);
 		state->state = PARSE_SYMBOL_OBJECTS;
-		break; 
+		break;
 
-        case PARSE_SYMBOL_TEXT:
-                sscanf ( state->content->str,"(%d %d)%s",
-                        &state->object->u.text.x,
-                        &state->object->u.text.y,
-                        state->object->u.text.str
-                        );
-                state->symbol->symbol_objects = g_slist_prepend (state->symbol->symbol_objects, state->object);
-                state->state = PARSE_SYMBOL_OBJECTS;
-                break;
+	case PARSE_SYMBOL_TEXT:
+		sscanf ( state->content->str,"(%d %d)%s",
+				 &state->object->u.text.x,
+				 &state->object->u.text.y,
+				 state->object->u.text.str
+				 );
+		state->symbol->symbol_objects = g_slist_prepend (state->symbol->symbol_objects, state->object);
+		state->state = PARSE_SYMBOL_OBJECTS;
+		break;
 
 	case PARSE_SYMBOL_CONNECTIONS:
 		state->state = PARSE_SYMBOL;
 		state->symbol->connections = g_slist_reverse (state->symbol->connections);
 		break;
-	case PARSE_SYMBOL_CONNECTION: 
-		sscanf (state->content->str, "(%lf %lf)", &state->connection->pos.x, &state->connection->pos.y);
+	case PARSE_SYMBOL_CONNECTION:
+		sscanf (state->content->str, "(%lf %lf)", &state->connection->pos.x,
+				&state->connection->pos.y);
 		state->symbol->connections = g_slist_prepend (state->symbol->connections, state->connection);
 		state->state = PARSE_SYMBOL_CONNECTIONS;
-		break; 
+		break;
 
 	case PARSE_PARTS:
 		state->state = PARSE_LIBRARY;
 		break;
 	case PARSE_PART:
-		g_hash_table_insert (state->library->part_hash, state->part->name, state->part);
+		g_hash_table_insert (state->library->part_hash, state->part->name,
+							 state->part);
 		state->state = PARSE_PARTS;
 		break;
 	case PARSE_PART_NAME:
@@ -549,7 +553,8 @@ end_element (ParseState *state, const CHAR *name)
 		break;
 	case PARSE_PART_LABEL:
 		state->state = PARSE_PART_LABELS;
-		state->part->labels = g_slist_prepend (state->part->labels, state->label); 
+		state->part->labels = g_slist_prepend (state->part->labels,
+											   state->label);
 		break;
 	case PARSE_PART_LABEL_NAME:
 		state->label->name = g_strdup (state->content->str);
@@ -560,7 +565,8 @@ end_element (ParseState *state, const CHAR *name)
 		state->state = PARSE_PART_LABEL;
 		break;
 	case PARSE_PART_LABEL_POS:
-		sscanf (state->content->str, "(%lf %lf)", &state->label->pos.x, &state->label->pos.y);
+		sscanf (state->content->str, "(%lf %lf)", &state->label->pos.x,
+				&state->label->pos.y);
 		state->state = PARSE_PART_LABEL;
 		break;
 	case PARSE_PART_PROPERTIES:
@@ -568,7 +574,8 @@ end_element (ParseState *state, const CHAR *name)
 		break;
 	case PARSE_PART_PROPERTY:
 		state->state = PARSE_PART_PROPERTIES;
-		state->part->properties = g_slist_prepend (state->part->properties, state->property);
+		state->part->properties = g_slist_prepend (state->part->properties,
+												   state->property);
 		break;
 	case PARSE_PART_PROPERTY_NAME:
 		state->property->name = g_strdup (state->content->str);
@@ -588,21 +595,21 @@ end_element (ParseState *state, const CHAR *name)
 	case PARSE_START:
 	case PARSE_FINISH:
 		/* There should not be a closing tag in this state. */
-                g_assert_not_reached ();
+		g_assert_not_reached ();
 		break;
 	}
 	/*g_message("End element %s (state %s)", name, states[state->state]);*/
 }
 
 static void
-characters (ParseState *state, const CHAR *chars, int len)
+my_characters (ParseState *state, const xmlChar *chars, int len)
 {
 	int i;
-	
+
 	if (state->state == PARSE_FINISH ||
-	    state->state == PARSE_START ||
-	    state->state == PARSE_PARTS ||
-	    state->state == PARSE_PART)
+		state->state == PARSE_START ||
+		state->state == PARSE_PARTS ||
+		state->state == PARSE_PART)
 		return;
 
 	for (i = 0; i < len; i++)
@@ -610,7 +617,7 @@ characters (ParseState *state, const CHAR *chars, int len)
 }
 
 static xmlEntityPtr
-get_entity (void *user_data, const CHAR *name)
+get_entity (void *user_data, const xmlChar *name)
 {
 	return xmlGetPredefinedEntity (name);
 }
@@ -619,7 +626,7 @@ static void
 my_warning (void *user_data, const char *msg, ...)
 {
 	va_list args;
-	
+
 	va_start (args, msg);
 	g_logv ("XML", G_LOG_LEVEL_WARNING, msg, args);
 	va_end (args);
@@ -629,9 +636,9 @@ static void
 my_error (void *user_data, const char *msg, ...)
 {
 	va_list args;
-	
+
 	va_start (args, msg);
-        g_logv ("XML", G_LOG_LEVEL_CRITICAL, msg, args);
+	g_logv ("XML", G_LOG_LEVEL_CRITICAL, msg, args);
 	va_end (args);
 }
 
@@ -639,7 +646,7 @@ static void
 my_fatal_error (void *user_data, const char *msg, ...)
 {
 	va_list args;
-	
+
 	va_start (args, msg);
 	g_logv ("XML", G_LOG_LEVEL_ERROR, msg, args);
 	va_end (args);
