@@ -1,20 +1,23 @@
 /*
  * load-schematic.c
  *
- * 
- * Author: 
+ *
+ * Author:
  *  Richard Hult <rhult@hem.passagen.se>
- * 
- *  http://www.dtek.chalmers.se/~d4hult/oregano/ 
- * 
- * Copyright (C) 1999,2000  Richard Hult 
- * 
- * This program is free software; you can redistribute it and/or 
- * modify it under the terms of the GNU General Public License as 
- * published by the Free Software Foundation; either version 2 of the 
- * License, or (at your option) any later version. 
- * 
- * This program is distributed in the hope that it will be useful, 
+ *  Ricardo Markiewicz <rmarkie@fi.uba.ar>
+ *  Andres de Barbara <adebarbara@fi.uba.ar>
+ *
+ * Web page: http://arrakis.lug.fi.uba.ar/
+ *
+ * Copyright (C) 1999-2001  Richard Hult
+ * Copyright (C) 2003,2004  LUGFI
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
@@ -27,6 +30,7 @@
 
 #include <config.h>
 #include <gnome.h>
+#include <string.h>
 #include "xml-compat.h"
 #include "main.h"
 #include "xml-helper.h"
@@ -43,6 +47,7 @@ typedef enum {
 	PARSE_SCHEMATIC,
 	PARSE_TITLE,
 	PARSE_AUTHOR,
+	PARSE_COMMENTS,
 
 	PARSE_SIMULATION_SETTINGS,
 	PARSE_TRANSIENT_SETTINGS,
@@ -51,33 +56,34 @@ typedef enum {
 	PARSE_TRANSIENT_STOP,
 	PARSE_TRANSIENT_STEP,
 	PARSE_TRANSIENT_STEP_ENABLE,
+	PARSE_TRANSIENT_INIT_COND,
 
-        PARSE_AC_SETTINGS,
-        PARSE_AC_ENABLED,
-        PARSE_AC_NPOINTS,
-        PARSE_AC_START,
-        PARSE_AC_STOP,
+	PARSE_AC_SETTINGS,
+	PARSE_AC_ENABLED,
+	PARSE_AC_NPOINTS,
+	PARSE_AC_START,
+	PARSE_AC_STOP,
 
-        PARSE_DC_SETTINGS,
-        PARSE_DC_ENABLED,
-        PARSE_DC_VSRC1,
-        PARSE_DC_START1,
-        PARSE_DC_STOP1,
-        PARSE_DC_STEP1,
-        PARSE_DC_VSRC2,
-        PARSE_DC_START2,
-        PARSE_DC_STOP2,
-        PARSE_DC_STEP2,
+	PARSE_DC_SETTINGS,
+	PARSE_DC_ENABLED,
+	PARSE_DC_VSRC1,
+	PARSE_DC_START1,
+	PARSE_DC_STOP1,
+	PARSE_DC_STEP1,
+	PARSE_DC_VSRC2,
+	PARSE_DC_START2,
+	PARSE_DC_STOP2,
+	PARSE_DC_STEP2,
 
 
-        PARSE_OPTION_LIST,
-        PARSE_OPTION,
-        PARSE_OPTION_NAME,
-        PARSE_OPTION_VALUE,
+	PARSE_OPTION_LIST,
+	PARSE_OPTION,
+	PARSE_OPTION_NAME,
+	PARSE_OPTION_VALUE,
 
 
 	PARSE_ZOOM,
-	
+
 	PARSE_PARTS,
 	PARSE_PART,
 	PARSE_PART_NAME,
@@ -108,7 +114,7 @@ typedef enum {
 	PARSE_TEXTBOX_POSITION,
 	PARSE_FINISH,
 	PARSE_UNKNOWN,
-	PARSE_ERROR,
+	PARSE_ERROR
 } State;
 
 typedef struct {
@@ -121,13 +127,14 @@ typedef struct {
 
 	char *title;
 	char *author;
+	char *comments;
 
 	char *textbox_text;
 
 	/* Temporary place holder for a wire */
-        SheetPos wire_start;
+	SheetPos wire_start;
 	SheetPos wire_end;
-	
+
 	/* Temporary place holder for a part */
 	LibraryPart *part;
 	PartLabel *label;
@@ -136,17 +143,18 @@ typedef struct {
 	int rotation;
 	IDFlip flip;
 
-      /*  Temporary place holder for an option */
-      SimOption *option;
+	/*	Temporary place holder for an option */
+	SimOption *option;
 } ParseState;
 
-static xmlEntityPtr get_entity (void *user_data, const CHAR *name);
+static xmlEntityPtr get_entity (void *user_data, const xmlChar *name);
 static void start_document (ParseState *state);
 static void end_document (ParseState *state);
-static void start_element (ParseState *state, const CHAR *name, const CHAR **attrs);
-static void end_element (ParseState *state, const CHAR *name);
+static void start_element (ParseState *state, const xmlChar *name,
+						   const xmlChar **attrs);
+static void end_element (ParseState *state, const xmlChar *name);
 
-static void characters (ParseState *state, const CHAR *chars, int len);
+static void my_characters (ParseState *state, const xmlChar *chars, int len);
 static void my_warning (void *user_data, const char *msg, ...);
 static void my_error (void *user_data, const char *msg, ...);
 static void my_fatal_error (void *user_data, const char *msg, ...);
@@ -155,30 +163,30 @@ static void create_wire (ParseState *state);
 static void create_part (ParseState *state);
 
 static xmlSAXHandler oreganoSAXParser = {
-    0, /* internalSubset */
-    0, /* isStandalone */
-    0, /* hasInternalSubset */
-    0, /* hasExternalSubset */
-    0, /* resolveEntity */
-    (getEntitySAXFunc) get_entity, /* getEntity */
-    0, /* entityDecl */
-    0, /* notationDecl */
-    0, /* attributeDecl */
-    0, /* elementDecl */
-    0, /* unparsedEntityDecl */
-    0, /* setDocumentLocator */
-    (startDocumentSAXFunc) start_document, /* startDocument */
-    (endDocumentSAXFunc) end_document, /* endDocument */
-    (startElementSAXFunc)start_element, /* startElement */
-    (endElementSAXFunc)end_element, /* endElement */
-    0, /* reference */
-    (charactersSAXFunc) characters, /* characters */
-    0, /* ignorableWhitespace */
-    0, /* processingInstruction */
-    0, /*(commentSAXFunc)0,  comment */
-    (warningSAXFunc)my_warning, /* warning */
-    (errorSAXFunc)my_error, /* error */
-    (fatalErrorSAXFunc)my_fatal_error, /* fatalError */
+	0, /* internalSubset */
+	0, /* isStandalone */
+	0, /* hasInternalSubset */
+	0, /* hasExternalSubset */
+	0, /* resolveEntity */
+	(getEntitySAXFunc) get_entity, /* getEntity */
+	0, /* entityDecl */
+	0, /* notationDecl */
+	0, /* attributeDecl */
+	0, /* elementDecl */
+	0, /* unparsedEntityDecl */
+	0, /* setDocumentLocator */
+	(startDocumentSAXFunc) start_document, /* startDocument */
+	(endDocumentSAXFunc) end_document, /* endDocument */
+	(startElementSAXFunc)start_element, /* startElement */
+	(endElementSAXFunc)end_element, /* endElement */
+	0, /* reference */
+	(charactersSAXFunc) my_characters, /* characters */
+	0, /* ignorableWhitespace */
+	0, /* processingInstruction */
+	0, /*(commentSAXFunc)0,	 comment */
+	(warningSAXFunc)my_warning, /* warning */
+	(errorSAXFunc)my_error, /* error */
+	(fatalErrorSAXFunc)my_fatal_error, /* fatalError */
 };
 
 static void
@@ -240,15 +248,22 @@ schematic_parse_xml_file (Schematic *sm, const char *filename)
 
 	state.schematic = sm;
 	state.sim_settings = schematic_get_sim_settings (sm);
+	state.title = NULL;
+	state.author = NULL;
+	state.comments = NULL;
 
 	if (oreganoXmlSAXParseFile (&oreganoSAXParser, &state, filename) < 0) {
-                g_warning ("Document not well formed!");
+		g_warning ("Document not well formed!");
 		retval = -1;
 	}
 
 	if (state.state == PARSE_ERROR)
 		retval = -2;
 
+	schematic_set_author (sm, state.author);
+	schematic_set_title (sm, state.title);
+	schematic_set_comments (sm, state.comments);
+
 	return retval;
 }
 
@@ -258,8 +273,8 @@ start_document (ParseState *state)
 	state->state = PARSE_START;
 	state->unknown_depth = 0;
 	state->prev_state = PARSE_UNKNOWN;
-	
-        state->content = g_string_sized_new (128);
+
+	state->content = g_string_sized_new (128);
 	state->part = NULL;
 }
 
@@ -268,11 +283,11 @@ end_document (ParseState *state)
 {
 
 	if (state->unknown_depth != 0)
-                g_warning ("unknown_depth != 0 (%d)", state->unknown_depth);
+		g_warning ("unknown_depth != 0 (%d)", state->unknown_depth);
 }
 
 static void
-start_element (ParseState *state, const CHAR *name, const CHAR **attrs)
+start_element (ParseState *state, const xmlChar *name, const xmlChar **attrs)
 {
 	switch (state->state) {
 	case PARSE_START:
@@ -284,23 +299,26 @@ start_element (ParseState *state, const CHAR *name, const CHAR **attrs)
 		break;
 
 	case PARSE_SCHEMATIC:
-                if (!strcmp (name, "ogo:author")) {
+		if (!strcmp (name, "ogo:author")) {
 			state->state = PARSE_AUTHOR;
 			g_string_truncate (state->content, 0);
-                } else if (!strcmp (name, "ogo:title")) {
+		} else if (!strcmp (name, "ogo:title")) {
 			state->state = PARSE_TITLE;
 			g_string_truncate (state->content, 0);
-                } else if (!strcmp (name, "ogo:zoom")) {
+		} else if (!strcmp (name, "ogo:comments")) {
+			state->state = PARSE_COMMENTS;
+			g_string_truncate (state->content, 0);
+		} else if (!strcmp (name, "ogo:zoom")) {
 			state->state = PARSE_ZOOM;
 			g_string_truncate (state->content, 0);
-                } else if (!strcmp (name, "ogo:simulation-settings")) {
+		} else if (!strcmp (name, "ogo:simulation-settings")) {
 			state->state = PARSE_SIMULATION_SETTINGS;
-                } else if (!strcmp (name, "ogo:parts")) {
+		} else if (!strcmp (name, "ogo:parts")) {
 			state->state = PARSE_PARTS;
-                } else if (!strcmp (name, "ogo:wires")) {
-			state->state = PARSE_WIRES;	
-                } else if (!strcmp (name, "ogo:textboxes")) {
-			state->state = PARSE_TEXTBOXES;	
+		} else if (!strcmp (name, "ogo:wires")) {
+			state->state = PARSE_WIRES;
+		} else if (!strcmp (name, "ogo:textboxes")) {
+			state->state = PARSE_TEXTBOXES;
 		} else {
 			state->prev_state = state->state;
 			state->state = PARSE_UNKNOWN;
@@ -309,15 +327,15 @@ start_element (ParseState *state, const CHAR *name, const CHAR **attrs)
 		break;
 
 	case PARSE_SIMULATION_SETTINGS:
-                if (!strcmp (name, "ogo:transient")) {
+		if (!strcmp (name, "ogo:transient")) {
 			state->state = PARSE_TRANSIENT_SETTINGS;
-                } else if ( !strcmp (name,"ogo:ac")) {
-                        state->state = PARSE_AC_SETTINGS;
+		} else if ( !strcmp (name,"ogo:ac")) {
+			state->state = PARSE_AC_SETTINGS;
 		} else if ( !strcmp (name,"ogo:dc-sweep")) {
-                        state->state = PARSE_DC_SETTINGS;
-                }
-                else if (!strcmp (name, "ogo:options")) {
-                   state->state = PARSE_OPTION_LIST;
+			state->state = PARSE_DC_SETTINGS;
+		}
+		else if (!strcmp (name, "ogo:options")) {
+			state->state = PARSE_OPTION_LIST;
 		} else {
 			state->prev_state = state->state;
 			state->state = PARSE_UNKNOWN;
@@ -326,110 +344,113 @@ start_element (ParseState *state, const CHAR *name, const CHAR **attrs)
 		break;
 
 	case PARSE_TRANSIENT_SETTINGS:
-                if (!strcmp (name, "ogo:enabled")) {
+		if (!strcmp (name, "ogo:enabled")) {
 			state->state = PARSE_TRANSIENT_ENABLED;
 			g_string_truncate (state->content, 0);
-                } else if (!strcmp (name, "ogo:start")) {
+		} else if (!strcmp (name, "ogo:start")) {
 			state->state = PARSE_TRANSIENT_START;
 			g_string_truncate (state->content, 0);
-                } else if (!strcmp (name, "ogo:stop")) {
+		} else if (!strcmp (name, "ogo:stop")) {
 			state->state = PARSE_TRANSIENT_STOP;
 			g_string_truncate (state->content, 0);
-                } else if (!strcmp (name, "ogo:step")) {
+		} else if (!strcmp (name, "ogo:step")) {
 			state->state = PARSE_TRANSIENT_STEP;
 			g_string_truncate (state->content, 0);
-                } else if (!strcmp (name, "ogo:step-enabled")) {
+		} else if (!strcmp (name, "ogo:step-enabled")) {
 			state->state = PARSE_TRANSIENT_STEP_ENABLE;
 			g_string_truncate (state->content, 0);
+		} else if (!strcmp (name, "ogo:init-conditions")) {
+			state->state = PARSE_TRANSIENT_INIT_COND;
+			g_string_truncate (state->content, 0);
 		} else {
 			state->prev_state = state->state;
 			state->state = PARSE_UNKNOWN;
 			state->unknown_depth++;
 		}
 		break;
-		
-        case PARSE_AC_SETTINGS:
-                if (!strcmp (name, "ogo:enabled")) {
-                        state->state = PARSE_AC_ENABLED;
-                        g_string_truncate (state->content, 0);
-                } else if (!strcmp (name, "ogo:npoints")) {
-                        state->state = PARSE_AC_NPOINTS;
-                        g_string_truncate (state->content, 0);
-                } else if (!strcmp (name, "ogo:start")) {
-                        state->state = PARSE_AC_START;
-                        g_string_truncate (state->content, 0);
-                } else if (!strcmp (name, "ogo:stop")) {
-                        state->state = PARSE_AC_STOP;
-                        g_string_truncate (state->content, 0);
-                } else {
-                        state->prev_state = state->state;
-                        state->state = PARSE_UNKNOWN;
-                        state->unknown_depth++;
-                }
-                break;
 
-        case PARSE_DC_SETTINGS:
-                if (!strcmp (name, "ogo:enabled")) {
-                        state->state = PARSE_DC_ENABLED;
-                        g_string_truncate (state->content, 0);
-                } else if (!strcmp (name, "ogo:vsrc1")) {
-                        state->state = PARSE_DC_VSRC1;
-                        g_string_truncate (state->content, 0);
-                } else if (!strcmp (name, "ogo:start1")) {
-                        state->state = PARSE_DC_START1;
-                        g_string_truncate (state->content, 0);
-                } else if (!strcmp (name, "ogo:stop1")) {
-                        state->state = PARSE_DC_STOP1;
-                        g_string_truncate (state->content, 0);
-                } else if (!strcmp (name, "ogo:step1")) {
-                        state->state = PARSE_DC_STEP1;
-                        g_string_truncate (state->content, 0);
-                } else if (!strcmp (name, "ogo:vsrc2")) {
-                        state->state = PARSE_DC_VSRC2;
-                        g_string_truncate (state->content, 0);
-                } else if (!strcmp (name, "ogo:start2")) {
-                        state->state = PARSE_DC_START2;
-                        g_string_truncate (state->content, 0);
-                } else if (!strcmp (name, "ogo:stop2")) {
-                        state->state = PARSE_DC_STOP2;
-                        g_string_truncate (state->content, 0);
-                } else if (!strcmp (name, "ogo:step2")) {
-                        state->state = PARSE_DC_STEP2;
-                        g_string_truncate (state->content, 0);
-                } else {
-                        state->prev_state = state->state;
-                        state->state = PARSE_UNKNOWN;
-                        state->unknown_depth++;
-                }
-                break;
+	case PARSE_AC_SETTINGS:
+		if (!strcmp (name, "ogo:enabled")) {
+			state->state = PARSE_AC_ENABLED;
+			g_string_truncate (state->content, 0);
+		} else if (!strcmp (name, "ogo:npoints")) {
+			state->state = PARSE_AC_NPOINTS;
+			g_string_truncate (state->content, 0);
+		} else if (!strcmp (name, "ogo:start")) {
+			state->state = PARSE_AC_START;
+			g_string_truncate (state->content, 0);
+		} else if (!strcmp (name, "ogo:stop")) {
+			state->state = PARSE_AC_STOP;
+			g_string_truncate (state->content, 0);
+		} else {
+			state->prev_state = state->state;
+			state->state = PARSE_UNKNOWN;
+			state->unknown_depth++;
+		}
+		break;
 
-        case PARSE_OPTION_LIST:
-               if (!strcmp (name, "ogo:option")) {
-                   state->state = PARSE_OPTION;
-                   state->option= g_new0(SimOption,1);
-                   g_string_truncate (state->content, 0);
-                } else {
-                        state->prev_state = state->state;
-                        state->state = PARSE_UNKNOWN;
-                        state->unknown_depth++;
-                }
-               break;
-       case PARSE_OPTION:
-          if (!strcmp (name, "ogo:name")) {
-             state->state = PARSE_OPTION_NAME;
-             g_string_truncate (state->content, 0);
-          } else if (!strcmp (name, "ogo:value")) {
-             state->state = PARSE_OPTION_VALUE;
-             g_string_truncate (state->content, 0);
-          } else {
-             state->prev_state = state->state;
-             state->state = PARSE_UNKNOWN;
-             state->unknown_depth++;
-          }
+	case PARSE_DC_SETTINGS:
+		if (!strcmp (name, "ogo:enabled")) {
+			state->state = PARSE_DC_ENABLED;
+			g_string_truncate (state->content, 0);
+		} else if (!strcmp (name, "ogo:vsrc1")) {
+			state->state = PARSE_DC_VSRC1;
+			g_string_truncate (state->content, 0);
+		} else if (!strcmp (name, "ogo:start1")) {
+			state->state = PARSE_DC_START1;
+			g_string_truncate (state->content, 0);
+		} else if (!strcmp (name, "ogo:stop1")) {
+			state->state = PARSE_DC_STOP1;
+			g_string_truncate (state->content, 0);
+		} else if (!strcmp (name, "ogo:step1")) {
+			state->state = PARSE_DC_STEP1;
+			g_string_truncate (state->content, 0);
+		} else if (!strcmp (name, "ogo:vsrc2")) {
+			state->state = PARSE_DC_VSRC2;
+			g_string_truncate (state->content, 0);
+		} else if (!strcmp (name, "ogo:start2")) {
+			state->state = PARSE_DC_START2;
+			g_string_truncate (state->content, 0);
+		} else if (!strcmp (name, "ogo:stop2")) {
+			state->state = PARSE_DC_STOP2;
+			g_string_truncate (state->content, 0);
+		} else if (!strcmp (name, "ogo:step2")) {
+			state->state = PARSE_DC_STEP2;
+			g_string_truncate (state->content, 0);
+		} else {
+			state->prev_state = state->state;
+			state->state = PARSE_UNKNOWN;
+			state->unknown_depth++;
+		}
+		break;
 
-          break;
+	case PARSE_OPTION_LIST:
+		if (!strcmp (name, "ogo:option")) {
+			state->state = PARSE_OPTION;
+			state->option= g_new0(SimOption,1);
+			g_string_truncate (state->content, 0);
+		} else {
+			state->prev_state = state->state;
+			state->state = PARSE_UNKNOWN;
+			state->unknown_depth++;
+		}
+		break;
+	case PARSE_OPTION:
+		if (!strcmp (name, "ogo:name")) {
+			state->state = PARSE_OPTION_NAME;
+			g_string_truncate (state->content, 0);
+		} else if (!strcmp (name, "ogo:value")) {
+			state->state = PARSE_OPTION_VALUE;
+			g_string_truncate (state->content, 0);
+		} else {
+			state->prev_state = state->state;
+			state->state = PARSE_UNKNOWN;
+			state->unknown_depth++;
+		}
+
+		break;
 	case PARSE_PARTS:
-                if (!strcmp (name, "ogo:part")) {
+		if (!strcmp (name, "ogo:part")) {
 			state->state = PARSE_PART;
 			state->part = g_new0 (LibraryPart, 1);
 			state->rotation = 0;
@@ -443,36 +464,36 @@ start_element (ParseState *state, const CHAR *name, const CHAR **attrs)
 		}
 		break;
 	case PARSE_PART:
-                if (!strcmp (name, "ogo:name")) {
+		if (!strcmp (name, "ogo:name")) {
 			state->state = PARSE_PART_NAME;
 			g_string_truncate (state->content, 0);
-                } else if (!strcmp (name, "ogo:library")) {
+		} else if (!strcmp (name, "ogo:library")) {
 			state->state = PARSE_PART_LIBNAME;
 			g_string_truncate (state->content, 0);
-                } else if (!strcmp (name, "ogo:refdes")) {
+		} else if (!strcmp (name, "ogo:refdes")) {
 			state->state = PARSE_PART_REFDES;
 			g_string_truncate (state->content, 0);
-                } else if (!strcmp (name, "ogo:position")) {
+		} else if (!strcmp (name, "ogo:position")) {
 			state->state = PARSE_PART_POSITION;
 			g_string_truncate (state->content, 0);
-                } else if (!strcmp (name, "ogo:rotation")) {
+		} else if (!strcmp (name, "ogo:rotation")) {
 			state->state = PARSE_PART_ROTATION;
 			g_string_truncate (state->content, 0);
-                } else if (!strcmp (name, "ogo:flip")) {
+		} else if (!strcmp (name, "ogo:flip")) {
 			state->state = PARSE_PART_FLIP;
 			g_string_truncate (state->content, 0);
-                } else if (!strcmp (name, "ogo:template")) {
+		} else if (!strcmp (name, "ogo:template")) {
 			state->state = PARSE_PART_TEMPLATE;
 			g_string_truncate (state->content, 0);
-                } else if (!strcmp (name, "ogo:model")) {
+		} else if (!strcmp (name, "ogo:model")) {
 			state->state = PARSE_PART_MODEL;
 			g_string_truncate (state->content, 0);
-                } else if (!strcmp (name, "ogo:symbol")) {
+		} else if (!strcmp (name, "ogo:symbol")) {
 			state->state = PARSE_PART_SYMNAME;
 			g_string_truncate (state->content, 0);
-                } else if (!strcmp (name, "ogo:labels")) {
+		} else if (!strcmp (name, "ogo:labels")) {
 			state->state = PARSE_PART_LABELS;
-                } else if (!strcmp (name, "ogo:properties")) {
+		} else if (!strcmp (name, "ogo:properties")) {
 			state->state = PARSE_PART_PROPERTIES;
 		} else {
 			state->prev_state = state->state;
@@ -481,7 +502,7 @@ start_element (ParseState *state, const CHAR *name, const CHAR **attrs)
 		}
 		break;
 	case PARSE_PART_LABELS:
-                if (!strcmp (name, "ogo:label")) {
+		if (!strcmp (name, "ogo:label")) {
 			state->state = PARSE_PART_LABEL;
 			state->label = g_new0 (PartLabel, 1);
 		} else {
@@ -491,14 +512,14 @@ start_element (ParseState *state, const CHAR *name, const CHAR **attrs)
 		}
 		break;
 	case PARSE_PART_LABEL:
-                if (!strcmp (name, "ogo:name")) {
+		if (!strcmp (name, "ogo:name")) {
 			state->state = PARSE_PART_LABEL_NAME;
 			g_string_truncate (state->content, 0);
-                } else if (!strcmp (name, "ogo:text")) {
-		        state->state = PARSE_PART_LABEL_TEXT;
+		} else if (!strcmp (name, "ogo:text")) {
+			state->state = PARSE_PART_LABEL_TEXT;
 			g_string_truncate (state->content, 0);
-                } else if (!strcmp (name, "ogo:position")) {
-		        state->state = PARSE_PART_LABEL_POS;
+		} else if (!strcmp (name, "ogo:position")) {
+			state->state = PARSE_PART_LABEL_POS;
 			g_string_truncate (state->content, 0);
 		} else {
 			state->prev_state = state->state;
@@ -506,9 +527,9 @@ start_element (ParseState *state, const CHAR *name, const CHAR **attrs)
 			state->unknown_depth++;
 		}
 		break;
-		
+
 	case PARSE_PART_PROPERTIES:
-                if (!strcmp (name, "ogo:property")) {
+		if (!strcmp (name, "ogo:property")) {
 			state->state = PARSE_PART_PROPERTY;
 			state->property = g_new0 (Property, 1);
 		} else {
@@ -518,11 +539,11 @@ start_element (ParseState *state, const CHAR *name, const CHAR **attrs)
 		}
 		break;
 	case PARSE_PART_PROPERTY:
-                if (!strcmp (name, "ogo:name")) {
+		if (!strcmp (name, "ogo:name")) {
 			state->state = PARSE_PART_PROPERTY_NAME;
 			g_string_truncate (state->content, 0);
-                } else if (!strcmp (name, "ogo:value")) {
-		        state->state = PARSE_PART_PROPERTY_VALUE;
+		} else if (!strcmp (name, "ogo:value")) {
+			state->state = PARSE_PART_PROPERTY_VALUE;
 			g_string_truncate (state->content, 0);
 		} else {
 			state->prev_state = state->state;
@@ -533,7 +554,7 @@ start_element (ParseState *state, const CHAR *name, const CHAR **attrs)
 
 
 	case PARSE_WIRES:
-                if (!strcmp (name, "ogo:wire")) {
+		if (!strcmp (name, "ogo:wire")) {
 			state->state = PARSE_WIRE;
 		} else {
 			state->prev_state = state->state;
@@ -542,7 +563,7 @@ start_element (ParseState *state, const CHAR *name, const CHAR **attrs)
 		}
 		break;
 	case PARSE_WIRE:
-                if (!strcmp (name, "ogo:points")) {
+		if (!strcmp (name, "ogo:points")) {
 			state->state = PARSE_WIRE_POINTS;
 			g_string_truncate (state->content, 0);
 		} else {
@@ -553,7 +574,7 @@ start_element (ParseState *state, const CHAR *name, const CHAR **attrs)
 		break;
 
 	case PARSE_TEXTBOXES:
-                if (!strcmp (name, "ogo:textbox")) {
+		if (!strcmp (name, "ogo:textbox")) {
 			state->state = PARSE_TEXTBOX;
 		} else {
 			state->prev_state = state->state;
@@ -562,10 +583,10 @@ start_element (ParseState *state, const CHAR *name, const CHAR **attrs)
 		}
 		break;
 	case PARSE_TEXTBOX:
-                if (!strcmp (name, "ogo:position")) {
+		if (!strcmp (name, "ogo:position")) {
 			state->state = PARSE_TEXTBOX_POSITION;
 			g_string_truncate (state->content, 0);
-                } else if (!strcmp (name, "ogo:text")) {
+		} else if (!strcmp (name, "ogo:text")) {
 			state->state = PARSE_TEXTBOX_TEXT;
 			g_string_truncate (state->content, 0);
 		} else {
@@ -581,24 +602,24 @@ start_element (ParseState *state, const CHAR *name, const CHAR **attrs)
 	case PARSE_TRANSIENT_STEP:
 	case PARSE_TRANSIENT_STEP_ENABLE:
 
-        case PARSE_AC_ENABLED:
-        case PARSE_AC_NPOINTS:
-        case PARSE_AC_START:
-        case PARSE_AC_STOP:
+	case PARSE_AC_ENABLED:
+	case PARSE_AC_NPOINTS:
+	case PARSE_AC_START:
+	case PARSE_AC_STOP:
+
+	case PARSE_DC_ENABLED:
+	case PARSE_DC_VSRC1:
+	case PARSE_DC_START1:
+	case PARSE_DC_STOP1:
+	case PARSE_DC_STEP1:
+	case PARSE_DC_VSRC2:
+	case PARSE_DC_START2:
+	case PARSE_DC_STOP2:
+	case PARSE_DC_STEP2:
 
-        case PARSE_DC_ENABLED:
-        case PARSE_DC_VSRC1:
-        case PARSE_DC_START1:
-        case PARSE_DC_STOP1:
-        case PARSE_DC_STEP1:
-        case PARSE_DC_VSRC2:
-        case PARSE_DC_START2:
-        case PARSE_DC_STOP2:
-        case PARSE_DC_STEP2:
-   
 	case PARSE_WIRE_POINTS:
-        case PARSE_OPTION_NAME:
-        case PARSE_OPTION_VALUE:
+	case PARSE_OPTION_NAME:
+	case PARSE_OPTION_VALUE:
 	case PARSE_PART_NAME:
 	case PARSE_PART_LIBNAME:
 	case PARSE_PART_TEMPLATE:
@@ -618,8 +639,9 @@ start_element (ParseState *state, const CHAR *name, const CHAR **attrs)
 	case PARSE_ZOOM:
 	case PARSE_TITLE:
 	case PARSE_AUTHOR:
+	case PARSE_COMMENTS:
 		/* there should be no tags inside these types of tags */
-                g_message ("*** '%s' tag found", name);
+		g_message ("*** '%s' tag found", name);
 		state->prev_state = state->state;
 		state->state = PARSE_UNKNOWN;
 		state->unknown_depth++;
@@ -632,7 +654,9 @@ start_element (ParseState *state, const CHAR *name, const CHAR **attrs)
 		break;
 	case PARSE_FINISH:
 		/* should not start new elements in this state */
-                g_assert_not_reached ();
+		g_assert_not_reached ();
+		break;
+	case PARSE_TRANSIENT_INIT_COND:
 		break;
 	}
 	/*g_message("Start element %s (state %s)", name, states[state->state]);*/
@@ -643,10 +667,10 @@ start_element (ParseState *state, const CHAR *name, const CHAR **attrs)
  *
  */
 static void
-end_element (ParseState *state, const CHAR *name)
+end_element (ParseState *state, const xmlChar *name)
 {
 	GList *libs;
-	
+
 	switch (state->state) {
 	case PARSE_UNKNOWN:
 		state->unknown_depth--;
@@ -661,7 +685,10 @@ end_element (ParseState *state, const CHAR *name)
 		state->title = g_strdup (state->content->str);
 		state->state = PARSE_SCHEMATIC;
 		break;
-
+	case PARSE_COMMENTS:
+		state->comments = g_strdup (state->content->str);
+		state->state = PARSE_SCHEMATIC;
+		break;
 	case PARSE_ZOOM:
 	{
 		double zoom;
@@ -680,11 +707,13 @@ end_element (ParseState *state, const CHAR *name)
 		state->state = PARSE_SIMULATION_SETTINGS;
 		break;
 	case PARSE_TRANSIENT_ENABLED:
-		sim_settings_set_trans (state->sim_settings, !g_strcasecmp (state->content->str, "true"));
+		sim_settings_set_trans (state->sim_settings,
+								!g_strcasecmp (state->content->str, "true"));
 		state->state = PARSE_TRANSIENT_SETTINGS;
 		break;
 	case PARSE_TRANSIENT_START:
-		sim_settings_set_trans_start (state->sim_settings, state->content->str);
+		sim_settings_set_trans_start (state->sim_settings,
+									  state->content->str);
 		state->state = PARSE_TRANSIENT_SETTINGS;
 		break;
 	case PARSE_TRANSIENT_STOP:
@@ -696,68 +725,76 @@ end_element (ParseState *state, const CHAR *name)
 		state->state = PARSE_TRANSIENT_SETTINGS;
 		break;
 	case PARSE_TRANSIENT_STEP_ENABLE:
-		sim_settings_set_trans_step_enable (state->sim_settings, !g_strcasecmp (state->content->str, "true"));
+		sim_settings_set_trans_step_enable (state->sim_settings,
+											!g_strcasecmp (state->content->str, "true"));
+		state->state = PARSE_TRANSIENT_SETTINGS;
+		break;
+	case PARSE_TRANSIENT_INIT_COND:
+		sim_settings_set_trans_init_cond(state->sim_settings,
+										 !g_strcasecmp (state->content->str, "true"));
 		state->state = PARSE_TRANSIENT_SETTINGS;
 		break;
 
-        case PARSE_AC_SETTINGS:
-                state->state = PARSE_SIMULATION_SETTINGS;
-                break;
-        case PARSE_AC_ENABLED:
-                sim_settings_set_ac (state->sim_settings, !g_strcasecmp (state->content->str, "true"));
-                state->state = PARSE_AC_SETTINGS;
-        case PARSE_AC_NPOINTS:
-                sim_settings_set_ac_npoints (state->sim_settings, state->content->str);
-                state->state = PARSE_AC_SETTINGS;
-        case PARSE_AC_START:
-                sim_settings_set_ac_start (state->sim_settings, state->content->str);
-                state->state = PARSE_AC_SETTINGS;
-                break;
-        case PARSE_AC_STOP:
-                sim_settings_set_ac_stop (state->sim_settings, state->content->str);
-                state->state = PARSE_AC_SETTINGS;
-                break;
+	case PARSE_AC_SETTINGS:
+		state->state = PARSE_SIMULATION_SETTINGS;
+		break;
+	case PARSE_AC_ENABLED:
+		sim_settings_set_ac (state->sim_settings,
+							 !g_strcasecmp (state->content->str, "true"));
+		state->state = PARSE_AC_SETTINGS;
+	case PARSE_AC_NPOINTS:
+		sim_settings_set_ac_npoints (state->sim_settings, state->content->str);
+		state->state = PARSE_AC_SETTINGS;
+	case PARSE_AC_START:
+		sim_settings_set_ac_start (state->sim_settings, state->content->str);
+		state->state = PARSE_AC_SETTINGS;
+		break;
+	case PARSE_AC_STOP:
+		sim_settings_set_ac_stop (state->sim_settings, state->content->str);
+		state->state = PARSE_AC_SETTINGS;
+		break;
 
 	case PARSE_DC_SETTINGS:
 		state->state = PARSE_SIMULATION_SETTINGS;
 		break;
 
 	case PARSE_DC_ENABLED:
-		sim_settings_set_dc (state->sim_settings, !g_strcasecmp (state->content->str, "true"));
-                state->state = PARSE_DC_SETTINGS;
-        case PARSE_DC_VSRC1:
-                sim_settings_set_dc_vsrc (state->sim_settings, 0, state->content->str);
-                state->state = PARSE_DC_SETTINGS;
-        case PARSE_DC_START1:
-                sim_settings_set_dc_start (state->sim_settings, 0, state->content->str);
-                state->state = PARSE_DC_SETTINGS;
-                break;
-        case PARSE_DC_STOP1:
-                sim_settings_set_dc_stop (state->sim_settings, 0, state->content->str);
-                state->state = PARSE_DC_SETTINGS;
-                break;
-        case PARSE_DC_STEP1:
-                sim_settings_set_dc_step (state->sim_settings, 0, state->content->str);
-                state->state = PARSE_DC_SETTINGS;
-                break;
-        case PARSE_DC_VSRC2:
-                sim_settings_set_dc_vsrc (state->sim_settings, 1, state->content->str);
-                state->state = PARSE_DC_SETTINGS;
-        case PARSE_DC_START2:
-                sim_settings_set_dc_start (state->sim_settings, 1, state->content->str);
-                state->state = PARSE_DC_SETTINGS;
-                break;
-        case PARSE_DC_STOP2:
-                sim_settings_set_dc_stop (state->sim_settings, 1, state->content->str);
-                state->state = PARSE_DC_SETTINGS;
-                break;
-        case PARSE_DC_STEP2:
-                sim_settings_set_dc_step (state->sim_settings, 1, state->content->str);
-                state->state = PARSE_DC_SETTINGS;
-                break;
+		sim_settings_set_dc (state->sim_settings,
+							 !g_strcasecmp (state->content->str, "true"));
+		state->state = PARSE_DC_SETTINGS;
+	case PARSE_DC_VSRC1:
+		sim_settings_set_dc_vsrc (state->sim_settings, 0, state->content->str);
+		state->state = PARSE_DC_SETTINGS;
+	case PARSE_DC_START1:
+		sim_settings_set_dc_start (state->sim_settings, 0, state->content->str);
+		state->state = PARSE_DC_SETTINGS;
+		break;
+	case PARSE_DC_STOP1:
+		sim_settings_set_dc_stop (state->sim_settings, 0, state->content->str);
+		state->state = PARSE_DC_SETTINGS;
+		break;
+	case PARSE_DC_STEP1:
+		sim_settings_set_dc_step (state->sim_settings, 0, state->content->str);
+		state->state = PARSE_DC_SETTINGS;
+		break;
+	case PARSE_DC_VSRC2:
+		sim_settings_set_dc_vsrc (state->sim_settings, 1, state->content->str);
+		state->state = PARSE_DC_SETTINGS;
+	case PARSE_DC_START2:
+		sim_settings_set_dc_start (state->sim_settings, 1, state->content->str);
+		state->state = PARSE_DC_SETTINGS;
+		break;
+	case PARSE_DC_STOP2:
+		sim_settings_set_dc_stop (state->sim_settings, 1, state->content->str);
+		state->state = PARSE_DC_SETTINGS;
+		break;
+	case PARSE_DC_STEP2:
+		sim_settings_set_dc_step (state->sim_settings, 1, state->content->str);
+		state->state = PARSE_DC_SETTINGS;
+		break;
 
 
-        case PARSE_OPTION_LIST:
+	case PARSE_OPTION_LIST:
 		state->state = PARSE_SIMULATION_SETTINGS;
 		break;
 	case PARSE_OPTION:
@@ -789,21 +826,21 @@ end_element (ParseState *state, const CHAR *name)
 	case PARSE_PART_LIBNAME:
 
 		/* FIXME: if the library name has changed, so it will
-		not be found, oregano will probably crash (this problem 
+		not be found, oregano will probably crash (this problem
 		also happen storing the library name in the part).
 		Why not search for the part name in all libraries ? */
-		
+
 		state->state = PARSE_PART;
 		state->part->library = NULL;
 		libs = oregano.libraries;
 		while (libs) {
 			Library *lib = (Library *) libs->data;
-                        if (g_strcasecmp (state->content->str, lib->name) == 0) {
+			if (g_strcasecmp (state->content->str, lib->name) == 0) {
 				state->part->library = lib;
 				break;
 			}
 			libs = libs->next;
-		}		
+		}
 		break;
 	case PARSE_PART_POSITION:
 		sscanf (state->content->str, "(%lf %lf)", &state->pos.x, &state->pos.y);
@@ -814,9 +851,9 @@ end_element (ParseState *state, const CHAR *name)
 		state->state = PARSE_PART;
 		break;
 	case PARSE_PART_FLIP:
-                if (g_strcasecmp ( state->content->str, "horizontal") == 0)
+		if (g_strcasecmp ( state->content->str, "horizontal") == 0)
 			state->flip = state->flip | ID_FLIP_HORIZ;
-                else if (g_strcasecmp ( state->content->str, "vertical") == 0)
+		else if (g_strcasecmp ( state->content->str, "vertical") == 0)
 			state->flip = state->flip | ID_FLIP_VERT;
 		state->state = PARSE_PART;
 		break;
@@ -841,7 +878,8 @@ end_element (ParseState *state, const CHAR *name)
 		break;
 	case PARSE_PART_LABEL:
 		state->state = PARSE_PART_LABELS;
-		state->part->labels = g_slist_prepend (state->part->labels, state->label); 
+		state->part->labels = g_slist_prepend (state->part->labels,
+											   state->label);
 		break;
 	case PARSE_PART_LABEL_NAME:
 		state->label->name = g_strdup (state->content->str);
@@ -852,7 +890,8 @@ end_element (ParseState *state, const CHAR *name)
 		state->state = PARSE_PART_LABEL;
 		break;
 	case PARSE_PART_LABEL_POS:
-		sscanf (state->content->str, "(%lf %lf)", &state->label->pos.x, &state->label->pos.y);
+		sscanf (state->content->str, "(%lf %lf)", &state->label->pos.x,
+				&state->label->pos.y);
 		state->state = PARSE_PART_LABEL;
 		break;
 	case PARSE_PART_PROPERTIES:
@@ -860,7 +899,8 @@ end_element (ParseState *state, const CHAR *name)
 		break;
 	case PARSE_PART_PROPERTY:
 		state->state = PARSE_PART_PROPERTIES;
-		state->part->properties = g_slist_prepend (state->part->properties, state->property);
+		state->part->properties = g_slist_prepend (state->part->properties,
+												   state->property);
 		break;
 	case PARSE_PART_PROPERTY_NAME:
 		state->property->name = g_strdup (state->content->str);
@@ -879,7 +919,7 @@ end_element (ParseState *state, const CHAR *name)
 		state->state = PARSE_WIRES;
 		break;
 	case PARSE_WIRE_POINTS:
-		sscanf (state->content->str, "(%lf %lf)(%lf %lf)", 
+		sscanf (state->content->str, "(%lf %lf)(%lf %lf)",
 			&state->wire_start.x, &state->wire_start.y,
 			&state->wire_end.x, &state->wire_end.y);
 		state->state = PARSE_WIRE;
@@ -893,7 +933,7 @@ end_element (ParseState *state, const CHAR *name)
 		state->state = PARSE_TEXTBOXES;
 		break;
 	case PARSE_TEXTBOX_POSITION:
-		sscanf (state->content->str, "(%lf %lf)", 
+		sscanf (state->content->str, "(%lf %lf)",
 			&state->pos.x, &state->pos.y);
 		state->state = PARSE_TEXTBOX;
 		break;
@@ -913,58 +953,58 @@ end_element (ParseState *state, const CHAR *name)
 	case PARSE_START:
 	case PARSE_FINISH:
 		/* There should not be a closing tag in this state. */
-                g_assert_not_reached ();
+		g_assert_not_reached ();
 		break;
 	}
 	/*g_message("End element %s (state %s)", name, states[state->state]);*/
 }
 
-static void 
-characters (ParseState *state, const CHAR *chars, int len)
+static void
+my_characters (ParseState *state, const xmlChar *chars, int len)
 {
 	int i;
-	
+
 	if (state->state == PARSE_FINISH ||
-	    state->state == PARSE_START ||
-	    state->state == PARSE_PARTS ||
-	    state->state == PARSE_PART)
+		state->state == PARSE_START ||
+		state->state == PARSE_PARTS ||
+		state->state == PARSE_PART)
 		return;
 
 	for (i = 0; i < len; i++)
 		g_string_append_c (state->content, chars[i]);
 }
 
-static xmlEntityPtr 
-get_entity (void *user_data, const CHAR *name)
+static xmlEntityPtr
+get_entity (void *user_data, const xmlChar *name)
 {
 	return xmlGetPredefinedEntity (name);
 }
 
-static void 
+static void
 my_warning (void *user_data, const char *msg, ...)
 {
 	va_list args;
-	
+
 	va_start (args, msg);
 	g_logv ("XML", G_LOG_LEVEL_WARNING, msg, args);
 	va_end (args);
 }
 
-static void 
+static void
 my_error (void *user_data, const char *msg, ...)
 {
 	va_list args;
-	
+
 	va_start (args, msg);
-        g_logv ("XML", G_LOG_LEVEL_CRITICAL, msg, args);
+	g_logv ("XML", G_LOG_LEVEL_CRITICAL, msg, args);
 	va_end (args);
 }
 
-static void 
+static void
 my_fatal_error (void *user_data, const char *msg, ...)
 {
 	va_list args;
-	
+
 	va_start (args, msg);
 	g_logv ("XML", G_LOG_LEVEL_ERROR, msg, args);
 	va_end (args);
