@@ -2,20 +2,23 @@
  * node-store.c
  *
  * This is where the circuit elements (pins/wires) are stored.
- * 
- * Author: 
+ *
+ * Authors:
  *  Richard Hult <rhult@hem.passagen.se>
- * 
- *  http://www.dtek.chalmers.se/~d4hult/oregano/ 
- * 
- * Copyright (C) 1999,2000  Richard Hult 
- * 
- * This program is free software; you can redistribute it and/or 
- * modify it under the terms of the GNU General Public License as 
- * published by the Free Software Foundation; either version 2 of the 
- * License, or (at your option) any later version. 
- * 
- * This program is distributed in the hope that it will be useful, 
+ *  Ricardo Markiewicz <rmarkie@fi.uba.ar>
+ *  Andres de Barbara <adebarbara@fi.uba.ar>
+ *
+ * Web page: http://arrakis.lug.fi.uba.ar/
+ *
+ * Copyright (C) 1999-2001  Richard Hult
+ * Copyright (C) 2003,2004  LUGFI
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
@@ -28,7 +31,8 @@
 
 #include <config.h>
 #include <math.h>
-#include <gtk/gtk.h>
+#include <glib.h>
+#include <glib-object.h>
 #include "node-store.h"
 #include "node.h"
 #include "part.h"
@@ -48,22 +52,24 @@
 #define SEP(p1x,p1y,p2x,p2y) (IS_EQ(p1x, p2x) && IS_EQ(p1y, p2y))
 
 /* Equals? */
-#define IS_EQ(a,b) (fabs (a - b) < NODE_EPSILON)
+#define IS_EQ(a,b) (fabs ((a) - (b)) < NODE_EPSILON)
 
 #define ON_THE_WIRE(p1,start,end) ( fabs((end.y-start.y)*(p1.x-start.x)-(end.x-start.x)*(p1.y-start.y))<NODE_EPSILON )
 
-static void       node_store_class_init (NodeStoreClass *klass);
-static void       node_store_init (NodeStore *store);
-static Node      *node_store_get_or_create_node (NodeStore *store, SheetPos pos);
-static guint      node_hash (gconstpointer key);
-static int        node_equal (gconstpointer a, gconstpointer b);
-static GSList    *wires_intersect (NodeStore *store, double x1, double y1, double x2, double y2);
-static GSList    *wire_intersect_parts (NodeStore *store, Wire *wire);
-static int        is_wire_at_pos (double x1, double y1, double x2, double y2, SheetPos pos);
-static GSList    *wires_at_pos (NodeStore *store, SheetPos pos);
-static int        do_wires_intersect (double Ax, double Ay, double Bx, double By, 
-				      double Cx, double Cy, double Dx, double Dy, 
-				      SheetPos *pos);
+static void	   node_store_class_init (NodeStoreClass *klass);
+static void	   node_store_init (NodeStore *store);
+static guint   node_hash (gconstpointer key);
+static int	   node_equal (gconstpointer a, gconstpointer b);
+static GSList *wires_intersect (NodeStore *store, double x1, double y1,
+	double x2, double y2);
+static GSList *wire_intersect_parts (NodeStore *store, Wire *wire);
+static int     is_wire_at_pos (double x1, double y1, double x2, double y2,
+	SheetPos pos);
+static GSList *wires_at_pos (NodeStore *store, SheetPos pos);
+static int	   do_wires_intersect (double Ax, double Ay, double Bx, double By,
+	double Cx, double Cy, double Dx, double Dy, SheetPos *pos);
+static void	   node_store_finalize(GObject *self);
+static void	   node_store_dispose(GObject *self);
 
 typedef struct {
 	Wire *wire;
@@ -77,93 +83,114 @@ enum {
 };
 
 static guint node_store_signals [LAST_SIGNAL] = { 0 };
-static GtkObject *parent_class = NULL;
+static GObjectClass *parent_class = NULL;
 
-guint
+GType
 node_store_get_type (void)
 {
 	static guint node_store_type = 0;
 
 	if (!node_store_type) {
-		static const GtkTypeInfo node_store_info = {
-			"NodeStore",
-			sizeof (NodeStore),
+		static const GTypeInfo node_store_info = {
 			sizeof (NodeStoreClass),
-			(GtkClassInitFunc) node_store_class_init,
-			(GtkObjectInitFunc) node_store_init,
-			/* reserved_1 */ NULL,
-			/* reserved_2 */ NULL,
-			(GtkClassInitFunc) NULL,
+			NULL,
+			NULL,
+			(GClassInitFunc) node_store_class_init,
+			NULL,
+			NULL,
+			sizeof (NodeStore),
+			0,
+			(GInstanceInitFunc) node_store_init,
+			NULL
 		};
-		
-		node_store_type = gtk_type_unique (GTK_TYPE_OBJECT, &node_store_info);
+
+		node_store_type = g_type_register_static (G_TYPE_OBJECT, "NodeStore",
+			&node_store_info, 0);
 	}
 
 	return node_store_type;
 }
 
 static void
-node_store_destroy (GtkObject *object)
+node_store_finalize(GObject *object)
 {
-	NodeStore *store;
+	NodeStore *self = NODE_STORE(object);
 
-	store = NODE_STORE (object);
-       
-	g_hash_table_destroy (store->nodes);
+	if (self->nodes) {
+		g_hash_table_destroy (self->nodes);
+		self->nodes = NULL;
+	}
 
-	g_list_free (store->wires);
-	g_list_free (store->parts);
-	g_list_free (store->items);
+	if (self->wires) {
+		g_list_free (self->wires);
+		self->wires = NULL;
+	}
+	if (self->parts) {
+		g_list_free (self->parts);
+		self->parts = NULL;
+	}
+	if (self->items) {
+		g_list_free (self->items);
+		self->items = NULL;
+	}
 
-	GTK_OBJECT_CLASS (parent_class)->destroy (object);
+	parent_class->finalize(object);
+}
+
+static void
+node_store_dispose(GObject *self)
+{
+	parent_class->dispose(self);
 }
 
 static void
 node_store_class_init (NodeStoreClass *klass)
 {
-	GtkObjectClass *object_class;
+	GObjectClass *gobject_class;
 
-	object_class = (GtkObjectClass *)klass;
-	parent_class = gtk_type_class (GTK_TYPE_OBJECT);
+	gobject_class = G_OBJECT_CLASS(klass);
+	parent_class = g_type_class_peek_parent(klass);
 
-	object_class->destroy = node_store_destroy;
+	gobject_class->finalize = node_store_finalize;
+	gobject_class->dispose = node_store_dispose;
 
-	node_store_signals [DOT_ADDED] = 
-		gtk_signal_new ("dot_added",
-				GTK_RUN_FIRST,
-				object_class->type,
-				0,
-				gtk_marshal_NONE__POINTER,
-				GTK_TYPE_NONE, 1, GTK_TYPE_POINTER);
+	node_store_signals [DOT_ADDED] =
+		g_signal_new ("dot_added",
+		  G_TYPE_FROM_CLASS(gobject_class),
+		  G_SIGNAL_RUN_FIRST,
+		  G_STRUCT_OFFSET (NodeStoreClass, dot_added),
+		  NULL,
+		  NULL,
+		  g_cclosure_marshal_VOID__POINTER,
+		  G_TYPE_NONE,
+		  1, G_TYPE_POINTER);
 
-	node_store_signals [DOT_REMOVED] = 
-		gtk_signal_new ("dot_removed",
-				GTK_RUN_FIRST,
-				object_class->type,
-				0,
-				gtk_marshal_NONE__POINTER,
-				GTK_TYPE_NONE, 1, GTK_TYPE_POINTER);
-
-	gtk_object_class_add_signals (object_class, node_store_signals, LAST_SIGNAL);
+	node_store_signals [DOT_REMOVED] =
+		g_signal_new ("dot_removed",
+		  TYPE_NODE_STORE,
+		  G_SIGNAL_RUN_FIRST,
+		  G_STRUCT_OFFSET (NodeStoreClass, dot_removed),
+		  NULL,
+		  NULL,
+		  g_cclosure_marshal_VOID__POINTER,
+		  G_TYPE_NONE,
+		  1, G_TYPE_POINTER);
 }
 
 static void
-node_store_init (NodeStore *store)
+node_store_init (NodeStore *self)
 {
-	store->nodes = g_hash_table_new (node_hash, node_equal);
-	store->wires = NULL;
-	store->parts = NULL;
-	store->items = NULL;
+	self->nodes = g_hash_table_new (node_hash, node_equal);
+	self->wires = NULL;
+	self->parts = NULL;
+	self->items = NULL;
+	self->textbox = NULL;
 }
 
 NodeStore *
 node_store_new (void)
 {
-	NodeStore *store;
-
-	store = gtk_type_new (node_store_get_type ());
-
-	return store;
+	return NODE_STORE(g_object_new (TYPE_NODE_STORE, NULL));
 }
 
 static void
@@ -172,7 +199,7 @@ dot_added_callback (Node *node, SheetPos *pos, NodeStore *store)
 	g_return_if_fail (store != NULL);
 	g_return_if_fail (IS_NODE_STORE (store));
 
-	gtk_signal_emit (GTK_OBJECT (store), node_store_signals[DOT_ADDED], pos);
+	g_signal_emit_by_name(G_OBJECT(store), "dot_added", pos);
 }
 
 static void
@@ -181,18 +208,18 @@ dot_removed_callback (Node *node, SheetPos *pos, NodeStore *store)
 	g_return_if_fail (store != NULL);
 	g_return_if_fail (IS_NODE_STORE (store));
 
-	gtk_signal_emit (GTK_OBJECT (store), node_store_signals[DOT_REMOVED], pos);
+	g_signal_emit_by_name(G_OBJECT(store), "dot_removed", pos);
 }
 
-static Node *
-node_store_get_or_create_node (NodeStore *store, SheetPos pos)
+Node *
+node_store_get_or_create_node (NodeStore *self, SheetPos pos)
 {
 	Node *node;
 
-	g_return_val_if_fail (store != NULL, NULL);
-	g_return_val_if_fail (IS_NODE_STORE (store), NULL);
-	
-	node = g_hash_table_lookup (store->nodes, &pos);
+	g_return_val_if_fail (self != NULL, NULL);
+	g_return_val_if_fail (IS_NODE_STORE (self), NULL);
+
+	node = g_hash_table_lookup (self->nodes, &pos);
 
 	if (!node) {
 		/*
@@ -200,24 +227,24 @@ node_store_get_or_create_node (NodeStore *store, SheetPos pos)
 		 */
 		node = node_new (pos);
 
-		gtk_signal_connect_while_alive (
-			GTK_OBJECT (node),
+		g_signal_connect_object(
+			G_OBJECT (node),
 			"dot_added",
-			dot_added_callback,
-			store, 
-			GTK_OBJECT (store));
+			G_CALLBACK (dot_added_callback),
+			G_OBJECT (self),
+			0);
 
-		gtk_signal_connect_while_alive (
-			GTK_OBJECT (node),
+		g_signal_connect_object (
+			G_OBJECT (node),
 			"dot_removed",
-			dot_removed_callback,
-			store,
-			GTK_OBJECT (store));
-		
-		g_hash_table_insert (store->nodes, &node->key, node);
+			G_CALLBACK (dot_removed_callback),
+			G_OBJECT (self),
+			0);
+
+		g_hash_table_insert (self->nodes, &node->key, node);
 	}
 
-	/* 
+	/*
 	 * If there was a previously stored node here, just
 	 * return that node.
 	 */
@@ -225,7 +252,7 @@ node_store_get_or_create_node (NodeStore *store, SheetPos pos)
 }
 
 int
-node_store_add_part (NodeStore *store, Part *part)
+node_store_add_part (NodeStore *self, Part *part)
 {
 	GSList *wire_list, *list;
 	Node *node;
@@ -234,8 +261,8 @@ node_store_add_part (NodeStore *store, Part *part)
 	gdouble x, y;
 	int i, num_pins;
 
-	g_return_val_if_fail (store != NULL, FALSE);
-	g_return_val_if_fail (IS_NODE_STORE (store), FALSE);
+	g_return_val_if_fail (self != NULL, FALSE);
+	g_return_val_if_fail (IS_NODE_STORE (self), FALSE);
 	g_return_val_if_fail (part != NULL, FALSE);
 	g_return_val_if_fail (IS_PART (part), FALSE);
 
@@ -245,101 +272,99 @@ node_store_add_part (NodeStore *store, Part *part)
 
 	for (i = 0; i < num_pins; i++) {
 		Pin *pins;
-		
+
 		pins = part_get_pins (part);
 		x = part_pos.x + pins[i].offset.x;
 		y = part_pos.y + pins[i].offset.y;
 
 		/*
-		 * Use the position of the pin as hash key. 
+		 * Use the position of the pin as hash key.
 		 */
 		lookup_key.x = x;
 		lookup_key.y = y;
 
-		/* 
+		/*
 		 * Retrieve a node for this position.
 		 */
-		node = node_store_get_or_create_node (store, lookup_key);
+		node = node_store_get_or_create_node (self, lookup_key);
 
 		/*
 		 * Add all the wires that intersect this pin to the node store.
 		 */
-		wire_list = wires_at_pos (store, lookup_key);
+		wire_list = wires_at_pos (self, lookup_key);
 
 		for (list = wire_list; list; list = list->next) {
 			Wire *wire = list->data;
 
-/*			g_print ("Add pin to wire.\n");*/
-			
+		/* g_print ("Add pin to wire.\n"); */
+
 			node_add_wire (node, wire);
 			wire_add_node (wire, node);
 		}
-		
+
 		g_slist_free (wire_list);
 
 		node_add_pin (node, &pins[i]);
 	}
 
-	gtk_object_set (GTK_OBJECT (part), "store", store, NULL);
-	store->parts = g_list_prepend (store->parts, part);		
-	store->items = g_list_prepend (store->items, part);		
+	g_object_set (G_OBJECT (part), "store", self, NULL);
+	self->parts = g_list_prepend (self->parts, part);
+	self->items = g_list_prepend (self->items, part);
 
 	return TRUE;
 }
 
 int
-node_store_remove_part (NodeStore *store, Part *part)
+node_store_remove_part (NodeStore *self, Part *part)
 {
 	Node *node;
 	SheetPos lookup_key;
 	SheetPos pos;
 	gdouble x, y;
 	int i, num_pins;
+	Pin *pins;
 
-	g_return_val_if_fail (store != NULL, FALSE);
-	g_return_val_if_fail (IS_NODE_STORE (store), FALSE);
+	g_return_val_if_fail (self != NULL, FALSE);
+	g_return_val_if_fail (IS_NODE_STORE (self), FALSE);
 	g_return_val_if_fail (part != NULL, FALSE);
 	g_return_val_if_fail (IS_PART (part), FALSE);
 
-	store->parts = g_list_remove (store->parts, part);		
-	store->items = g_list_remove (store->items, part);		
+	self->parts = g_list_remove (self->parts, part);
+	self->items = g_list_remove (self->items, part);
 
 	num_pins = part_get_num_pins (part);
 	item_data_get_pos (ITEM_DATA (part), &pos);
 
+	pins = part_get_pins (part);
 	for (i = 0; i < num_pins; i++) {
-		Pin *pins;
-
-		pins = part_get_pins (part);
-
 		x = pos.x + pins[i].offset.x;
 		y = pos.y + pins[i].offset.y;
 
 		/*
-		 * Use the position of the pin as lookup key. 
+		 * Use the position of the pin as lookup key.
 		 */
 		lookup_key.x = x;
 		lookup_key.y = y;
-		
-		node = g_hash_table_lookup (store->nodes, &lookup_key);
+
+		node = g_hash_table_lookup (self->nodes, &lookup_key);
 		if (node) {
 			if (!node_remove_pin (node, &pins[i])) {
 				g_warning ("Couldn't remove pin.");
 				return FALSE;
 			}
 
-			/* 
-			 * If the node is empty after removing the pin, 
+			/*
+			 * If the node is empty after removing the pin,
 			 * remove the node as well.
 			 */
 			if (node_is_empty (node)) {
-				g_hash_table_remove (store->nodes, &lookup_key);
-				gtk_object_destroy (GTK_OBJECT (node));
+				g_hash_table_remove (self->nodes, &lookup_key);
+				g_object_unref(G_OBJECT(node));
 			}
-			
 		} else {
-			/* FIXME: Fix this or just silently return if the part is non-existant? */
-/*			g_warning ("No node to remove part from.");*/
+			/* FIXME: Fix this or just silently return if the part is
+			   non-existant? */
+			g_warning ("No node to remove part from.");
 			return FALSE;
 		}
 	}
@@ -347,6 +372,19 @@ node_store_remove_part (NodeStore *store, Part *part)
 	return TRUE;
 }
 
+int
+node_store_add_textbox (NodeStore *self, Textbox *text)
+{
+	g_object_set (G_OBJECT (text), "store", self, NULL);
+	self->textbox = g_list_prepend (self->textbox, text);
+}
+
+int
+node_store_remove_textbox (NodeStore *self, Textbox *text)
+{
+	self->textbox = g_list_remove (self->textbox, text);
+}
+
 int
 node_store_add_wire (NodeStore *store, Wire *wire)
 {
@@ -381,33 +419,70 @@ node_store_add_wire (NodeStore *store, Wire *wire)
 
 	/*
 	 * Check for intersection with other wires.
-	 */ 
+	 */
 	ip_list = wires_intersect (store, x1, y1, x2, y2);
 
 	for (list = ip_list; list; list = list->next) {
 		ipoint = list->data;
-		
-/*
-		g_print ("(%g %g) (%g %g), ip (%g %g)\n", x1, y1, x2, y2, ipoint->pos.x, ipoint->pos.y);
+
+
+		//g_print ("(%g %g) (%g %g), ip (%g %g)\n", x1, y1, x2, y2, ipoint->pos.x, ipoint->pos.y);
 
 
 		if (IS_EQ (x1, x2) && ((ipoint->pos.y == y1) || (ipoint->pos.y == y2))) {
-			g_print ("vertical match.\n");
+			SheetPos w_pos, w_length;
+			wire_get_pos_and_length (ipoint->wire, &w_pos, &w_length);
+			gdouble _x1, _x2, _y1, _y2;
+
+			_x1 = w_pos.x;
+			_y1 = w_pos.y;
+			_x2 = x1 + w_length.x;
+			_y2 = y1 + w_length.y;
+
+			if (IS_EQ(_x1, _x2)) {
+				if (w_pos.x < pos.x) pos.x = w_pos.x;
+				if (w_pos.y < pos.y) pos.y = w_pos.y;
+				length.x += w_length.x;
+				length.y += w_length.y;
+
+				/* Update the new size and pos of the wire */
+				item_data_unregister (ITEM_DATA (ipoint->wire));
+				wire_set_length (ipoint->wire, &length);
+				item_data_set_pos (ITEM_DATA (ipoint->wire), &pos);
+				wire_update_bbox (ipoint->wire);
+				item_data_register (ITEM_DATA (ipoint->wire));
+
+				/* Done!, return -1 so wire si deleted */
+				return -1;
+			}
 		} else if (IS_EQ (y1, y2) && ((ipoint->pos.x == x1) || (ipoint->pos.x == x2))) {
-			g_print ("horizontal match.\n");
+			SheetPos w_pos, w_length;
+			wire_get_pos_and_length (ipoint->wire, &w_pos, &w_length);
+			gdouble _x1, _x2, _y1, _y2;
 
-		} else if (IS_EQ (x1, ipoint->pos.x) && ((x2 == x1) || (x2 == ipoint->pos.x))) {
-			g_print ("vertical match 2.\n");
-		} else if (IS_EQ (y1, y2) && ((ipoint->pos.x == x1) || (ipoint->pos.x == x2))) {
-			g_print ("horizontal match 2.\n");
+			_x1 = w_pos.x;
+			_y1 = w_pos.y;
+			_x2 = x1 + w_length.x;
+			_y2 = y1 + w_length.y;
 
-		} else {
-			g_print ("No match.\n");
+			if (IS_EQ(_y1, _y2)) {
+				if (w_pos.x < pos.x) pos.x = w_pos.x;
+				if (w_pos.y < pos.y) pos.y = w_pos.y;
+				length.x += w_length.x;
+				length.y += w_length.y;
 
-			g_free (ipoint);
-			continue;
+				/* Update the new size and pos of the wire */
+				item_data_unregister (ITEM_DATA (ipoint->wire));
+				wire_set_length (ipoint->wire, &length);
+				item_data_set_pos (ITEM_DATA (ipoint->wire), &pos);
+				wire_update_bbox (ipoint->wire);
+				item_data_register (ITEM_DATA (ipoint->wire));
+
+				/* Done!, return -1 so wire si deleted */
+				return -1;
+			}
 		}
-*/
+
 		node = node_store_get_or_create_node (store, ipoint->pos);
 
 		/*
@@ -444,7 +519,7 @@ node_store_add_wire (NodeStore *store, Wire *wire)
 
 	g_slist_free (ip_list);
 
-	gtk_object_set (GTK_OBJECT (wire), "store", store, NULL);
+	g_object_set (G_OBJECT (wire), "store", store, NULL);
 	store->wires = g_list_prepend (store->wires, wire);
 	store->items = g_list_prepend (store->items, wire);
 
@@ -466,11 +541,10 @@ node_store_remove_wire (NodeStore *store, Wire *wire)
 
 	priv = wire->priv;
 
-/*	if (item_data_get_store (ITEM_DATA (wire)) == NULL) {
+	if (item_data_get_store (ITEM_DATA (wire)) == NULL) {
 		g_warning ("Trying to remove non-stored wire.");
 		return FALSE;
 	}
-*/
 
 	wire_get_pos_and_length (wire, &pos, &length);
 
@@ -527,7 +601,7 @@ wire_intersect_parts (NodeStore *store, Wire *wire)
 	g_return_val_if_fail (IS_NODE_STORE (store), FALSE);
 	g_return_val_if_fail (wire != NULL, FALSE);
 	g_return_val_if_fail (IS_WIRE (wire), FALSE);
-	
+
 	ip_list = NULL;
 
 	wire_get_pos_and_length (wire, &wire_pos, &wire_length);
@@ -549,15 +623,15 @@ wire_intersect_parts (NodeStore *store, Wire *wire)
 
 		for (i = 0; i < num_pins; i++) {
 			Pin *pins;
-		
+
 			pins = part_get_pins (part);
 			x = part_pos.x + pins[i].offset.x;
 			y = part_pos.y + pins[i].offset.y;
 
 			lookup_pos.x = x;
 			lookup_pos.y = y;
-			
-			/* 
+
+			/*
 			 * If there is a wire at this pin's position,
 			 * add it to the return list.
 			 */
@@ -576,7 +650,7 @@ wire_intersect_parts (NodeStore *store, Wire *wire)
 }
 
 static GSList *
-wires_at_pos (NodeStore *store, SheetPos pos) 
+wires_at_pos (NodeStore *store, SheetPos pos)
 {
 	GList *list;
 	GSList *wire_list;
@@ -597,16 +671,16 @@ wires_at_pos (NodeStore *store, SheetPos pos)
 		y1 = wire_pos.y;
 		x2 = x1 + wire_length.x;
 		y2 = y1 + wire_length.y;
-		
+
 		if (is_wire_at_pos (x1, y1, x2, y2, pos))
 			wire_list = g_slist_prepend (wire_list, wire);
 	}
-	
+
 	return wire_list;
 }
 
 int
-node_store_is_wire_at_pos (NodeStore *store, SheetPos pos) 
+node_store_is_wire_at_pos (NodeStore *store, SheetPos pos)
 {
 	GList *list;
 	Wire *wire;
@@ -628,7 +702,7 @@ node_store_is_wire_at_pos (NodeStore *store, SheetPos pos)
 		if (is_wire_at_pos (x1, y1, x2, y2, pos))
 			return TRUE;
 	}
-	
+
 	return FALSE;
 }
 
@@ -649,7 +723,7 @@ wires_intersect (NodeStore *store, double x1, double y1, double x2, double y2)
 	wire1_start_pos.y = y1;
 	wire1_end_pos.x = x2;
 	wire1_end_pos.y = y2;
-	
+
 	/*
 	 * Search through all the wires. Is there a better way?
 	 */
@@ -663,7 +737,8 @@ wires_intersect (NodeStore *store, double x1, double y1, double x2, double y2)
 		wire2_x2 = wire2_x1 + wire2_length.x;
 		wire2_y2 = wire2_y1 + wire2_length.y;
 
-		if (do_wires_intersect (x1, y1, x2, y2, wire2_x1, wire2_y1, wire2_x2, wire2_y2, &pos)) {
+		if (do_wires_intersect (x1, y1, x2, y2, wire2_x1, wire2_y1,
+								wire2_x2, wire2_y2, &pos)) {
 			IntersectionPoint *ip;
 
 			ip = g_new0 (IntersectionPoint, 1);
@@ -683,15 +758,15 @@ wires_intersect (NodeStore *store, double x1, double y1, double x2, double y2)
  * Find the node that has an element at a certain position.
  */
 Node *
-node_store_get_node (NodeStore *store, SheetPos pos) 
+node_store_get_node (NodeStore *store, SheetPos pos)
 {
 	Node *node;
 
 	g_return_val_if_fail (store != NULL, NULL);
 	g_return_val_if_fail (IS_NODE_STORE (store), NULL);
-	
+
 	node = g_hash_table_lookup (store->nodes, &pos);
-	
+
 /*	if (!node)
 		g_print ("No node at (%g, %g)\n", pos.x, pos.y);
 	else
@@ -707,6 +782,7 @@ node_hash (gconstpointer key)
 	int x, y;
 
 	/* hasha p varannan bit? */
+	/* Can anybody translate this?	*/
 
 	x = (int)rint (sp->x) % 256;
 	y = (int)rint (sp->y) % 256;
@@ -772,7 +848,7 @@ is_wire_at_pos (double x1, double y1, double x2, double y2, SheetPos pos)
 	}
 
 //	g_print ("no match: (%g %g) -> (%g %g), (%g %g)\n", x1, y1, x2, y2, pos.x, pos.y);
-	
+
 	return FALSE;
 }
 
@@ -782,7 +858,8 @@ is_wire_at_pos (double x1, double y1, double x2, double y2, SheetPos pos)
  * is returned in pos.
  */
 static int
-do_wires_intersect (double Ax, double Ay, double Bx, double By, double Cx, double Cy, double Dx, double Dy, SheetPos *pos)
+do_wires_intersect (double Ax, double Ay, double Bx, double By, double Cx,
+					double Cy, double Dx, double Dy, SheetPos *pos)
 {
 	double r, s, d;
 
@@ -826,7 +903,7 @@ do_wires_intersect (double Ax, double Ay, double Bx, double By, double Cx, doubl
 	 *            not needed since we already checked for starts and ends
 	 */
 	if (fabs (r) < NODE_EPSILON) {
-		
+
 	}
 
 	r = r / d;
@@ -837,25 +914,25 @@ do_wires_intersect (double Ax, double Ay, double Bx, double By, double Cx, doubl
 	 * r and s in [0, 1].
 	 */
 	if (r >= 0 &&
-	    (r - 1.0) < NODE_EPSILON &&
-	    s >= 0 && 
-	    (s - 1.0) < NODE_EPSILON) {
+		(r - 1.0) < NODE_EPSILON &&
+		s >= 0 &&
+		(s - 1.0) < NODE_EPSILON) {
 
 		/*
 		 * Calculate the intersection point.
 		 */
 		pos->x = Ax + r * (Bx - Ax);
 		pos->y = Ay + r * (By - Ay);
-		
-		/* 
-		   to be accepted only if it coincides with the start or end 
+
+		/*
+		   to be accepted only if it coincides with the start or end
 		   of any of the wires
 		*/
 		if ( SEP(pos->x,pos->y,Ax,Ay) ||
-		     SEP(pos->x,pos->y,Bx,By) ||
-		     SEP(pos->x,pos->y,Cx,Cy) ||
-		     SEP(pos->x,pos->y,Dx,Dy) 
-		     )
+			 SEP(pos->x,pos->y,Bx,By) ||
+			 SEP(pos->x,pos->y,Cx,Cy) ||
+			 SEP(pos->x,pos->y,Dx,Dy)
+			 )
 
 		return TRUE;
 		else
@@ -910,7 +987,7 @@ node_store_dump_wires (NodeStore *store)
 		wire = wires->data;
 		wire_get_pos_and_length (wire, &start, &length);
 
-		g_print ("(%g %g) -> (%g %g):   %d nodes.\n", 
+		g_print ("(%g %g) -> (%g %g):   %d nodes.\n",
 			 start.x, start.y,
 			 start.x + length.x, start.y + length.y,
 			 g_slist_length (wire->priv->nodes));
@@ -974,7 +1051,7 @@ node_store_get_bounds (NodeStore *store, ArtDRect *rect)
 	rect->y0 = G_MAXDOUBLE;
 	rect->x1 = -G_MAXDOUBLE;
 	rect->y1 = -G_MAXDOUBLE;
-	
+
 	for (list = store->items; list; list = list->next) {
 		item_data_get_absolute_bbox (ITEM_DATA (list->data), &p1, &p2);
 
@@ -1002,7 +1079,8 @@ node_store_count_items (NodeStore *store, ArtDRect *rect)
 	for (list = store->items, n = 0; list; list = list->next) {
 		data = ITEM_DATA (list->data);
 		item_data_get_absolute_bbox (data, &p1, &p2);
-		if (p1.x <= rect->x1 && p1.y <= rect->y1 && p2.x >= rect->x0 && p2.y >= rect->y0) {
+		if (p1.x <= rect->x1 && p1.y <= rect->y1 &&
+			p2.x >= rect->x0 && p2.y >= rect->y0) {
 			n++;
 /*
 		if (p1.x >= rect->x0 && p1.y >= rect->y0 && p2.x <= rect->x1 && p2.y <= rect->y1) {
@@ -1028,8 +1106,66 @@ node_store_print_items (NodeStore *store, OreganoPrintContext *opc, ArtDRect *re
 		data = ITEM_DATA (list->data);
 		item_data_get_absolute_bbox (data, &p1, &p2);
 //		if (p1.x >= rect->x0 && p1.y >= rect->y0 && p2.x <= rect->x1 && p2.y <= rect->y1) {
-		if (p1.x <= rect->x1 && p1.y <= rect->y1 && p2.x >= rect->x0 && p2.y >= rect->y0) {
+		if ((p1.x <= rect->x1) && (p1.y <= rect->y1) &&
+			(p2.x >= rect->x0) && (p2.y >= rect->y0)) {
 			item_data_print (data, opc);
 		}
 	}
 }
+
+void
+node_store_print_labels (NodeStore *store, OreganoPrintContext *opc, ArtDRect *rect)
+{
+	GList *list;
+	SheetPos p1, p2;
+	ItemData *data;
+
+	g_return_if_fail (store != NULL);
+	g_return_if_fail (IS_NODE_STORE (store));
+	g_return_if_fail (rect != NULL);
+
+	for (list = store->textbox; list; list = list->next) {
+		data = ITEM_DATA (list->data);
+		item_data_get_absolute_bbox (data, &p1, &p2);
+//		if (p1.x >= rect->x0 && p1.y >= rect->y0 && p2.x <= rect->x1 && p2.y <= rect->y1) {
+		if ((p1.x <= rect->x1) && (p1.y <= rect->y1) &&
+			(p2.x >= rect->x0) && (p2.y >= rect->y0)) {
+			item_data_print (data, opc);
+		}
+	}
+}
+
+int
+node_store_is_pin_at_pos (NodeStore *store, SheetPos pos)
+{
+	int ret = 0;
+	int num_pins;
+	SheetPos part_pos;
+	GList *p;
+	Part *part;
+	Pin *pins;
+	int i;
+	gdouble x, y;
+
+	for (p = store->parts; p; p = p->next) {
+		part = PART (p->data);
+
+		num_pins = part_get_num_pins (part);
+
+		item_data_get_pos (ITEM_DATA (part), &part_pos);
+
+		pins = part_get_pins (part);
+		for (i = 0; i < num_pins; i++) {
+			x = part_pos.x + pins[i].offset.x;
+			y = part_pos.y + pins[i].offset.y;
+
+			if ((x == pos.x) && (y == pos.y)) {
+				ret = 1;
+				/* Fast exit ;-) */
+				goto fast_exit;
+			}
+		}
+	}
+fast_exit:
+	return ret;
+}
