@@ -2,20 +2,24 @@
  * part.c
  *
  * Part object: represents a schematic part
- * 
- * Author: 
+ *
+ * Authors:
  *  Richard Hult <rhult@hem.passagen.se>
- * 
- *  http://www.dtek.chalmers.se/~d4hult/oregano/ 
- * 
- * Copyright (C) 1999,2000  Richard Hult 
- * 
- * This program is free software; you can redistribute it and/or 
- * modify it under the terms of the GNU General Public License as 
- * published by the Free Software Foundation; either version 2 of the 
- * License, or (at your option) any later version. 
- * 
- * This program is distributed in the hope that it will be useful, 
+ *  Ricardo Markiewicz <rmarkie@fi.uba.ar>
+ *  Andres de Barbara <adebarbara@fi.uba.ar>
+ *
+ * Web page: http://arrakis.lug.fi.uba.ar/
+ *
+ * Copyright (C) 1999-2001  Richard Hult
+ * Copyright (C) 2003,2004  LUGFI
+ *
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
@@ -27,9 +31,10 @@
  */
 
 #include <config.h>
-#include <math.h>
-#include <gnome.h>
 #include <ctype.h>
+#include <gnome.h>
+#include <math.h>
+#include <string.h>
 #include "part.h"
 #include "item-data.h"
 #include "part-property.h"
@@ -40,22 +45,38 @@
 #include "load-library.h"
 #include "print.h"
 
-static void      part_class_init (PartClass *klass);
-static void      part_init (Part *part);
-static void      part_set_arg (GtkObject *object, GtkArg *arg, guint arg_id);
-static void      part_get_arg (GtkObject *object, GtkArg *arg, guint arg_id);
-static int       part_set_properties (Part *part, GSList *properties);
-static int       part_set_labels (Part *part, GSList *labels);
-static void      part_copy (ItemData *dest, ItemData *src);
+static void part_class_init (PartClass *klass);
+
+static void part_init (Part *part);
+
+static void part_set_gproperty (GObject *object, guint prop_id,
+	const GValue *value, GParamSpec *spec);
+
+static void part_get_gproperty (GObject *object, guint prop_id, GValue *value,
+	GParamSpec *spec);
+
+static int part_set_properties (Part *part, GSList *properties);
+
+static int part_set_labels (Part *part, GSList *labels);
+
+static void part_copy (ItemData *dest, ItemData *src);
+
 static ItemData *part_clone (ItemData *src);
-static void      part_rotate (ItemData *data, int angle, SheetPos *center);
-static void      part_flip (ItemData *data, gboolean horizontal, SheetPos *center);
-static void      part_update_bbox (Part *part);
-static void      part_unregister (ItemData *data);
-static void      part_register (ItemData *data);
-static void      part_set_property (ItemData *data, char *property, char *value);
-static char     *part_get_refdes_prefix (ItemData *data);
-static void	 part_print (ItemData *data, OreganoPrintContext *opc);
+
+static void part_rotate (ItemData *data, int angle, SheetPos *center);
+
+static void part_flip (ItemData *data, gboolean horizontal, SheetPos *center);
+
+static void part_update_bbox (Part *part);
+
+static void part_unregister (ItemData *data);
+
+static int part_register (ItemData *data);
+
+static void part_set_property (ItemData *data, char *property, char *value);
+
+static char *part_get_refdes_prefix (ItemData *data);
+static void part_print (ItemData *data, OreganoPrintContext *opc);
 
 enum {
 	ARG_0,
@@ -64,37 +85,40 @@ enum {
 };
 
 enum {
+	CHANGED,
 	LAST_SIGNAL
 };
 
-/*static guint part_signals [LAST_SIGNAL] = { 0 };*/
+static guint part_signals [LAST_SIGNAL] = { 0 };
 static ItemDataClass *parent_class = NULL;
 
-guint
+GType
 part_get_type (void)
 {
-	static guint part_type = 0;
+	static GType part_type = 0;
 
 	if (!part_type) {
-		static const GtkTypeInfo part_info = {
-			"Part",
-			sizeof (Part),
-			sizeof (PartClass),
-			(GtkClassInitFunc) part_class_init,
-			(GtkObjectInitFunc) part_init,
-			/* reserved_1 */ NULL,
-			/* reserved_2 */ NULL,
-			(GtkClassInitFunc) NULL,
+		static const GTypeInfo part_info = {
+			sizeof(PartClass),
+			NULL, /* Base Init */
+			NULL, /* Base Finalize */
+			(GClassInitFunc)part_class_init, /* Class Init */
+			NULL, /* Class Finalize */
+			NULL, /* Class Data */
+			sizeof(Part),
+			0, /* n_preallocs */
+			(GInstanceInitFunc)part_init, /* Instance init */
+			NULL
 		};
-	
-		part_type = gtk_type_unique (TYPE_ITEM_DATA, &part_info);
+		part_type = g_type_register_static(TYPE_ITEM_DATA,
+			"Part", &part_info, 0);
 	}
 
 	return part_type;
 }
 
 static void
-part_destroy (GtkObject *object)
+part_finalize(GObject *object)
 {
 	Part *part;
 	PartPriv *priv;
@@ -102,50 +126,65 @@ part_destroy (GtkObject *object)
 
 	part = PART (object);
 	priv = part->priv;
-	
-	g_free (priv->name);
 
-	for (list = priv->properties; list; list = list->next) {
-		PartProperty *property = list->data;
+	if (priv) {
+		g_free (priv->name);
 
-		g_free (property->name);
-		g_free (property->value);
-		g_free (property);
+		for (list = priv->properties; list; list = list->next) {
+			PartProperty *property = list->data;
+
+			g_free (property->name);
+			g_free (property->value);
+			g_free (property);
+		}
+		g_slist_free (priv->properties);
+
+		for (list = priv->labels; list; list = list->next) {
+			PartLabel *label = list->data;
+
+			g_free (label->name);
+			g_free (label->text);
+			g_free (label);
+		}
+		g_slist_free (priv->labels);
+
+		g_free (priv->pins);
+		g_free (priv->symbol_name);
+		g_free (priv);
+		part->priv = NULL;
 	}
-	g_slist_free (priv->properties);
 
-	for (list = priv->labels; list; list = list->next) {
-		PartLabel *label = list->data;
+	G_OBJECT_CLASS(parent_class)->finalize(object);
+}
 
-		g_free (label->name);
-		g_free (label->text);
-		g_free (label);
-	}
-	g_slist_free (priv->labels);
-
-	g_free (priv->pins);
-	g_free (priv->symbol_name);
-
-	GTK_OBJECT_CLASS (parent_class)->destroy (object);
+static void
+part_dispose(GObject *object)
+{
+	G_OBJECT_CLASS(parent_class)->dispose(object);
 }
 
 static void
 part_class_init (PartClass *klass)
 {
-	GtkObjectClass *object_class;
+	GObjectClass *object_class;
 	ItemDataClass *item_data_class;
 
-	parent_class = gtk_type_class (TYPE_ITEM_DATA);
+	parent_class = g_type_class_peek(TYPE_ITEM_DATA);
 
-	object_class = (GtkObjectClass *) klass;
-	item_data_class = (ItemDataClass *) klass;
+	object_class = G_OBJECT_CLASS(klass);
+	item_data_class = ITEM_DATA_CLASS(klass);
 
-	gtk_object_add_arg_type ("Part::properties", GTK_TYPE_POINTER, GTK_ARG_WRITABLE, ARG_PROPERTIES);
-	gtk_object_add_arg_type ("Part::labels", GTK_TYPE_POINTER, GTK_ARG_READWRITE, ARG_LABELS);
+	object_class->set_property = part_set_gproperty;
+	object_class->get_property = part_get_gproperty;
+	object_class->dispose = part_dispose;
+	object_class->finalize = part_finalize;
 
-	object_class->destroy = part_destroy;
-	object_class->set_arg = part_set_arg;
-	object_class->get_arg = part_get_arg;
+	g_object_class_install_property ( object_class,	ARG_PROPERTIES,
+		g_param_spec_pointer("properties", "properties",
+			"the properties", G_PARAM_READWRITE));
+	g_object_class_install_property(object_class, ARG_LABELS,
+		g_param_spec_pointer("labels", "labels", "the labels",
+			G_PARAM_READWRITE));
 
 	item_data_class->clone = part_clone;
 	item_data_class->copy = part_copy;
@@ -158,21 +197,25 @@ part_class_init (PartClass *klass)
 	item_data_class->set_property = part_set_property;
 	item_data_class->print = part_print;
 
-/*	part_signals [CHANGED] = 
-		gtk_signal_new ("changed",
-				GTK_RUN_FIRST,
-				object_class->type,
-				GTK_SIGNAL_OFFSET (PartClass, changed),
-				gtk_marshal_NONE__NONE,
-				GTK_TYPE_NONE, 0);
-*/
-//	gtk_object_class_add_signals (object_class, part_signals, LAST_SIGNAL);
+	part_signals[CHANGED] =
+		g_signal_new ("changed", G_TYPE_FROM_CLASS(object_class),
+			G_SIGNAL_RUN_FIRST,
+			G_STRUCT_OFFSET (PartClass, changed),
+			NULL,
+			NULL,
+			g_cclosure_marshal_VOID__VOID,
+			G_TYPE_NONE,
+			0);
+
+// ?	g_object_class_add_signals (object_class, part_signals, LAST_SIGNAL);
 }
 
 static void
 part_init (Part *part)
 {
-	PartPriv *priv = g_new0 (PartPriv, 1);
+	PartPriv *priv;
+
+	priv = g_new0(PartPriv, 1);
 
 	part->priv = priv;
 }
@@ -182,7 +225,7 @@ part_new (void)
 {
 	Part *part;
 
-	part = gtk_type_new (part_get_type ());
+	part = PART(g_object_new(TYPE_PART, NULL));
 
 	return part;
 }
@@ -197,7 +240,7 @@ part_new_from_library_part (LibraryPart *library_part)
 
 	g_return_val_if_fail (library_part != NULL, NULL);
 
-	part = part_new ();
+	part = part_new();
 	if (!part)
 		return NULL;
 
@@ -205,22 +248,22 @@ part_new_from_library_part (LibraryPart *library_part)
 
 	symbol = library_get_symbol (library_part->symbol_name);
 	if (symbol ==  NULL){
-		g_warning ("Couldn't find the requested symbol %s for part %s in library.\n", 
-			   library_part->symbol_name, 
-			   library_part->name);
+		g_warning ("Couldn't find the requested symbol %s for part %s in library.\n",
+			library_part->symbol_name,
+			library_part->name);
 		return NULL;
 	}
-	
+
 	pins = symbol->connections;
 
-	if ( pins ) 
-	part_set_pins (part, pins);
+	if ( pins )
+		part_set_pins (part, pins);
 
 
-	gtk_object_set (
-		GTK_OBJECT (part),
-		"properties", library_part->properties,
-		"labels", library_part->labels,
+	g_object_set (
+		G_OBJECT (part),
+		"Part::properties", library_part->properties,
+		"Part::labels", library_part->labels,
 		NULL);
 
 	/* FIXME: */
@@ -229,42 +272,42 @@ part_new_from_library_part (LibraryPart *library_part)
 	priv->library = library_part->library;
 
 	part_update_bbox (part);
-	
+
 	return part;
 }
 
 static void
-part_set_arg (GtkObject *object, GtkArg *arg, guint arg_id)
+part_set_gproperty(GObject *object, guint prop_id, const GValue *value,
+	GParamSpec *spec)
 {
+	GSList *list;
 	Part *part = PART (object);
 
-	switch (arg_id) {
+	switch (prop_id) {
 	case ARG_PROPERTIES:
-		part_set_properties (part, GTK_VALUE_POINTER (*arg));
+		list = g_value_get_pointer(value);
+		part_set_properties (part, list);
 		break;
 	case ARG_LABELS:
-		part_set_labels (part, GTK_VALUE_POINTER (*arg));
-		break;
-
-	default:
+		list = g_value_get_pointer(value);
+		part_set_labels (part, list);
 		break;
 	}
 }
 
 static void
-part_get_arg (GtkObject *object, GtkArg *arg, guint arg_id)
+part_get_gproperty(GObject *object, guint prop_id, GValue *value,
+	GParamSpec *spec)
 {
 	Part *part = PART (object);
 	PartPriv *priv = part->priv;
 
-	switch (arg_id) {
+	switch (prop_id) {
 	case ARG_LABELS:
-		GTK_VALUE_POINTER (*arg) = priv->labels;
+		g_value_set_pointer(value, priv->labels);
 		break;
-
 	default:
-		arg->type = GTK_TYPE_INVALID;
-		break;
+		G_OBJECT_WARN_INVALID_PROPERTY_ID(part, prop_id, spec);
 	}
 }
 
@@ -324,7 +367,7 @@ part_set_properties (Part *part, GSList *properties)
 
 	g_return_val_if_fail (part != NULL, FALSE);
 	g_return_val_if_fail (IS_PART (part), FALSE);
-	
+
 	priv = part->priv;
 
 	if (priv->properties != NULL)
@@ -354,7 +397,7 @@ part_get_properties (Part *part)
 
 	g_return_val_if_fail (part != NULL, FALSE);
 	g_return_val_if_fail (IS_PART (part), FALSE);
-	
+
 	priv = part->priv;
 
 	return priv->properties;
@@ -407,7 +450,7 @@ part_set_labels (Part *part, GSList *labels)
 
 	for (list = labels; list; list = list->next) {
 		PartLabel *label, *label_copy;
-		
+
 		label = list->data;
 
 		label_copy = g_new0 (PartLabel, 1);
@@ -415,9 +458,9 @@ part_set_labels (Part *part, GSList *labels)
 		label_copy->text = g_strdup (label->text);
 		label_copy->pos.x = label->pos.x;
 		label_copy->pos.y = label->pos.y;
-		priv->labels = g_slist_prepend (priv->labels, label_copy);	
-	}	
-	
+		priv->labels = g_slist_prepend (priv->labels, label_copy);
+	}
+
 	return TRUE;
 }
 
@@ -441,7 +484,7 @@ part_set_pins (Part *part, GSList *pins)
 
 	priv->pins = g_new0 (Pin, num_pins);
 	priv->num_pins = num_pins;
-	
+
 	for (list = pins, i = 0; list; list = list->next, i++) {
 		/*
 		 * Note that this is slightly hackish. The list contains
@@ -450,11 +493,12 @@ part_set_pins (Part *part, GSList *pins)
 		Pin *pin = list->data;
 
 		priv->pins[i].pin_nr = i;
+		priv->pins[i].node_nr= 0;
 		priv->pins[i].offset.x = pin->offset.x;
 		priv->pins[i].offset.y = pin->offset.y;
 		priv->pins[i].part = part;
-	}	
-	
+	}
+
 	return TRUE;
 }
 
@@ -465,7 +509,7 @@ part_get_labels (Part *part)
 
 	g_return_val_if_fail (part != NULL, NULL);
 	g_return_val_if_fail (IS_PART (part), NULL);
-	
+
 	priv = part->priv;
 
 	return priv->labels;
@@ -491,8 +535,8 @@ part_rotate (ItemData *data, int angle, SheetPos *center)
 
 	priv = part->priv;
 
-	tot_rotation = priv->rotation + angle;
-	tot_rotation = tot_rotation % 360;
+	tot_rotation = (priv->rotation + angle)%360;
+	
 	priv->rotation = tot_rotation;
 
 	art_affine_rotate (affine, angle);
@@ -514,16 +558,9 @@ part_rotate (ItemData *data, int angle, SheetPos *center)
 		priv->pins[i].offset.y = dst.y;
 	}
 
-	/*
-	 * Let the views (canvas items) know about the rotation.
-	 */
-	gtk_signal_emit_by_name (GTK_OBJECT (part), "rotated", angle);
-
-	if (center) {
-		item_data_get_relative_bbox (ITEM_DATA (part), &b1, &b2);
-		part_center_before.x = b1.x + (b2.x - b1.x) / 2;
-		part_center_before.y = b1.y + (b2.y - b1.y) / 2;
-	}
+	item_data_get_relative_bbox (ITEM_DATA (part), &b1, &b2);
+	part_center_before.x = b1.x + (b2.x - b1.x) / 2;
+	part_center_before.y = b1.y + (b2.y - b1.y) / 2;
 
 	/*
 	 * Rotate the bounding box.
@@ -547,21 +584,27 @@ part_rotate (ItemData *data, int angle, SheetPos *center)
 
 		part_center_after.x = b1.x + (b2.x - b1.x) / 2;
 		part_center_after.y = b1.y + (b2.y - b1.y) / 2;
-		
+
 		dx = part_center_before.x - part_center_after.x;
 		dy = part_center_before.y - part_center_after.y;
-		
+
 		item_data_get_pos (ITEM_DATA (part), &part_pos);
 
 		src.x = part_center_before.x - center->x + part_pos.x;
 		src.y = part_center_before.y - center->y + part_pos.y;
 		art_affine_point (&dst, &src, affine);
-		
+
 		delta.x = dx - src.x + dst.x;
 		delta.y = dy - src.y + dst.y;
 
-		item_data_move (ITEM_DATA (part), &delta);		
+		item_data_move (ITEM_DATA (part), &delta);
 	}
+	
+	/*
+	 * Let the views (canvas items) know about the rotation.
+	 */
+	g_signal_emit_by_name (G_OBJECT (part), "rotated", angle);
+
 }
 
 static void
@@ -574,7 +617,7 @@ part_flip (ItemData *data, gboolean horizontal, SheetPos *center)
 	double affine[6];
 	ArtPoint src, dst;
 	SheetPos part_center_before, part_center_after;
-	
+
 	g_return_if_fail (data != NULL);
 	g_return_if_fail (IS_PART (data));
 
@@ -604,9 +647,9 @@ part_flip (ItemData *data, gboolean horizontal, SheetPos *center)
 		src.y = priv->pins[i].offset.y;
 		art_affine_point (&dst, &src, affine);
 
-		if (fabs (dst.x) < 1e-2) 
+		if (fabs (dst.x) < 1e-2)
 			dst.x = 0;
-		if (fabs (dst.y) < 1e-2) 
+		if (fabs (dst.y) < 1e-2)
 			dst.y = 0;
 
 		priv->pins[i].offset.x = dst.x;
@@ -616,7 +659,7 @@ part_flip (ItemData *data, gboolean horizontal, SheetPos *center)
 	/*
 	 * Tell the views.
 	 */
-	gtk_signal_emit_by_name (GTK_OBJECT (part), "flipped", horizontal);
+	g_signal_emit_by_name (G_OBJECT (part), "flipped", horizontal);
 
 	if (center) {
 		item_data_get_relative_bbox (ITEM_DATA (part), &b1, &b2);
@@ -647,20 +690,20 @@ part_flip (ItemData *data, gboolean horizontal, SheetPos *center)
 
 		part_center_after.x = b1.x + (b2.x - b1.x) / 2;
 		part_center_after.y = b1.y + (b2.y - b1.y) / 2;
-		
+
 		dx = part_center_before.x - part_center_after.x;
 		dy = part_center_before.y - part_center_after.y;
-		
+
 		item_data_get_pos (ITEM_DATA (part), &part_pos);
 
 		src.x = part_center_before.x - center->x + part_pos.x;
 		src.y = part_center_before.y - center->y + part_pos.y;
 		art_affine_point (&dst, &src, affine);
-		
+
 		delta.x = dx - src.x + dst.x;
 		delta.y = dy - src.y + dst.y;
 
-		item_data_move (ITEM_DATA (part), &delta);		
+		item_data_move (ITEM_DATA (part), &delta);
 	}
 }
 
@@ -669,17 +712,17 @@ part_clone (ItemData *src)
 {
 	Part *src_part, *new_part;
 	ItemDataClass *id_class;
-	
+
 	g_return_val_if_fail (src != NULL, NULL);
 	g_return_val_if_fail (IS_PART (src), NULL);
 
-	id_class = ITEM_DATA_CLASS (GTK_OBJECT(src)->klass);
+	id_class = ITEM_DATA_CLASS (G_OBJECT_GET_CLASS(src));
 	if (id_class->copy == NULL)
 		return NULL;
 
 	src_part = PART (src);
-	new_part = gtk_type_new (TYPE_PART);
-	new_part->priv->pins = g_new0 (Pin, src_part->priv->num_pins); 
+	new_part = PART(g_object_new(TYPE_PART, NULL));
+	new_part->priv->pins = g_new0 (Pin, src_part->priv->num_pins);
 	id_class->copy (ITEM_DATA (new_part), src);
 
 	return ITEM_DATA (new_part);
@@ -691,7 +734,7 @@ part_copy (ItemData *dest, ItemData *src)
 	Part *dest_part, *src_part;
 	GSList *list;
 	int i;
-	
+
 	g_return_if_fail (dest != NULL);
 	g_return_if_fail (IS_PART (dest));
 	g_return_if_fail (src != NULL);
@@ -709,14 +752,16 @@ part_copy (ItemData *dest, ItemData *src)
 	dest_part->priv->name = g_strdup (src_part->priv->name);
 	dest_part->priv->symbol_name = g_strdup (src_part->priv->symbol_name);
 
-	memcpy (dest_part->priv->pins, src_part->priv->pins, src_part->priv->num_pins * sizeof (Pin));
+	memcpy (dest_part->priv->pins, src_part->priv->pins,
+		src_part->priv->num_pins * sizeof (Pin));
 	for (i = 0; i < dest_part->priv->num_pins; i++)
 		dest_part->priv->pins[i].part = dest_part;
-	
+
 	/*
 	 * Copy properties and labels.
 	 */
-	dest_part->priv->properties = g_slist_copy (src_part->priv->properties);
+	dest_part->priv->properties =
+		g_slist_copy (src_part->priv->properties);
 	for (list = dest_part->priv->properties; list; list = list->next) {
 		PartProperty *prop, *new_prop;
 
@@ -743,9 +788,9 @@ part_copy (ItemData *dest, ItemData *src)
 static void
 part_update_bbox (Part *part)
 {
-	GSList            *objects;
-	LibrarySymbol     *symbol;
-	SymbolObject      *object;
+	GSList *objects;
+	LibrarySymbol *symbol;
+	SymbolObject *object;
 	GnomeCanvasPoints *points;
 	int i;
 
@@ -756,10 +801,11 @@ part_update_bbox (Part *part)
 		g_warning ("Couldn't find the requested symbol.");
 		return;
 	}
-	
+
 	b1.x = b1.y = b2.x = b2.y = 0.0;
 
-	for (objects = symbol->symbol_objects; objects; objects = objects->next){
+	for (objects = symbol->symbol_objects; objects;
+	     objects = objects->next){
 		object = objects->data;
 		switch (object->type){
 		case SYMBOL_OBJECT_LINE:
@@ -771,9 +817,9 @@ part_update_bbox (Part *part)
 
 				b2.x = MAX (points->coords[i * 2], b2.x);
 				b2.y = MAX (points->coords[i * 2 + 1], b2.y);
-			}				
+			}
 			break;
-			
+
 		case SYMBOL_OBJECT_ARC:
 			b1.x = MIN (object->u.arc.x1, b1.x);
 			b1.y = MIN (object->u.arc.y1, b1.y);
@@ -789,15 +835,15 @@ part_update_bbox (Part *part)
 
 			break;
 
-		case SYMBOL_OBJECT_TEXT: 
+		case SYMBOL_OBJECT_TEXT:
 		   {
-		      GdkFont  *font = 
-			 gdk_font_load ("-*-helvetica-medium-r-*-*-*-100-*-*-*-*-*-*");
-		      b1.x = b1.y =  0;
-		      b2.x = 2*object->u.text.x + 
-			     gdk_string_width (font, object->u.text.str );
-		      b2.y = 2*object->u.text.y + 
-			     gdk_string_height (font,object->u.text.str );
+			   /*GdkFont *font = gdk_font_load ("Sans 10");
+				 b1.x = b1.y =  0;
+				 b2.x = 2*object->u.text.x +
+				 gdk_string_width (font, object->u.text.str );
+				 b2.y = 2*object->u.text.y +
+				 gdk_string_height (font,object->u.text.str );
+				*/
 		   }
 		   break;
 
@@ -822,15 +868,17 @@ part_unregister (ItemData *data)
 	node_store_remove_part (store, PART (data));
 }
 
-static void
+static int 
 part_register (ItemData *data)
 {
 	NodeStore *store;
 
-	g_return_if_fail (IS_PART (data));
+	g_return_val_if_fail (IS_PART (data), -1);
 
 	store = item_data_get_store (data);
 	node_store_add_part (store, PART (data));
+
+	return 0;
 }
 
 static char *
@@ -848,13 +896,13 @@ part_get_refdes_prefix (ItemData *data)
 	refdes = part_get_property (part, "refdes");
 	if (refdes == NULL)
 		return NULL;
-	
+
 	/*
 	 * FIXME: the ref.des. should be stored in two parts,
 	 * as a char and an integer.
 	 */
 
-	/* 
+	 /*
 	 * Get the 'prefix' i.e R for resistors.
 	 */
 	found_num = FALSE;
@@ -870,7 +918,7 @@ part_get_refdes_prefix (ItemData *data)
 		return NULL; //g_strndup (refdes, i - 1);
 	else
 		return g_strdup (refdes);
-			
+
 	return NULL;
 }
 
@@ -938,65 +986,68 @@ part_print (ItemData *data, OreganoPrintContext *opc)
 
 	flip = part_get_flip (part);
 	if (flip) {
-		art_affine_flip (affine, affine, flip & ID_FLIP_HORIZ, flip & ID_FLIP_VERT);
+		art_affine_flip (affine, affine,
+			flip & ID_FLIP_HORIZ, flip & ID_FLIP_VERT);
 	}
 
 	symbol = library_get_symbol (priv->symbol_name);
 	if (symbol == NULL) {
 		return;
 	}
-	
+
 	for (objects = symbol->symbol_objects; objects; objects = objects->next) {
 		object = (SymbolObject *)(objects->data);
-		
+
 		switch (object->type) {
-		case SYMBOL_OBJECT_LINE:
-			gnome_print_setlinewidth (opc->ctx, 0);
-			line = object->u.uline.line;
-			for (i = 0; i < line->num_points; i++) {
-				double x, y;
-				
-				x = line->coords[i * 2];
-				y = line->coords[i * 2 + 1];
-				
-				if (rotation != 0 || flip != ID_FLIP_NONE) {
-					src.x = x;
-					src.y = y;
-					art_affine_point (&dst, &src, affine);
-					x = dst.x;
-					y = dst.y;
+			case SYMBOL_OBJECT_LINE:
+				gnome_print_setlinewidth (opc->ctx, 0);
+				line = object->u.uline.line;
+				for (i = 0; i < line->num_points; i++) {
+					double x, y;
+
+					x = line->coords[i * 2];
+					y = line->coords[i * 2 + 1];
+
+					if (rotation != 0 || flip != ID_FLIP_NONE) {
+						src.x = x;
+						src.y = y;
+						art_affine_point (&dst, &src, affine);
+						x = dst.x;
+						y = dst.y;
+					}
+
+					if (i == 0)
+						gnome_print_moveto (opc->ctx, x0 + x, y0 + y);
+					else
+						gnome_print_lineto (opc->ctx, x0 + x, y0 + y);
 				}
-				
-				if (i == 0)
-					gnome_print_moveto (opc->ctx, x0 + x, y0 + y);
-				else
-					gnome_print_lineto (opc->ctx, x0 + x, y0 + y);
+			break;
+			case SYMBOL_OBJECT_ARC: {
+				gdouble x1 = object->u.arc.x1;
+				gdouble y1 = object->u.arc.y1;
+				gdouble x2 = object->u.arc.x2;
+				gdouble y2 = object->u.arc.y2;
+				gdouble width, height;
+				ArtPoint center;
+
+				src.x = (x2 - x1) / 2 + x1;
+				src.y = (y2 - y1) / 2 + y1;
+				width = x2 - x1;
+				height = y2 - y1;
+
+				art_affine_point (&center, &src, affine);
+				center.x += x0;
+				center.y += y0;
+
+				gnome_print_setlinewidth (opc->ctx, 0);
+				print_draw_ellipse (opc->ctx, &center, width, height);
 			}
 			break;
-			
-		case SYMBOL_OBJECT_ARC: {
-			gdouble x1 = object->u.arc.x1;
-			gdouble y1 = object->u.arc.y1;
-			gdouble x2 = object->u.arc.x2;
-			gdouble y2 = object->u.arc.y2;
-			gdouble width, height;
-			ArtPoint center;
-
-			src.x = (x2 - x1) / 2 + x1;
-			src.y = (y2 - y1) / 2 + y1;
-			width = x2 - x1; 
-			height = y2 - y1; 
-
-			art_affine_point (&center, &src, affine);
-			center.x += x0;
-			center.y += y0;
-		
-			gnome_print_setlinewidth (opc->ctx, 0);
-			print_draw_ellipse (opc->ctx, &center, width, height);
-			break;
-		}
-		default:
-			g_warning ("Print part: Part %s contains unknown object.", priv->name);
+			default:
+				g_warning (
+					"Print part: Part %s contains unknown object.",
+					priv->name
+				);
 			continue;
 		}
 
@@ -1007,7 +1058,7 @@ part_print (ItemData *data, OreganoPrintContext *opc)
 			ArtPoint label_pos;
 			gchar *text;
 			gint text_width, text_height;
-			
+
 			label_pos.x = label->pos.x;
 			label_pos.y = label->pos.y;
 
@@ -1018,38 +1069,41 @@ part_print (ItemData *data, OreganoPrintContext *opc)
 				label_pos.x = dst.x;
 				label_pos.y = dst.y;
 			}
-
+			
 			label_pos.x += x0;
 			label_pos.y += y0;
-			
+
 			text = part_property_expand_macros (part, label->text);
 
-			text_height = gnome_font_get_ascender (opc->label_font) +  gnome_font_get_descender (opc->label_font);
-			text_width = gnome_font_get_width_string (opc->label_font, text);
-			
+			text_height =
+				gnome_font_face_get_ascender(opc->label_font) +
+				gnome_font_face_get_descender(opc->label_font);
+			text_width = 5*strlen(text);
+			//gnome_font_get_width_string (opc->label_font, text);
+
 			/* Align the label. */
 			switch (rotation) {
 			case 90:
 				/* North west */
-				label_pos.y += text_height;
-				break;
-
+				label_pos.y += text_height*opc->scale;
+			break;
 			case 180:
 				/* West */
-				break;
-
+			break;
 			case 270:
 				/* South east */
-				label_pos.x -= text_width;
-				break;
-
+				label_pos.x -= text_width*opc->scale;
+			break;
 			case 0:
 			default:
 				/* South west */
-				break;
+			break;
 			}
 
-			gnome_print_setfont (opc->ctx, opc->label_font);
+			gnome_print_setfont(opc->ctx,
+				gnome_font_face_get_font_default(
+				opc->label_font, 6)
+			);
 			print_draw_text (opc->ctx, text, &label_pos);
 			g_free (text);
 		}
