@@ -1,20 +1,23 @@
 /*
  * sheet-item.c
  *
- * 
- * Author: 
+ *
+ * Authors:
  *  Richard Hult <rhult@hem.passagen.se>
- * 
- *  http://www.dtek.chalmers.se/~d4hult/oregano/ 
- * 
- * Copyright (C) 1999,2000  Richard Hult 
- * 
- * This program is free software; you can redistribute it and/or 
- * modify it under the terms of the GNU General Public License as 
- * published by the Free Software Foundation; either version 2 of the 
- * License, or (at your option) any later version. 
- * 
- * This program is distributed in the hope that it will be useful, 
+ *  Ricardo Markiewicz <rmarkie@fi.uba.ar>
+ *  Andres de Barbara <adebarbara@fi.uba.ar>
+ *
+ * Web page: http://arrakis.lug.fi.uba.ar/
+ *
+ * Copyright (C) 1999-2001  Richard Hult
+ * Copyright (C) 2003,2004  LUGFI
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
@@ -26,7 +29,6 @@
  */
 
 #include <config.h>
-#include <gnome.h>
 #include <math.h>
 #include "main.h"
 #include "sheet-private.h"
@@ -36,20 +38,28 @@
 #include "schematic-view.h"
 
 static void sheet_item_class_init (SheetItemClass * klass);
+
 static void sheet_item_init (SheetItem *item);
 
-static void sheet_item_set_arg (GtkObject *object, GtkArg *arg, guint arg_id);
-static void sheet_item_get_arg (GtkObject *object, GtkArg *arg, guint arg_id);
+static void sheet_item_set_property (GObject *object, guint prop_id,
+									 const GValue *value, GParamSpec *spec);
+
+static void sheet_item_get_property (GObject *object, guint prop_id,
+										 GValue *value, GParamSpec *spec);
+
 static void sheet_item_destroy (GtkObject *object);
-static void sheet_item_run_menu (SheetItem *item, SchematicView *sv, GdkEventButton *event, SheetItemMenu *context_menu);
+
+static void sheet_item_run_menu (SheetItem *item, SchematicView *sv,
+	GdkEventButton *event,
+	SheetItemMenu *context_menu);
 
 static GnomeCanvasGroupClass *sheet_item_parent_class = NULL;
-extern GtkObject *clipboard_data_get_item_data ();
-extern GtkObjectClass *clipboard_data_get_item_class ();
+extern GObject *clipboard_data_get_item_data ();
+extern GObjectClass *clipboard_data_get_item_class ();
 
 struct _SheetItemPriv {
-	gboolean selected : 1;
-	gboolean preserve_selection : 1;
+	guint selected : 1;
+	guint preserve_selection : 1;
 	ItemData *data;
 };
 
@@ -71,23 +81,27 @@ enum {
 
 static guint so_signals[LAST_SIGNAL] = { 0 };
 
-GtkType
+GType
 sheet_item_get_type ()
 {
-	static GtkType sheet_item_type = 0;
-	
+	static GType sheet_item_type = 0;
+
 	if (!sheet_item_type) {
-		GtkTypeInfo sheet_item_info = {
-			"SheetItem",
-			sizeof (SheetItem),
+		static const GTypeInfo sheet_item_info = {
 			sizeof (SheetItemClass),
-			(GtkClassInitFunc) sheet_item_class_init,
-			(GtkObjectInitFunc) sheet_item_init,
-			(GtkArgSetFunc) NULL,
-			(GtkArgGetFunc) NULL,
+			NULL,
+			NULL,
+			(GClassInitFunc) sheet_item_class_init,
+			NULL,
+			NULL,
+			sizeof (SheetItem),
+			0,
+			(GInstanceInitFunc)sheet_item_init,
 			NULL
 		};
-		sheet_item_type = gtk_type_unique (gnome_canvas_group_get_type (), &sheet_item_info);
+		sheet_item_type = g_type_register_static(GNOME_TYPE_CANVAS_GROUP,
+												 "SheetItem",
+												 &sheet_item_info, 0);
 	}
 	return sheet_item_type;
 }
@@ -95,19 +109,32 @@ sheet_item_get_type ()
 static void
 sheet_item_class_init (SheetItemClass *sheet_item_class)
 {
-	GtkObjectClass *object_class;
-	
-	object_class = (GtkObjectClass *) sheet_item_class;
-	sheet_item_parent_class = gtk_type_class (gnome_canvas_group_get_type ());
+	GObjectClass *object_class;
+	GtkObjectClass *gtk_object_class;
 
-	gtk_object_add_arg_type ("SheetItem::data", GTK_TYPE_POINTER, GTK_ARG_READWRITE, ARG_DATA);
-	gtk_object_add_arg_type ("SheetItem::sheet", GTK_TYPE_POINTER, GTK_ARG_READABLE, ARG_SHEET);
+	object_class = G_OBJECT_CLASS(sheet_item_class);
+	gtk_object_class = GTK_OBJECT_CLASS(sheet_item_class);
+	sheet_item_parent_class = g_type_class_peek_parent(sheet_item_class);
+
+	object_class->set_property = sheet_item_set_property;
+	object_class->get_property = sheet_item_get_property;
+
+	g_object_class_install_property(
+				object_class,
+				ARG_DATA,
+				g_param_spec_pointer("data", "SheetItem::data", "the data",
+									 G_PARAM_READWRITE)
+	);
+	g_object_class_install_property(
+				object_class,
+				ARG_SHEET,
+				g_param_spec_pointer("sheet", "SheetItem::sheet", "the sheet",
+									 G_PARAM_READABLE)
+	);
 
 	sheet_item_class->context_menu = NULL;
 
-	object_class->set_arg = sheet_item_set_arg;
-	object_class->get_arg = sheet_item_get_arg;
-	object_class->destroy = sheet_item_destroy;
+	gtk_object_class->destroy = sheet_item_destroy;
 
 	sheet_item_class->is_in_area = NULL;
 	sheet_item_class->show_labels = NULL;
@@ -118,46 +145,45 @@ sheet_item_class_init (SheetItemClass *sheet_item_class)
 	sheet_item_class->mouse_over = NULL;
 
 	so_signals[PLACED] =
-		gtk_signal_new ("placed",
-				GTK_RUN_FIRST,
-				object_class->type,
+		g_signal_new ("placed",
+				G_TYPE_FROM_CLASS(object_class),
+				G_SIGNAL_RUN_FIRST,
 				0,
-				gtk_marshal_NONE__NONE,
-				GTK_TYPE_NONE, 0);
+				NULL,
+				NULL,
+				g_cclosure_marshal_VOID__VOID,
+				G_TYPE_NONE, 0);
 
-	so_signals[MOVED] =
-		gtk_signal_new ("moved",
-				GTK_RUN_FIRST,
-				object_class->type,
-				GTK_SIGNAL_OFFSET (SheetItemClass, moved),
-				gtk_marshal_NONE__NONE,
-				GTK_TYPE_NONE, 0);
+	so_signals[MOVED] = g_signal_new ("moved",
+		G_TYPE_FROM_CLASS(object_class),
+		G_SIGNAL_RUN_FIRST,
+		G_STRUCT_OFFSET(SheetItemClass, moved),
+		NULL, NULL,
+		g_cclosure_marshal_VOID__VOID,
+		G_TYPE_NONE, 0);
 
-	so_signals[SELECTION_CHANGED] =
-		gtk_signal_new ("selection_changed",
-				GTK_RUN_FIRST,
-				object_class->type,		
-				GTK_SIGNAL_OFFSET (SheetItemClass, selection_changed),
-				gtk_marshal_NONE__INT,
-				GTK_TYPE_NONE, 1, GTK_TYPE_INT);
+	so_signals[SELECTION_CHANGED] = g_signal_new ("selection_changed",
+		G_TYPE_FROM_CLASS(object_class),
+		G_SIGNAL_RUN_FIRST,
+		G_STRUCT_OFFSET(SheetItemClass, selection_changed),
+		NULL, NULL,
+		g_cclosure_marshal_VOID__INT,
+		G_TYPE_NONE, 1, G_TYPE_INT);
 
-	so_signals[MOUSE_OVER] =
-		gtk_signal_new ("mouse_over",
-				GTK_RUN_FIRST,
-				object_class->type,
-				GTK_SIGNAL_OFFSET (SheetItemClass, mouse_over),
-				gtk_marshal_NONE__NONE,
-				GTK_TYPE_NONE, 0);
+	so_signals[MOUSE_OVER] = g_signal_new ("mouse_over",
+		G_TYPE_FROM_CLASS(object_class),
+		G_SIGNAL_RUN_FIRST,
+		G_STRUCT_OFFSET(SheetItemClass, mouse_over),
+		NULL, NULL,
+		g_cclosure_marshal_VOID__VOID,
+		G_TYPE_NONE, 0);
 
-	so_signals[DOUBLE_CLICKED] =
-		gtk_signal_new ("double_clicked",
-				GTK_RUN_FIRST,
-				object_class->type,
-				0,
-				gtk_marshal_NONE__NONE,
-				GTK_TYPE_NONE, 0);
-	
-	gtk_object_class_add_signals (object_class, so_signals, LAST_SIGNAL);
+	so_signals[DOUBLE_CLICKED] = g_signal_new ("double_clicked",
+		G_TYPE_FROM_CLASS(object_class),
+		G_SIGNAL_RUN_FIRST,
+		0, NULL, NULL,
+		g_cclosure_marshal_VOID__VOID,
+		G_TYPE_NONE, 0);
 }
 
 static void
@@ -170,25 +196,26 @@ sheet_item_init (SheetItem *item)
 }
 
 static void
-sheet_item_set_arg (GtkObject *object, GtkArg *arg, guint arg_id)
+sheet_item_set_property (GObject *object, guint prop_id, const GValue *value,
+						 GParamSpec *spec)
 {
 	SheetItem *sheet_item;
 	SheetPos pos;
 
 	sheet_item = SHEET_ITEM (object);
 
-	switch (arg_id) {
+	switch (prop_id) {
 	case ARG_DATA:
 		if (sheet_item->priv->data) {
 			g_warning ("Cannot set ItemData after creation.");
 			break;
 		}
-		sheet_item->priv->data = GTK_VALUE_POINTER (*arg);
-	        item_data_get_pos (sheet_item->priv->data, &pos);
+		sheet_item->priv->data = g_value_get_pointer (value);
+		item_data_get_pos (sheet_item->priv->data, &pos);
 		gnome_canvas_item_set (GNOME_CANVAS_ITEM (object),
-				       "x", pos.x,
-				       "y", pos.y,
-				       NULL);
+			"x", pos.x,
+			"y", pos.y,
+			NULL);
 		break;
 	default:
 		break;
@@ -196,21 +223,22 @@ sheet_item_set_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 }
 
 static void
-sheet_item_get_arg (GtkObject *object, GtkArg *arg, guint arg_id)
+sheet_item_get_property (GObject *object, guint prop_id, GValue *value,
+	GParamSpec *spec)
 {
 	SheetItem *sheet_item;
 
 	sheet_item = SHEET_ITEM (object);
 
-	switch (arg_id) {
+	switch (prop_id) {
 	case ARG_DATA:
-		GTK_VALUE_POINTER (*arg) = sheet_item->priv->data;
+		g_value_set_pointer(value, sheet_item->priv->data);
 		break;
 	case ARG_SHEET:
-		GTK_VALUE_POINTER (*arg) = sheet_item_get_sheet (sheet_item);
+		g_value_set_pointer (value, sheet_item_get_sheet (sheet_item));
 		break;
 	default:
-		arg->type = GTK_TYPE_INVALID;
+		G_OBJECT_WARN_INVALID_PROPERTY_ID(sheet_item, prop_id, spec);
 		break;
 	}
 }
@@ -224,11 +252,15 @@ sheet_item_destroy (GtkObject *object)
 	g_return_if_fail (IS_SHEET_ITEM (object));
 
 	sheet_item = SHEET_ITEM (object);
-	
-	gtk_object_unref (GTK_OBJECT (sheet_item->priv->data));
 
-	if (GTK_OBJECT_CLASS (sheet_item_parent_class)->destroy) {
-		(* GTK_OBJECT_CLASS (sheet_item_parent_class)->destroy) (object);
+	if (sheet_item->priv) {
+		g_object_unref(G_OBJECT(sheet_item->priv->data));
+		g_free(sheet_item->priv);
+		sheet_item->priv = NULL;
+	}
+
+	if (GTK_OBJECT_CLASS(sheet_item_parent_class)->destroy) {
+		GTK_OBJECT_CLASS(sheet_item_parent_class)->destroy(object);
 	}
 }
 
@@ -273,23 +305,29 @@ rotate_cmd (GtkWidget *widget, SchematicView *sv)
  * that are the same for all objects, e.g. parts and wires.
  */
 static const GnomeUIInfo object_popup_menu [] = {
-	{ GNOME_APP_UI_ITEM, N_("Copy"), N_("Copy the object to the clipboard"), copy_cmd, NULL, NULL,
-	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_COPY, 0, 0 },
-	{ GNOME_APP_UI_ITEM, N_("Cut"), N_("Cut the object to the clipboard"), cut_cmd, NULL, NULL,
-	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_CUT, 0, 0 },
-	{ GNOME_APP_UI_ITEM, N_("Delete"), N_("Delete the object"), delete_cmd, NULL, NULL,
-	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_TRASH, 0, 0 },
+	{ GNOME_APP_UI_ITEM, N_("Copy"), N_("Copy the object to the clipboard"),
+	  copy_cmd, NULL, NULL,
+	  GNOME_APP_PIXMAP_STOCK, GTK_STOCK_COPY, 0, 0 },
+	{ GNOME_APP_UI_ITEM, N_("Cut"), N_("Cut the object to the clipboard"),
+	  cut_cmd, NULL, NULL,
+	  GNOME_APP_PIXMAP_STOCK, GTK_STOCK_CUT, 0, 0 },
+	{ GNOME_APP_UI_ITEM, N_("Delete"), N_("Delete the object"),
+	  delete_cmd, NULL, NULL,
+	  GNOME_APP_PIXMAP_STOCK, GTK_STOCK_DELETE, 0, 0 },
 
 	GNOMEUIINFO_SEPARATOR,
-	
-	{ GNOME_APP_UI_ITEM, N_("Rotate"), N_("Rotate the selected objects 90 degrees clockwise"), rotate_cmd, NULL, NULL,
+
+	{ GNOME_APP_UI_ITEM, N_("Rotate"),
+	  N_("Rotate the selected objects 90 degrees clockwise"),
+	  rotate_cmd, NULL, NULL,
 	  GNOME_APP_PIXMAP_STOCK, STOCK_PIXMAP_ROTATE, 0, 0 },
 
 	GNOMEUIINFO_END
 };
 
 static void
-sheet_item_run_menu (SheetItem *item, SchematicView *sv, GdkEventButton *event, SheetItemMenu *context_menu)
+sheet_item_run_menu (SheetItem *item, SchematicView *sv, GdkEventButton *event,
+					 SheetItemMenu *context_menu)
 {
 	GtkWidget *menu;
 	int i, top_size, bottom_size;
@@ -299,7 +337,7 @@ sheet_item_run_menu (SheetItem *item, SchematicView *sv, GdkEventButton *event,
 	g_return_if_fail (IS_SHEET_ITEM (item));
 	g_return_if_fail (sv != NULL);
 	g_return_if_fail (IS_SCHEMATIC_VIEW (sv));
-	
+
 	/*
 	 * Calculate the number of menu items in the general top part of the
 	 * menu. Don't count the GNOMEUIINFO_END.
@@ -310,7 +348,7 @@ sheet_item_run_menu (SheetItem *item, SchematicView *sv, GdkEventButton *event,
 		bottom_size = context_menu->size;
 	else
 		bottom_size = 1;
-	
+
 	popup_menu = g_malloc0 (sizeof (GnomeUIInfo) * (top_size + bottom_size));
 
 	for (i = 0; i < top_size; i++)
@@ -320,14 +358,14 @@ sheet_item_run_menu (SheetItem *item, SchematicView *sv, GdkEventButton *event,
 		for (i = 0; i < bottom_size; i++)
 			popup_menu[top_size + i] = context_menu->menu[i];
 	} else {
-		/* 
+		/*
 		 * Fill in the uiinfo_end element.
 		 */
 		popup_menu[top_size] = object_popup_menu[top_size];
 	}
 
-	menu =  gnome_popup_menu_new (popup_menu);
-	gnome_popup_menu_do_popup_modal (menu, NULL, NULL, event, sv);
+	menu = GTK_WIDGET (gnome_popup_menu_new (popup_menu));
+	gnome_popup_menu_do_popup_modal (menu, NULL, NULL, event, sv, GTK_WIDGET(sv->toplevel));
 	g_free (popup_menu);
 }
 
@@ -337,7 +375,8 @@ scroll_timeout_cb (Sheet *sheet)
 	int width, height;
 	int x, y, dx = 0, dy = 0;
 
-	/* Get the current mouse position so that we can decide if the pointer is inside the viewport. */
+	/* Get the current mouse position so that we can decide if the pointer is
+	   inside the viewport. */
 	gdk_window_get_pointer (GTK_WIDGET (sheet)->window, &x, &y, NULL);
 
 	width = GTK_WIDGET (sheet)->allocation.width;
@@ -355,7 +394,7 @@ scroll_timeout_cb (Sheet *sheet)
 
 	if (!(x > 0 && x < width && y > 0 && y < height))
 		sheet_scroll (sheet, dx*5, dy*5);
-	
+
 	return TRUE;
 }
 
@@ -373,23 +412,17 @@ sheet_item_event (SheetItem *sheet_item, const GdkEvent *event, SchematicView *s
 	Sheet *sheet;
 	SheetPriv *priv;
 	GList *list;
-	GtkArg arg[2];
 	SheetPos delta;
-
 	/* Remember the last position of the mouse cursor. */
 	static double last_x, last_y;
-
 	/* Mouse cursor position in window coordinates, snapped to the grid spacing. */
 	double snapped_x, snapped_y;
-
 	/* Move the selected item(s) by this movement. */
 	double dx, dy;
-
 	/* The selected group's bounding box in window resp. canvas coordinates. */
 	double x1, y1, x2, y2;
 	static double bb_x1, bb_y1, bb_x2, bb_y2;
 	int cx1, cy1, cx2, cy2;
-
 	/* The sheet's width and the its viewport's width. */
 	int sheet_width, sheet_height;
 
@@ -408,22 +441,23 @@ sheet_item_event (SheetItem *sheet_item, const GdkEvent *event, SchematicView *s
 		 * Debugging...
 		 */
 		if (event->crossing.state & GDK_CONTROL_MASK)
-			gtk_signal_emit_by_name (GTK_OBJECT (sheet_item), "mouse_over");
+			g_signal_emit_by_name (G_OBJECT (sheet_item), "mouse_over");
 		return TRUE;
 
 	case GDK_BUTTON_PRESS:
+		/* Grab focus to sheet for correct use of events */
+		gtk_widget_grab_focus (GTK_WIDGET (sheet));
 		switch (event->button.button){
 		case 1:
-			gtk_signal_emit_stop_by_name (GTK_OBJECT (sheet), "event");
+			g_signal_stop_emission_by_name (G_OBJECT (sheet), "event");
 			sheet->state = SHEET_STATE_DRAG_START;
 
 			last_x = event->button.x;
 			last_y = event->button.y;
-			snap_to_grid (sheet->grid, &last_x, &last_y); 
+			snap_to_grid (sheet->grid, &last_x, &last_y);
 			break;
-
 		case 3:
-			gtk_signal_emit_stop_by_name (GTK_OBJECT (sheet), "event");
+			g_signal_stop_emission_by_name (G_OBJECT (sheet), "event");
 
 			if (sheet->state != SHEET_STATE_NONE)
 				return TRUE;
@@ -431,17 +465,18 @@ sheet_item_event (SheetItem *sheet_item, const GdkEvent *event, SchematicView *s
 			/*
 			 * Bring up a context menu for right button clicks.
 			 */
-			if (!sheet_item->priv->selected && !((event->button.state & GDK_SHIFT_MASK) == GDK_SHIFT_MASK))
+			if (!sheet_item->priv->selected &&
+				!((event->button.state & GDK_SHIFT_MASK) == GDK_SHIFT_MASK))
 				schematic_view_select_all (sv, FALSE);
 
 //			if (!sheet_item->priv->selected)
 				sheet_item_select (sheet_item, TRUE);
 
-			class = SHEET_ITEM_CLASS (GTK_OBJECT(sheet_item)->klass);
+			class = SHEET_ITEM_CLASS (GTK_OBJECT_GET_CLASS(sheet_item));
 			sheet_item_run_menu (
 				sheet_item,
 				sv,
-				(GdkEventButton *) event, 
+				(GdkEventButton *) event,
 				class->context_menu);
 			break;
 		default:
@@ -451,7 +486,7 @@ sheet_item_event (SheetItem *sheet_item, const GdkEvent *event, SchematicView *s
 
 	case GDK_2BUTTON_PRESS:
 		/*
-		 * Do not interfere with object dragging. 
+		 * Do not interfere with object dragging.
 		 */
 		if (sheet->state == SHEET_STATE_DRAG)
 			return FALSE;
@@ -460,8 +495,8 @@ sheet_item_event (SheetItem *sheet_item, const GdkEvent *event, SchematicView *s
 		case 1:
 			if (sheet->state == SHEET_STATE_DRAG_START)
 				sheet->state = SHEET_STATE_NONE;
-			gtk_signal_emit_stop_by_name (GTK_OBJECT (sheet), "event");
-			gtk_signal_emit_by_name (GTK_OBJECT (sheet_item), "double_clicked");
+			g_signal_stop_emission_by_name (G_OBJECT (sheet), "event");
+			g_signal_emit_by_name (G_OBJECT (sheet_item), "double_clicked");
 			break;
 
 		default:
@@ -470,52 +505,55 @@ sheet_item_event (SheetItem *sheet_item, const GdkEvent *event, SchematicView *s
 		break;
 
 	case GDK_3BUTTON_PRESS:
-		gtk_signal_emit_stop_by_name (GTK_OBJECT (sheet), "event");
+		g_signal_stop_emission_by_name (G_OBJECT (sheet), "event");
 		return TRUE;
 
 	case GDK_BUTTON_RELEASE:
 		switch (event->button.button){
 		case 1:
-			if (sheet->state != SHEET_STATE_DRAG && sheet->state != SHEET_STATE_DRAG_START)
+			if (sheet->state != SHEET_STATE_DRAG &&
+				sheet->state != SHEET_STATE_DRAG_START)
 				return TRUE;
 
-			gtk_signal_emit_stop_by_name (GTK_OBJECT (sheet), "event");
+			g_signal_stop_emission_by_name (G_OBJECT (sheet), "event");
 
 			if (sheet->state == SHEET_STATE_DRAG_START) {
 				sheet->state = SHEET_STATE_NONE;
-				
+
 				if (!(event->button.state & GDK_SHIFT_MASK))
 					schematic_view_select_all (sv, FALSE);
-				
+
 				sheet_item_select (sheet_item, TRUE);
 
 				return TRUE;
 			}
 
-			arg[0].name = "x";
-			arg[1].name = "y";
-			gtk_object_getv (GTK_OBJECT (priv->selected_group), 2, arg);
-			delta.x = GTK_VALUE_DOUBLE (arg[0]);
-			delta.y = GTK_VALUE_DOUBLE (arg[1]);
+			g_object_get (G_OBJECT (priv->selected_group),
+			  "x", &delta.x,
+			  "y", &delta.y,
+			  NULL);
+
+			gtk_timeout_remove (priv->scroll_timeout_id); // Esto no esta bien.
 
-			gtk_timeout_remove (priv->scroll_timeout_id);
-			
 			sheet->state = SHEET_STATE_NONE;
-			gnome_canvas_item_ungrab (GNOME_CANVAS_ITEM (sheet_item), 
-						  event->button.time);
+			gnome_canvas_item_ungrab (GNOME_CANVAS_ITEM (sheet_item), event->button.time);
+
 
-			
 			/*
 			 * HACK :(
 			 * FIXME: fix this later. The problem is that we don't want to
 			 * update the current view, since it acts as the controller and
 			 * already is updated. It's not really a problem, but an ugly hack.
 			 */
-			gnome_canvas_item_move (GNOME_CANVAS_ITEM (priv->selected_group), -delta.x, -delta.y);
+			gnome_canvas_item_move (
+				GNOME_CANVAS_ITEM (priv->selected_group),
+				-delta.x, -delta.y
+			);
 
 			/*
-			 * Make sure the objects are reparented back to the object group _before_
-			 * we register them. Otherwise they will get incorrect coordinates.
+			 * Make sure the objects are reparented back to the object
+			 * group _before_ we register them. Otherwise they will get
+			 * incorrect coordinates.
 			 */
 			for (list = priv->selected_objects; list; list = list->next) {
 				sheet_item_reparent (SHEET_ITEM (list->data), sheet->object_group);
@@ -535,58 +573,99 @@ sheet_item_event (SheetItem *sheet_item, const GdkEvent *event, SchematicView *s
 			}
 
 			break;
+	case GDK_KEY_PRESS:
+		switch (event->key.keyval) {
+		case GDK_r:
+			schematic_view_rotate_selection (sv);
+			{
+				int x, y;
+
+				gdk_window_get_pointer (GDK_WINDOW (GTK_WIDGET (sheet)->window), &x, &y, NULL);
+				gnome_canvas_window_to_world (GNOME_CANVAS (sheet), x, y, &snapped_x, &snapped_y);
+
+				/*
+				 * Center the objects around the mouse pointer.
+				 */
+				gnome_canvas_item_get_bounds (
+					GNOME_CANVAS_ITEM (priv->floating_group),
+					&x1, &y1, &x2, &y2
+				);
+
+				snap_to_grid (sheet->grid, &snapped_x, &snapped_y);
+
+				dx = snapped_x - (x1 + (x2 - x1) / 2);
+				dy = snapped_y - (y1 + (y2 - y1) / 2);
+				snap_to_grid (sheet->grid, &dx, &dy);
+
+				gnome_canvas_item_move (
+					GNOME_CANVAS_ITEM (priv->floating_group),
+					dx, dy
+				);
+
+				last_x = snapped_x;
+				last_y = snapped_y;
+			}
+			break;
+		default:
+			return FALSE;
+		}
+		return TRUE;
+
 		default:
 			return FALSE;
 		}
 		break;
 
 	case GDK_MOTION_NOTIFY:
-		if (sheet->state != SHEET_STATE_DRAG && sheet->state != SHEET_STATE_DRAG_START)
+		if (sheet->state != SHEET_STATE_DRAG &&
+			sheet->state != SHEET_STATE_DRAG_START)
 			return FALSE;
 
 		if (sheet->state == SHEET_STATE_DRAG_START) {
 			sheet->state = SHEET_STATE_DRAG;
 
-			/* 
+			/*
 			 * Update the selection if needed.
-			 */ 
+			 */
 			if (!sheet_item->priv->selected){
 				if (!(event->motion.state & GDK_SHIFT_MASK))
 					schematic_view_select_all (sv, FALSE);
 				sheet_item_select (sheet_item, TRUE);
 			}
 
-			gnome_canvas_item_set (GNOME_CANVAS_ITEM (priv->selected_group), 
-					       "x", 0.0, "y", 0.0, NULL);
+			gnome_canvas_item_set (
+				GNOME_CANVAS_ITEM (priv->selected_group),
+				"x", 0.0, "y", 0.0, NULL
+			);
 
 			/*
-			 * Reparent the selected objects so that we can 
+			 * Reparent the selected objects so that we can
 			 * move them efficiently.
 			 */
 			for (list = priv->selected_objects; list; list = list->next){
 				sheet_item_reparent (SHEET_ITEM (list->data), priv->selected_group);
 			}
-			
+
 			gnome_canvas_item_grab (GNOME_CANVAS_ITEM (sheet_item),
-						GDK_POINTER_MOTION_MASK |
-						GDK_BUTTON_RELEASE_MASK,
-						NULL, event->button.time);
-			
-			gnome_canvas_item_get_bounds (GNOME_CANVAS_ITEM (priv->selected_group), 
-						      &bb_x1, &bb_y1, &bb_x2, &bb_y2);
+				GDK_POINTER_MOTION_MASK |
+				GDK_BUTTON_RELEASE_MASK,
+				NULL, event->button.time);
+
+			gnome_canvas_item_get_bounds (
+				GNOME_CANVAS_ITEM (priv->selected_group),
+				&bb_x1, &bb_y1, &bb_x2, &bb_y2
+			);
 
 			/*
 			 * Start the autoscroll timeout.
 			 */
-			priv->scroll_timeout_id = gtk_timeout_add (
-				50,
-				(void *) scroll_timeout_cb,
-				sheet);
+			priv->scroll_timeout_id =
+				g_timeout_add (50, (void *) scroll_timeout_cb, sheet);
 		}
 
 		snapped_x = event->motion.x;
 		snapped_y = event->motion.y;
-		snap_to_grid (sheet->grid, &snapped_x, &snapped_y); 
+		snap_to_grid (sheet->grid, &snapped_x, &snapped_y);
 
 		dx = snapped_x - last_x;
 		dy = snapped_y - last_y;
@@ -595,10 +674,10 @@ sheet_item_event (SheetItem *sheet_item, const GdkEvent *event, SchematicView *s
 		x2 = bb_x2 + dx;
 		y1 = bb_y1 + dy;
 		y2 = bb_y2 + dy;
-		
+
 		gnome_canvas_w2c (canvas, x1, y1, &cx1, &cy1);
 		gnome_canvas_w2c (canvas, x2, y2, &cx2,	&cy2);
-		sheet_get_size_pixels (sheet, &sheet_width, &sheet_height); 		
+		sheet_get_size_pixels (sheet, &sheet_width, &sheet_height);
 
 		/* Check that we don't move outside the sheet... horizontally: */
 		if (cx1 <= 0){  /* leftmost edge */
@@ -610,16 +689,16 @@ sheet_item_event (SheetItem *sheet_item, const GdkEvent *event, SchematicView *s
 			snap_to_grid (sheet->grid, &dx, NULL);
 			snapped_x = last_x + dx;
 		}
-		
+
 		/* And vertically: */
 		if (cy1 <= 0){  /* upper edge */
 			dy = dy - y1;
 			snap_to_grid (sheet->grid, NULL, &dy);
-			snapped_y = last_y + dy; 
+			snapped_y = last_y + dy;
 		} else if (cy2 >= sheet_height){  /* lower edge */
 			dy = dy - (y2 - sheet_height / priv->zoom);
 			snap_to_grid (sheet->grid, NULL, &dy);
-			snapped_y = last_y + dy; 
+			snapped_y = last_y + dy;
 		}
 
 		last_x = snapped_x;
@@ -633,8 +712,9 @@ sheet_item_event (SheetItem *sheet_item, const GdkEvent *event, SchematicView *s
 		bb_x2 += dx;
 		bb_y1 += dy;
 		bb_y2 += dy;
-		
+
 		break;
+
 	default:
 		return FALSE;
 	}
@@ -658,11 +738,12 @@ sheet_item_cancel_floating (SchematicView *sv)
 	group = sheet->priv->floating_group;
 	if (group == NULL)
 		return;
-	
+
 	if (sheet->state != SHEET_STATE_FLOAT && sheet->state != SHEET_STATE_FLOAT_START)
 		return;
 
-	gtk_signal_disconnect (GTK_OBJECT (sheet), sheet->priv->float_handler_id);
+	g_signal_handler_disconnect(G_OBJECT (sheet), sheet->priv->float_handler_id);
+
 	gtk_object_destroy (GTK_OBJECT (group));
 
 	/*
@@ -675,12 +756,14 @@ sheet_item_cancel_floating (SchematicView *sv)
 		}
 	}
 
-	sheet->priv->floating_group = GNOME_CANVAS_GROUP (gnome_canvas_item_new (
-		sheet->object_group,
-		gnome_canvas_group_get_type (),
-		"x", 0.0,
-		"y", 0.0,
-		NULL));
+	sheet->priv->floating_group = GNOME_CANVAS_GROUP (
+		gnome_canvas_item_new (
+			sheet->object_group,
+			gnome_canvas_group_get_type (),
+			"x", 0.0,
+			"y", 0.0,
+			NULL)
+		);
 
 	sheet->priv->float_handler_id = 0;
 	sheet->state = SHEET_STATE_NONE;
@@ -696,13 +779,14 @@ sheet_item_floating_event (Sheet *sheet, const GdkEvent *event, SchematicView *s
 	GnomeCanvas *canvas;
 	SheetPriv *priv;
 	GList *list;
-	static SheetPos delta;
-	GtkArg arg[2];
+	static SheetPos delta, tmp;
+//	GtkArg arg[2];
 
 	/* Remember the last position of the mouse cursor. */
 	static double last_x, last_y;
 
-	/* Mouse cursor position in window coordinates, snapped to the grid spacing. */
+	/* Mouse cursor position in window coordinates, snapped to the grid
+	   spacing. */
 	double snapped_x, snapped_y;
 
 	/* Move the selected item(s) by this movement. */
@@ -722,15 +806,15 @@ sheet_item_floating_event (Sheet *sheet, const GdkEvent *event, SchematicView *s
 	g_return_val_if_fail (IS_SHEET (sheet), FALSE);
 	g_return_val_if_fail (schematic_view != NULL, FALSE);
 	g_return_val_if_fail (IS_SCHEMATIC_VIEW (schematic_view), FALSE);
-
-	g_return_val_if_fail (sheet->priv->floating_objects != NULL, FALSE); // assert?
+	/* assert? */
+	g_return_val_if_fail (sheet->priv->floating_objects != NULL, FALSE);
 
 	priv = sheet->priv;
 	canvas = GNOME_CANVAS (sheet);
-		
+
 	switch (event->type) {
 	case GDK_BUTTON_RELEASE:
-		gtk_signal_emit_stop_by_name (GTK_OBJECT (sheet), "event");
+		g_signal_stop_emission_by_name (G_OBJECT (sheet), "event");
 		break;
 
 	case GDK_BUTTON_PRESS:
@@ -742,38 +826,41 @@ sheet_item_floating_event (Sheet *sheet, const GdkEvent *event, SchematicView *s
 		case 4:
 		case 5:
 			return FALSE;
-			
+
 		case 1:
 			/* Stick the part. */
 			sheet->state = SHEET_STATE_NONE;
-			gtk_signal_emit_stop_by_name (GTK_OBJECT (sheet), "event");
-			gtk_signal_disconnect (GTK_OBJECT (sheet), sheet->priv->float_handler_id);
+			g_signal_stop_emission_by_name (G_OBJECT (sheet), "event");
+			g_signal_handler_disconnect ( G_OBJECT (sheet), sheet->priv->float_handler_id);
 			sheet->priv->float_handler_id = 0;
 
-			arg[0].name = "x";
-			arg[1].name = "y";
-			gtk_object_getv (GTK_OBJECT (sheet->priv->floating_group), 2, arg);
-			delta.x = GTK_VALUE_DOUBLE (arg[0]) - delta.x;
-			delta.y = GTK_VALUE_DOUBLE (arg[1]) - delta.y;
-			
+			g_object_get (G_OBJECT (sheet->priv->floating_group),
+				"x", &tmp.x,
+				"y", &tmp.y,
+				NULL);
+
+			delta.x = tmp.x - delta.x;
+			delta.y = tmp.y - delta.y;
+
 			for (list = priv->floating_objects; list; list = list->next) {
 				SheetItem *floating_item;
 				ItemData *floating_data;
 				/*
 				 * Destroy the ghost item and place a real item.
-				 */ 
+				 */
 				floating_item = list->data;
-				floating_data = sheet_item_get_data (floating_item);				
-				
-//				gtk_object_ref (GTK_OBJECT (floating_data));
+				floating_data = sheet_item_get_data (floating_item);
+
+				g_object_ref (G_OBJECT (floating_data));
 
 				item_data_move (floating_data, &delta);
-				schematic_add_item (schematic_view_get_schematic (schematic_view), 
-						    floating_data);
+				schematic_add_item (schematic_view_get_schematic (schematic_view),
+									floating_data);
 
-				gtk_object_destroy (GTK_OBJECT (floating_item));
+				/* It must be a destroy, because the father uses Destroy */
+				gtk_object_destroy (GTK_OBJECT(floating_item));
 			}
-			
+
 			g_list_free (sheet->priv->floating_objects);
 			sheet->priv->floating_objects = NULL;
 			break;
@@ -782,53 +869,56 @@ sheet_item_floating_event (Sheet *sheet, const GdkEvent *event, SchematicView *s
 			/*
 			 * Cancel the "float-placement" for button-3 clicks.
 			 */
-			gtk_signal_emit_stop_by_name (GTK_OBJECT (sheet), "event");
+			g_signal_stop_emission_by_name (G_OBJECT (sheet), "event");
 			sheet_item_cancel_floating (schematic_view);
 			break;
 		}
 		break;
-	
+
 	case GDK_2BUTTON_PRESS:
 	case GDK_3BUTTON_PRESS:
-		gtk_signal_emit_stop_by_name (GTK_OBJECT (sheet), "event");
+		g_signal_stop_emission_by_name (G_OBJECT (sheet), "event");
 		return TRUE;
-		
+
 	case GDK_MOTION_NOTIFY:
-		if (sheet->state != SHEET_STATE_FLOAT && sheet->state != SHEET_STATE_FLOAT_START)
+		if (sheet->state != SHEET_STATE_FLOAT &&
+			sheet->state != SHEET_STATE_FLOAT_START)
 			return FALSE;
-		
-		gtk_signal_emit_stop_by_name (GTK_OBJECT (sheet), "event");
-			
+
+		g_signal_stop_emission_by_name (G_OBJECT (sheet), "event");
+
 		if (sheet->state == SHEET_STATE_FLOAT_START) {
 			sheet->state = SHEET_STATE_FLOAT;
 
 			/*
-			 * Reparent the selected objects so that we can 
+			 * Reparent the selected objects so that we can
 			 * move them efficiently.
 			 */
 			for (list = priv->floating_objects; list; list = list->next) {
-			        sheet_item_reparent (SHEET_ITEM (list->data), priv->floating_group);
+				sheet_item_reparent (SHEET_ITEM (list->data), priv->floating_group);
 			}
 
-			arg[0].name = "x";
-			arg[1].name = "y";
-			gtk_object_getv (GTK_OBJECT (sheet->priv->floating_group), 2, arg);
-			delta.x = GTK_VALUE_DOUBLE (arg[0]);
-			delta.y = GTK_VALUE_DOUBLE (arg[1]);
+			gtk_object_get (GTK_OBJECT (sheet->priv->floating_group),
+					"x", &delta.x,
+					"y", &delta.y,
+					NULL);
 
 			/*
 			 * Center the objects around the mouse pointer.
 			 */
-			gnome_canvas_item_get_bounds (GNOME_CANVAS_ITEM (priv->floating_group), &x1, &y1, &x2, &y2);
-			
-			gnome_canvas_window_to_world (canvas, event->motion.x, event->motion.y, &snapped_x, &snapped_y);
-			snap_to_grid (sheet->grid, &snapped_x, &snapped_y); 
-			
+			gnome_canvas_item_get_bounds (GNOME_CANVAS_ITEM (priv->floating_group),
+										  &x1, &y1, &x2, &y2);
+
+			gnome_canvas_window_to_world (canvas, event->motion.x, event->motion.y,
+										  &snapped_x, &snapped_y);
+			snap_to_grid (sheet->grid, &snapped_x, &snapped_y);
+
 			dx = snapped_x - (x1 + (x2 - x1) / 2);
 			dy = snapped_y - (y1 + (y2 - y1) / 2);
-			snap_to_grid (sheet->grid, &dx, &dy); 
-			
-			gnome_canvas_item_move (GNOME_CANVAS_ITEM (priv->floating_group), dx, dy);
+			snap_to_grid (sheet->grid, &dx, &dy);
+
+			gnome_canvas_item_move (GNOME_CANVAS_ITEM (priv->floating_group),
+									dx, dy);
 
 			x1 += dx;
 			y1 += dy;
@@ -840,41 +930,44 @@ sheet_item_floating_event (Sheet *sheet, const GdkEvent *event, SchematicView *s
 
 			return TRUE;
 		}
-		
+
 		/*
-		 * We have to convert from window to world coordinates here, since we get
-		 * the coordinates relative the sheet and not the item like we do in
-		 * sheet_item_event ().
+		 * We have to convert from window to world coordinates here,
+		 * since we get the coordinates relative the sheet and not the
+		 * item like we do in sheet_item_event ().
 		 */
-		gnome_canvas_window_to_world (canvas, event->motion.x, event->motion.y, &snapped_x, &snapped_y);
-		
-		snap_to_grid (sheet->grid, &snapped_x, &snapped_y); 
-		
+		gnome_canvas_window_to_world (canvas, event->motion.x, event->motion.y,
+									  &snapped_x, &snapped_y);
+
+		snap_to_grid (sheet->grid, &snapped_x, &snapped_y);
+
 		/* Calculate which amount to move the selected objects by. */
 		dx = snapped_x - last_x;
 		dy = snapped_y - last_y;
-		
-		/* We need the bounding box to check that we don't move anything off-sheet. */
-		gnome_canvas_item_get_bounds (GNOME_CANVAS_ITEM (priv->floating_group), &x1, &y1, &x2, &y2);
-		
+
+		/* We need the bounding box to check that we don't move anything
+		   off-sheet. */
+		gnome_canvas_item_get_bounds (GNOME_CANVAS_ITEM (priv->floating_group),
+									  &x1, &y1, &x2, &y2);
+
 		x1 += dx;
 		x2 += dx;
 		y1 += dy;
 		y2 += dy;
-		
+
 		gnome_canvas_get_scroll_offsets (canvas, &offset_x, &offset_y);
 		gnome_canvas_w2c (canvas, x1, y1, &cx1, &cy1);
 		gnome_canvas_w2c (canvas, x2, y2, &cx2,	&cy2);
-		sheet_get_size_pixels (sheet, &sheet_width, &sheet_height); 		
+		sheet_get_size_pixels (sheet, &sheet_width, &sheet_height);
 
 		/* Check that we don't move outside the sheet horizontally */
 		if (cx1 <= 0){  /* leftmost edge */
 			dx = dx - x1;
-			snap_to_grid (sheet->grid, &dx, NULL); 
+			snap_to_grid (sheet->grid, &dx, NULL);
 			snapped_x = last_x + dx;
 		} else if (cx2 >= sheet_width){  /* rightmost edge */
 			dx = dx - (x2 - sheet_width/sheet->priv->zoom);
-			snap_to_grid (sheet->grid, &dx, NULL); 
+			snap_to_grid (sheet->grid, &dx, NULL);
 			snapped_x = last_x + dx;
 		}
 
@@ -882,18 +975,19 @@ sheet_item_floating_event (Sheet *sheet, const GdkEvent *event, SchematicView *s
 		if (cy1 <= 0){  /* upper edge */
 			dy = dy - y1;
 			snap_to_grid (sheet->grid, NULL, &dy);
-			snapped_y = last_y + dy; 
+			snapped_y = last_y + dy;
 		} else if (cy2 >= sheet_height){  /* lower edge */
 			dy = dy - (y2 - sheet_height/sheet->priv->zoom);
 			snap_to_grid (sheet->grid, NULL, &dy);
-			snapped_y = last_y + dy; 
+			snapped_y = last_y + dy;
 		}
 
 		last_x = snapped_x;
 		last_y = snapped_y;
 
 		if (dx != 0 || dy != 0){
-			gnome_canvas_item_move (GNOME_CANVAS_ITEM (priv->floating_group), dx, dy);
+			gnome_canvas_item_move (GNOME_CANVAS_ITEM (priv->floating_group),
+									dx, dy);
 		}
 		break;
 
@@ -901,30 +995,32 @@ sheet_item_floating_event (Sheet *sheet, const GdkEvent *event, SchematicView *s
 		switch (event->key.keyval) {
 		case GDK_r:
 			schematic_view_rotate_ghosts (schematic_view);
-
 			{
 				int x, y;
 
-				gdk_window_get_pointer (GTK_WIDGET (sheet)->window, &x, &y, NULL);
-				gnome_canvas_window_to_world (GNOME_CANVAS (sheet), x, y, &snapped_x, &snapped_y);
+				gdk_window_get_pointer (GDK_WINDOW (GTK_WIDGET (sheet)->window),
+										&x, &y, NULL);
+				gnome_canvas_window_to_world (GNOME_CANVAS (sheet), x, y,
+											  &snapped_x, &snapped_y);
 
 				/*
 				 * Center the objects around the mouse pointer.
 				 */
-				gnome_canvas_item_get_bounds (GNOME_CANVAS_ITEM (priv->floating_group), &x1, &y1, &x2, &y2);
-				
-				snap_to_grid (sheet->grid, &snapped_x, &snapped_y); 
-				
+				gnome_canvas_item_get_bounds (GNOME_CANVAS_ITEM (priv->floating_group),
+											  &x1, &y1, &x2, &y2);
+
+				snap_to_grid (sheet->grid, &snapped_x, &snapped_y);
+
 				dx = snapped_x - (x1 + (x2 - x1) / 2);
 				dy = snapped_y - (y1 + (y2 - y1) / 2);
-				snap_to_grid (sheet->grid, &dx, &dy); 
-				
-				gnome_canvas_item_move (GNOME_CANVAS_ITEM (priv->floating_group), dx, dy);
-				
+				snap_to_grid (sheet->grid, &dx, &dy);
+
+				gnome_canvas_item_move (GNOME_CANVAS_ITEM (priv->floating_group),
+										dx, dy);
+
 				last_x = snapped_x;
 				last_y = snapped_y;
 			}
-
 			break;
 		default:
 			return FALSE;
@@ -943,11 +1039,12 @@ sheet_item_select (SheetItem *item, gboolean select)
 	g_return_val_if_fail (item != NULL, FALSE);
 	g_return_val_if_fail (IS_SHEET_ITEM (item), FALSE);
 
-	if ((item->priv->selected && select) || (!item->priv->selected && !select)) {
+	if ((item->priv->selected && select) ||
+		(!item->priv->selected && !select)) {
 		return FALSE;
 	}
 
-	gtk_signal_emit_by_name (GTK_OBJECT (item), "selection_changed", select);	
+	g_signal_emit_by_name (G_OBJECT (item), "selection_changed", select);
 	item->priv->selected = select;
 
 	return TRUE;
@@ -964,12 +1061,13 @@ sheet_item_select_in_area (SheetItem *item, SheetPos *p1, SheetPos *p2)
 	g_return_if_fail (p1 != NULL);
 	g_return_if_fail (p2 != NULL);
 
-	si_class = SHEET_ITEM_CLASS (GTK_OBJECT (item)->klass);
+	si_class = SHEET_ITEM_CLASS (GTK_OBJECT_GET_CLASS (item));
 	in_area = si_class->is_in_area (item, p1, p2);
-	
+
 	if (in_area && !item->priv->selected)
 		sheet_item_select (item, TRUE);
-	else if (!in_area && item->priv->selected && !item->priv->preserve_selection)
+	else if (!in_area && item->priv->selected &&
+			 !item->priv->preserve_selection)
 		sheet_item_select (item, FALSE);
 }
 
@@ -981,30 +1079,30 @@ sheet_item_select_in_area (SheetItem *item, SheetPos *p1, SheetPos *p2)
 void
 sheet_item_reparent (SheetItem *object, GnomeCanvasGroup *group)
 {
-	GtkArg arg[2];
 	double x1, y1, x2, y2;
-	
+
 	g_return_if_fail (object != NULL);
 	g_return_if_fail (IS_SHEET_ITEM (object));
 	g_return_if_fail (group != NULL);
 	g_return_if_fail (GNOME_IS_CANVAS_GROUP (group));
 
-	arg[0].name = "x";
-	arg[1].name = "y";
-	gtk_object_getv (GTK_OBJECT (object), 2, arg);
-	x1 = GTK_VALUE_DOUBLE (arg[0]);
-	y1 = GTK_VALUE_DOUBLE (arg[1]);
+	g_object_get (G_OBJECT (object),
+				  "x", &x1,
+				  "y", &y1,
+				  NULL);
+
 	gnome_canvas_item_i2w (GNOME_CANVAS_ITEM (object), &x1, &y1);
 
 	gnome_canvas_item_reparent (GNOME_CANVAS_ITEM (object), group);
-	arg[0].name = "x";
-	arg[1].name = "y";
-	gtk_object_getv (GTK_OBJECT (object), 2, arg);
-	x2 = GTK_VALUE_DOUBLE (arg[0]);
-	y2 = GTK_VALUE_DOUBLE (arg[1]);
+
+	g_object_get (G_OBJECT (object),
+				  "x", &x2,
+				  "y", &y2,
+				  NULL);
+
 	gnome_canvas_item_i2w (GNOME_CANVAS_ITEM (object), &x2, &y2);
-	
-	gnome_canvas_item_move (GNOME_CANVAS_ITEM (object), x1 - x2, y1 - y2); 
+
+	gnome_canvas_item_move (GNOME_CANVAS_ITEM (object), x1 - x2, y1 - y2);
 
 	/* This is needed because of a bug (?) in gnome-libs. */
 	gnome_canvas_item_request_update (GNOME_CANVAS_ITEM (object));
@@ -1018,7 +1116,7 @@ sheet_item_edit_properties (SheetItem *item)
 	g_return_if_fail (item != NULL);
 	g_return_if_fail (IS_SHEET_ITEM (item));
 
-	si_class = SHEET_ITEM_CLASS (GTK_OBJECT (item)->klass);
+	si_class = SHEET_ITEM_CLASS (GTK_OBJECT_GET_CLASS (item));
 
 	if (si_class->edit_properties)
 		si_class->edit_properties (item);
@@ -1045,8 +1143,8 @@ sheet_item_paste (SchematicView *schematic_view, ClipboardData *data)
 	g_return_if_fail (data != NULL);
 
 	item_data = ITEM_DATA (clipboard_data_get_item_data (data));
-	
-	id_class = ITEM_DATA_CLASS (GTK_OBJECT (item_data)->klass);
+
+	id_class = ITEM_DATA_CLASS (G_OBJECT_GET_CLASS (item_data));
 	if (id_class->clone == NULL)
 		return;
 
@@ -1113,7 +1211,7 @@ sheet_item_place (SheetItem *item, SchematicView *sv)
 	g_return_if_fail (item != NULL);
 	g_return_if_fail (IS_SHEET_ITEM (item));
 
-	si_class = SHEET_ITEM_CLASS (GTK_OBJECT (item)->klass);
+	si_class = SHEET_ITEM_CLASS (GTK_OBJECT_GET_CLASS (item));
 
 	if (si_class->place)
 		si_class->place (item, sv);
@@ -1127,7 +1225,7 @@ sheet_item_place_ghost (SheetItem *item, SchematicView *sv)
 	g_return_if_fail (item != NULL);
 	g_return_if_fail (IS_SHEET_ITEM (item));
 
-	si_class = SHEET_ITEM_CLASS (GTK_OBJECT (item)->klass);
+	si_class = SHEET_ITEM_CLASS (GTK_OBJECT_GET_CLASS (item));
 
 	if (si_class->place_ghost)
 		si_class->place_ghost (item, sv);
