@@ -1,20 +1,23 @@
 /*
  * textbox-item.c
  *
- * 
- * Author: 
+ *
+ * Author:
  *  Richard Hult <rhult@hem.passagen.se>
- * 
- *  http://www.dtek.chalmers.se/~d4hult/oregano/ 
- * 
- * Copyright (C) 1999,2000  Richard Hult 
- * 
- * This program is free software; you can redistribute it and/or 
- * modify it under the terms of the GNU General Public License as 
- * published by the Free Software Foundation; either version 2 of the 
- * License, or (at your option) any later version. 
- * 
- * This program is distributed in the hope that it will be useful, 
+ *  Ricardo Markiewicz <rmarkie@fi.uba.ar>
+ *  Andres de Barbara <adebarbara@fi.uba.ar>
+ *
+ * Web page: http://arrakis.lug.fi.uba.ar/
+ *
+ * Copyright (C) 1999-2001  Richard Hult
+ * Copyright (C) 2003,2004  LUGFI
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
@@ -24,7 +27,7 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
-/* 
+/*
  * TextboxItem object: the graphical representation of a textbox.
  *
  * Author:
@@ -47,7 +50,8 @@
 #define NORMAL_COLOR "black"
 #define SELECTED_COLOR "red"
 #define HIGHLIGHT_COLOR "green"
-#define TEXTBOX_FONT "-*-helvetica-medium-r-*-*-*-100-*-*-*-*-*-*"
+//#define TEXTBOX_FONT "-*-helvetica-medium-r-*-*-*-100-*-*-*-*-*-*"
+#define TEXTBOX_FONT "Arial 10"
 
 static void textbox_item_class_init (TextboxItemClass *klass);
 static void textbox_item_init (TextboxItem *item);
@@ -56,18 +60,25 @@ static void textbox_item_get_arg (GtkObject *object, GtkArg *arg, guint arg_id);
 static void textbox_item_destroy (GtkObject *object);
 static void textbox_item_moved (SheetItem *object);
 
-static void textbox_rotated_callback (ItemData *data, int angle, SheetItem *sheet_item);
-static void textbox_flipped_callback (ItemData *data, gboolean horizontal, SheetItem *sheet_item);
-static void textbox_moved_callback (ItemData *data, SheetPos *pos, SheetItem *item);
-static void textbox_text_changed_callback (ItemData *data, gchar *new_text, SheetItem *item);
-static void textbox_font_changed_callback (ItemData *data, gchar *new_font, SheetItem *item);
+static void textbox_rotated_callback (ItemData *data, int angle,
+	SheetItem *sheet_item);
+static void textbox_flipped_callback (ItemData *data, gboolean horizontal,
+	SheetItem *sheet_item);
+static void textbox_moved_callback (ItemData *data, SheetPos *pos,
+	SheetItem *item);
+static void textbox_text_changed_callback (ItemData *data, gchar *new_text,
+	SheetItem *item);
+static void textbox_font_changed_callback (ItemData *data, gchar *new_font,
+	SheetItem *item);
 
 static void textbox_item_paste (SchematicView *sv, ItemData *data);
-static void selection_changed (TextboxItem *item, gboolean select, gpointer user_data);
+static void selection_changed (TextboxItem *item, gboolean select,
+	gpointer user_data);
 static int  select_idle_callback (TextboxItem *item);
 static int  deselect_idle_callback (TextboxItem *item);
 static gboolean is_in_area (SheetItem *object, SheetPos *p1, SheetPos *p2);
-inline static void get_cached_bounds (TextboxItem *item, SheetPos *p1, SheetPos *p2);
+inline static void get_cached_bounds (TextboxItem *item, SheetPos *p1,
+	SheetPos *p2);
 
 static void textbox_item_place (SheetItem *item, SchematicView *sv);
 static void textbox_item_place_ghost (SheetItem *item, SchematicView *sv);
@@ -76,8 +87,9 @@ static void edit_cmd (GtkWidget *widget, SchematicView *sv);
 static void edit_textbox (SheetItem *sheet_item);
 
 typedef struct {
-	GnomeDialog *dialog;
-	GtkWidget *entry;
+	GtkDialog *dialog;
+	GtkFontSelection *font;
+	GtkEntry *entry;
 } TextboxPropDialog;
 
 static TextboxPropDialog *prop_dialog = NULL;
@@ -85,11 +97,11 @@ static SheetItemClass *textbox_item_parent_class = NULL;
 
 static GnomeUIInfo textbox_popup_menu [] = {
 	GNOMEUIINFO_SEPARATOR,
-	
-	{ GNOME_APP_UI_ITEM, N_("Edit..."), N_("Edit the text"), edit_cmd, NULL, NULL,
-	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PROP, 0, 0 },
-	GNOMEUIINFO_END
-};
+
+	{ GNOME_APP_UI_ITEM, N_("Edit..."), N_("Edit the text"), edit_cmd,
+	  NULL, NULL, GNOME_APP_PIXMAP_STOCK, GTK_STOCK_PROPERTIES, 0, 0 },
+
+	GNOMEUIINFO_END };
 
 enum {
 	TEXTBOX_ITEM_ARG_0,
@@ -97,10 +109,10 @@ enum {
 };
 
 struct _TextboxItemPriv {
-	gboolean cache_valid : 1;
+	guint cache_valid : 1;
+
+	guint highlight : 1;
 
-	gboolean highlight : 1;
-	
 	// FIXME: More members.
 	GnomeCanvasItem *text_canvas_item;
 
@@ -112,23 +124,27 @@ struct _TextboxItemPriv {
 	SheetPos bbox_end;
 };
 
-GtkType
+GType
 textbox_item_get_type ()
 {
-	static GtkType textbox_item_type = 0;
-	
+	static GType textbox_item_type = 0;
+
 	if (!textbox_item_type) {
-		GtkTypeInfo textbox_item_info = {
-			"TextboxItem",
-			sizeof (TextboxItem),
-			sizeof (TextboxItemClass),
-			(GtkClassInitFunc) textbox_item_class_init,
-			(GtkObjectInitFunc) textbox_item_init,
-			(GtkArgSetFunc) NULL,
-			(GtkArgGetFunc) NULL,
+		static const GTypeInfo textbox_item_info = {
+			sizeof(TextboxItemClass),
+			NULL,
+			NULL,
+			(GClassInitFunc)textbox_item_class_init,
+			NULL,
+			NULL,
+			sizeof(TextboxItem),
+			0,
+			(GInstanceInitFunc)textbox_item_init,
 			NULL
 		};
-		textbox_item_type = gtk_type_unique (sheet_item_get_type (), &textbox_item_info);
+
+		textbox_item_type = g_type_register_static(TYPE_SHEET_ITEM,
+			"TextboxItem", &textbox_item_info, 0);
 	}
 	return textbox_item_type;
 }
@@ -136,18 +152,21 @@ textbox_item_get_type ()
 static void
 textbox_item_class_init (TextboxItemClass *textbox_item_class)
 {
-	GtkObjectClass *object_class;
+	GObjectClass *object_class;
+	GtkObjectClass *gtk_object_class;
 	SheetItemClass *sheet_item_class;
-	
-	object_class = (GtkObjectClass *) textbox_item_class;
-	sheet_item_class = (SheetItemClass *) textbox_item_class;
-	textbox_item_parent_class = gtk_type_class (sheet_item_get_type ());
 
-	gtk_object_add_arg_type ("TextboxItem::name", GTK_TYPE_POINTER, GTK_ARG_READWRITE, TEXTBOX_ITEM_ARG_NAME);
+	object_class = G_OBJECT_CLASS(textbox_item_class);
+	gtk_object_class = GTK_OBJECT_CLASS(textbox_item_class);
+	sheet_item_class = SHEET_ITEM_CLASS(textbox_item_class);
+	textbox_item_parent_class =
+		g_type_class_peek_parent(textbox_item_class);
+/*	gtk_object_add_arg_type ("TextboxItem::name",
+ *              GTK_TYPE_POINTER, GTK_ARG_READWRITE, TEXTBOX_ITEM_ARG_NAME);
 
 	object_class->set_arg = textbox_item_set_arg;
-	object_class->get_arg = textbox_item_get_arg;
-	object_class->destroy = textbox_item_destroy;
+	object_class->get_arg = textbox_item_get_arg;*/
+	gtk_object_class->destroy = textbox_item_destroy;
 
 	sheet_item_class->moved = textbox_item_moved;
 	sheet_item_class->paste = textbox_item_paste;
@@ -159,7 +178,8 @@ textbox_item_class_init (TextboxItemClass *textbox_item_class)
 
 	sheet_item_class->context_menu = g_new0 (SheetItemMenu, 1);
 	sheet_item_class->context_menu->menu = textbox_popup_menu;
-	sheet_item_class->context_menu->size = sizeof (textbox_popup_menu) / sizeof (textbox_popup_menu[0]);
+	sheet_item_class->context_menu->size =
+		sizeof (textbox_popup_menu) / sizeof (textbox_popup_menu[0]);
 }
 
 static void
@@ -174,8 +194,8 @@ textbox_item_init (TextboxItem *item)
 	priv->cache_valid = FALSE;
 }
 
-static void
-textbox_item_set_arg (GtkObject *object, GtkArg *arg, guint arg_id)
+/*static void
+textbox_item_set_arg (GObject *object, GtkArg *arg, guint arg_id)
 {
 	TextboxItem *textbox_item = TEXTBOX_ITEM (object);
 
@@ -201,7 +221,7 @@ textbox_item_get_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 		arg->type = GTK_TYPE_INVALID;
 		break;
 	}
-}
+}*/
 
 static void
 textbox_item_destroy (GtkObject *object)
@@ -215,10 +235,16 @@ textbox_item_destroy (GtkObject *object)
 	textbox = TEXTBOX_ITEM (object);
 	priv = textbox->priv;
 
-	g_free (priv);
+	if (priv) {
+		if (priv->text_canvas_item) {
+			gtk_object_destroy(GTK_OBJECT(priv->text_canvas_item));
+		}
+		textbox->priv = NULL;
+		g_free (priv);
+	}
 
-	if (GTK_OBJECT_CLASS (textbox_item_parent_class)->destroy){
-		(* GTK_OBJECT_CLASS (textbox_item_parent_class)->destroy) (object);
+	if (GTK_OBJECT_CLASS(textbox_item_parent_class)->destroy){
+		GTK_OBJECT_CLASS(textbox_item_parent_class)->destroy(object);
 	}
 }
 
@@ -245,18 +271,15 @@ textbox_item_new (Sheet *sheet, Textbox *textbox)
 	TextboxItem *item;
 	TextboxItemPriv *priv;
 	SheetPos pos;
-	
-	g_return_val_if_fail (sheet != NULL, NULL);
-	g_return_val_if_fail (IS_SHEET (sheet), NULL);
 
-	gtk_object_ref (GTK_OBJECT (textbox));
-	gtk_object_sink (GTK_OBJECT (textbox));
+	g_return_val_if_fail(sheet != NULL, NULL);
+	g_return_val_if_fail(IS_SHEET(sheet), NULL);
 
-	item_data_get_pos (ITEM_DATA (textbox), &pos);
+	item_data_get_pos(ITEM_DATA(textbox), &pos);
 
-	item = TEXTBOX_ITEM (gnome_canvas_item_new (
+	item = TEXTBOX_ITEM(gnome_canvas_item_new (
 		sheet->object_group,
-		textbox_item_get_type (),
+		textbox_item_get_type(),
 		"data", textbox,
 		"x", (double) pos.x,
 		"y", (double) pos.y,
@@ -274,12 +297,22 @@ textbox_item_new (Sheet *sheet, Textbox *textbox)
 		"anchor", GTK_ANCHOR_SW,
 		"font", TEXTBOX_FONT,
 		NULL);
-	
-	gtk_signal_connect_while_alive (GTK_OBJECT (textbox), "rotated", textbox_rotated_callback, item, GTK_OBJECT (item));
-	gtk_signal_connect_while_alive (GTK_OBJECT (textbox), "flipped", textbox_flipped_callback, item, GTK_OBJECT (item));
-	gtk_signal_connect_while_alive (GTK_OBJECT (textbox), "moved", textbox_moved_callback, item, GTK_OBJECT (item));
-	gtk_signal_connect_while_alive (GTK_OBJECT (textbox), "text_changed", textbox_text_changed_callback, item, GTK_OBJECT (item));
-	gtk_signal_connect_while_alive (GTK_OBJECT (textbox), "font_changed", textbox_font_changed_callback, item, GTK_OBJECT (item));
+
+	g_signal_connect_object(G_OBJECT (textbox),
+		"rotated", G_CALLBACK(textbox_rotated_callback),
+		G_OBJECT(item), 0);
+	g_signal_connect_object(G_OBJECT (textbox),
+		"flipped", G_CALLBACK(textbox_flipped_callback),
+		G_OBJECT(item), 0);
+	g_signal_connect_object(G_OBJECT (textbox),
+		"moved", G_CALLBACK(textbox_moved_callback),
+		G_OBJECT(item), 0);
+	g_signal_connect_object(G_OBJECT (textbox),
+		"text_changed", G_CALLBACK(textbox_text_changed_callback),
+		G_OBJECT(item), 0);
+	g_signal_connect_object(G_OBJECT (textbox),
+		"font_changed", G_CALLBACK(textbox_font_changed_callback),
+		G_OBJECT(item), 0);
 
 	textbox_update_bbox (textbox);
 
@@ -287,12 +320,13 @@ textbox_item_new (Sheet *sheet, Textbox *textbox)
 }
 
 void
-textbox_item_signal_connect_placed (TextboxItem *textbox_item, SchematicView *sv)
+textbox_item_signal_connect_placed (TextboxItem *textbox_item,
+	SchematicView *sv)
 {
-	gtk_signal_connect (
-		GTK_OBJECT (textbox_item), 
+	g_signal_connect (
+		G_OBJECT (textbox_item),
 		"event",
-		GTK_SIGNAL_FUNC (sheet_item_event),
+		G_CALLBACK(sheet_item_event),
 		sv);
 }
 
@@ -310,7 +344,8 @@ textbox_rotated_callback (ItemData *data, int angle, SheetItem *sheet_item)
 }
 
 static void
-textbox_flipped_callback (ItemData *data, gboolean horizontal, SheetItem *sheet_item)
+textbox_flipped_callback (ItemData *data,
+	gboolean horizontal, SheetItem *sheet_item)
 {
 	TextboxItem *item;
 
@@ -328,15 +363,16 @@ select_idle_callback (TextboxItem *item)
 	SheetPos bbox_start, bbox_end;
 	TextboxItemPriv *priv = item->priv;
 
-	if (GTK_OBJECT_DESTROYED (item))
-		return FALSE;
+	//	if (GTK_OBJECT_DESTROYED (item))
+	//		return FALSE;
 
 	get_cached_bounds (item, &bbox_start, &bbox_end);
-	gnome_canvas_item_set (GNOME_CANVAS_ITEM (priv->text_canvas_item), "fill_color", SELECTED_COLOR, NULL);
+	gnome_canvas_item_set (GNOME_CANVAS_ITEM (priv->text_canvas_item),
+		"fill_color", SELECTED_COLOR, NULL);
 
 	priv->highlight = TRUE;
 
-	gtk_object_unref (GTK_OBJECT (item));	
+	//	g_object_unref(G_OBJECT(item));
 	return FALSE;
 }
 
@@ -345,24 +381,22 @@ deselect_idle_callback (TextboxItem *item)
 {
 	TextboxItemPriv *priv = item->priv;
 
-	if (GTK_OBJECT_DESTROYED (item))
-		return FALSE;
-
-	gnome_canvas_item_set (GNOME_CANVAS_ITEM (priv->text_canvas_item), "fill_color", NORMAL_COLOR, NULL);
+	gnome_canvas_item_set (GNOME_CANVAS_ITEM (priv->text_canvas_item),
+		"fill_color", NORMAL_COLOR, NULL);
 
 	priv->highlight = FALSE;
 
-	gtk_object_unref (GTK_OBJECT (item));
+	//g_object_unref(G_OBJECT(item));
 	return FALSE;
 }
 
 static void
 selection_changed (TextboxItem *item, gboolean select, gpointer user_data)
 {
-	gtk_object_ref (GTK_OBJECT (item));		
+	//	g_object_ref(G_OBJECT(item));
 	if (select)
 		gtk_idle_add ((gpointer) select_idle_callback, item);
-	else 
+	else
 		gtk_idle_add ((gpointer) deselect_idle_callback, item);
 }
 
@@ -375,7 +409,7 @@ is_in_area (SheetItem *object, SheetPos *p1, SheetPos *p2)
 	item = TEXTBOX_ITEM (object);
 
 	get_cached_bounds (item, &bbox_start, &bbox_end);
-	
+
 	if (p1->x < bbox_start.x &&
 	    p2->x > bbox_end.x &&
 	    p1->y < bbox_start.y &&
@@ -385,14 +419,15 @@ is_in_area (SheetItem *object, SheetPos *p1, SheetPos *p2)
 	return FALSE;
 }
 
-/*
+/**
  * Retrieves the bounding box. We use a caching scheme for this
  * since it's too expensive to calculate it every time we need it.
  */
 inline static void
 get_cached_bounds (TextboxItem *item, SheetPos *p1, SheetPos *p2)
 {
-	GdkFont *font;
+	PangoFontDescription *font;
+	PangoFontMetrics *font_metric;
 	int width;
 	int rbearing;
 	int lbearing;
@@ -405,30 +440,32 @@ get_cached_bounds (TextboxItem *item, SheetPos *p1, SheetPos *p2)
 	if (!priv->cache_valid) {
 		SheetPos start_pos, end_pos;
 
-		font = gdk_font_load (TEXTBOX_FONT);
-		gdk_string_extents (font,
-				    textbox_get_text (TEXTBOX (sheet_item_get_data (SHEET_ITEM (item)))),
-				    &lbearing,
-				    &rbearing,
-				    &width,
-				    &ascent,
-				    &descent);
-		gdk_font_unref (font);
-		
-		item_data_get_pos (sheet_item_get_data (SHEET_ITEM (item)), &pos);
-		
+		font = pango_font_description_from_string(TEXTBOX_FONT);
+		//gdk_string_extents (font,
+		//		    textbox_get_text (TEXTBOX (sheet_item_get_data (SHEET_ITEM (item)))),
+		//		    &lbearing,
+		//		    &rbearing,
+		//		    &width,
+		//		    &ascent,
+		//		    &descent);
+
+		item_data_get_pos (sheet_item_get_data (SHEET_ITEM (item)),
+			&pos);
+
 		start_pos.x = pos.x;
-		start_pos.y = pos.y - font->ascent;
-		end_pos.x = pos.x + rbearing;
-		end_pos.y = pos.y + font->descent; 
+		start_pos.y = pos.y-5;// - font->ascent;
+		end_pos.x = pos.x+5; // + rbearing;
+		end_pos.y = pos.y+5;// + font->descent;
 
 		priv->bbox_start = start_pos;
 		priv->bbox_end = end_pos;
 		priv->cache_valid = TRUE;
+		pango_font_description_free(font);
 	}
-	
+
 	memcpy (p1, &priv->bbox_start, sizeof (SheetPos));
 	memcpy (p2, &priv->bbox_end, sizeof (SheetPos));
+
 }
 
 static void
@@ -442,7 +479,7 @@ textbox_item_paste (SchematicView *sv, ItemData *data)
 	schematic_view_add_ghost_item (sv, data);
 }
 
-/*
+/**
  * This is called when the textbox data was moved. Update the view accordingly.
  */
 static void
@@ -460,7 +497,7 @@ textbox_moved_callback (ItemData *data, SheetPos *pos, SheetItem *item)
 
 	textbox_item = TEXTBOX_ITEM (item);
 
-	/* 
+	/*
 	 * Move the canvas item and invalidate the bbox cache.
 	 */
 	gnome_canvas_item_move (GNOME_CANVAS_ITEM (item), pos->x, pos->y);
@@ -468,7 +505,8 @@ textbox_moved_callback (ItemData *data, SheetPos *pos, SheetItem *item)
 }
 
 static void
-textbox_text_changed_callback (ItemData *data, gchar *new_text, SheetItem *item)
+textbox_text_changed_callback (ItemData *data,
+	gchar *new_text, SheetItem *item)
 {
 	TextboxItem *textbox_item;
 
@@ -478,12 +516,16 @@ textbox_text_changed_callback (ItemData *data, gchar *new_text, SheetItem *item)
 	g_return_if_fail (IS_TEXTBOX_ITEM (item));
 
 	textbox_item = TEXTBOX_ITEM (item);
-	
-	gnome_canvas_item_set (GNOME_CANVAS_ITEM (textbox_item->priv->text_canvas_item), "text", new_text, NULL);
+
+	gnome_canvas_item_set (
+		GNOME_CANVAS_ITEM ( textbox_item->priv->text_canvas_item ),
+		"text", new_text, NULL );
+
 }
 
 static void
-textbox_font_changed_callback (ItemData *data, gchar *new_font, SheetItem *item)
+textbox_font_changed_callback (ItemData *data,
+	gchar *new_font, SheetItem *item)
 {
 	TextboxItem *textbox_item;
 
@@ -491,10 +533,13 @@ textbox_font_changed_callback (ItemData *data, gchar *new_font, SheetItem *item)
 	g_return_if_fail (IS_ITEM_DATA (data));
 	g_return_if_fail (item != NULL);
 	g_return_if_fail (IS_TEXTBOX_ITEM (item));
+	g_return_if_fail (new_font != NULL);
 
 	textbox_item = TEXTBOX_ITEM (item);
-	
-	gnome_canvas_item_set (GNOME_CANVAS_ITEM (textbox_item->priv->text_canvas_item), "font", new_font, NULL);
+
+	gnome_canvas_item_set (
+		GNOME_CANVAS_ITEM (textbox_item->priv->text_canvas_item),
+		"font", new_font, NULL);
 }
 
 static void
@@ -502,10 +547,10 @@ textbox_item_place (SheetItem *item, SchematicView *sv)
 {
 	textbox_item_signal_connect_placed (TEXTBOX_ITEM (item), sv);
 
-	gtk_signal_connect (
-		GTK_OBJECT (item), 
+	g_signal_connect (
+		G_OBJECT (item),
 		"double_clicked",
-		GTK_SIGNAL_FUNC (edit_textbox),
+		G_CALLBACK(edit_textbox),
 		item);
 }
 
@@ -547,19 +592,22 @@ create_textbox_event (Sheet *sheet, GdkEvent *event, SchematicView *sv)
 
 			pos.x = event->button.x;
 			pos.y = event->button.y;
-			
+
 			textbox = textbox_new (NULL);
 			item_data_set_pos (ITEM_DATA (textbox), &pos);
 			textbox_set_text (textbox, _("Label"));
 
-			schematic_add_item (schematic_view_get_schematic (sv), ITEM_DATA (textbox));
+			schematic_add_item (schematic_view_get_schematic (sv),
+				ITEM_DATA (textbox));
 
-			gtk_signal_emit_by_name (GTK_OBJECT (sheet), "reset_tool", NULL);
-			gtk_signal_disconnect_by_func (GTK_OBJECT (sheet), GTK_SIGNAL_FUNC (create_textbox_event), sv);
+			g_signal_emit_by_name(G_OBJECT(sheet),
+				"reset_tool", NULL);
+			g_signal_handlers_disconnect_by_func (G_OBJECT (sheet),
+				G_CALLBACK(create_textbox_event), sv);
 		}
-			
+
 		return TRUE;
-		
+
 	default:
 		return FALSE;
 	}
@@ -583,31 +631,19 @@ textbox_item_listen (SchematicView *sv)
 	 * let the user create a new textbox.
 	 */
 	sheet->state = SHEET_STATE_TEXTBOX_WAIT;
-	gtk_signal_connect (GTK_OBJECT (sheet), "event", GTK_SIGNAL_FUNC (create_textbox_event), sv);
-}
-
-static void
-edit_dialog_close (void)
-{
-	gnome_dialog_close (prop_dialog->dialog);
-	prop_dialog = NULL;
-}
-
-static void
-edit_dialog_cancel_cb (GtkWidget *widget, gpointer data)
-{
-	edit_dialog_close ();
+	g_signal_connect(G_OBJECT(sheet),
+		"event", G_CALLBACK(create_textbox_event), sv);
 }
 
 /*
  * Go through the properties and commit the changes.
  */
 static void
-edit_dialog_ok_cb (GtkWidget *widget, TextboxItem *item)
+edit_dialog_ok(TextboxItem *item)
 {
 	TextboxItemPriv *priv;
 	Textbox *textbox;
-	char *value;
+	const gchar *value;
 
 	g_return_if_fail (item != NULL);
 	g_return_if_fail (IS_TEXTBOX_ITEM (item));
@@ -615,24 +651,24 @@ edit_dialog_ok_cb (GtkWidget *widget, TextboxItem *item)
 	priv = item->priv;
 	textbox = TEXTBOX (sheet_item_get_data (SHEET_ITEM (item)));
 
-	value = gtk_editable_get_chars (GTK_EDITABLE (prop_dialog->entry), 0, -1);
+	value = gtk_entry_get_text(GTK_ENTRY(prop_dialog->entry));
 
-	textbox_set_text (textbox, value);
-	g_free (value);
-
-	edit_dialog_close ();
+	textbox_set_text(textbox, value);
+	textbox_set_font(textbox,
+		gtk_font_selection_get_font_name(prop_dialog->font));
 }
 
-static void 
+static void
 edit_textbox (SheetItem *sheet_item)
-{       
+{
 	Sheet *sheet;
 	TextboxItem *item;
 	TextboxItemPriv *priv;
 	Textbox *textbox;
 	char *msg, *value;
+	gint response;
 	GladeXML *gui;
-	
+
 	g_return_if_fail (sheet_item != NULL);
 	g_return_if_fail (IS_TEXTBOX_ITEM (sheet_item));
 
@@ -640,37 +676,53 @@ edit_textbox (SheetItem *sheet_item)
 	priv = item->priv;
 	textbox = TEXTBOX (sheet_item_get_data (sheet_item));
 
-	if (!g_file_exists (OREGANO_GLADEDIR "/textbox-properties-dialog.glade")) {
-		msg = g_strdup_printf (_("Could not find the required file:\n%s"), 
-				       OREGANO_GLADEDIR "/textbox-properties-dialog.glade");
+	if (!g_file_test (OREGANO_GLADEDIR "/textbox-properties-dialog.glade2",
+		    G_FILE_TEST_EXISTS)) {
+		msg = g_strdup_printf (
+			_("<span weight=\"bold\">Could not find the required file:\n</span>%s\n"),
+			OREGANO_GLADEDIR "/textbox-properties-dialog.glade2");
 		oregano_error (msg);
 		g_free (msg);
 		return;
 	}
 
-	gui = glade_xml_new (OREGANO_GLADEDIR "/textbox-properties-dialog.glade", NULL);
+	gui = glade_xml_new (
+		OREGANO_GLADEDIR "/textbox-properties-dialog.glade2",
+		NULL, NULL);
 	if (!gui) {
-		oregano_error (_("Could not create textbox properties dialog."));
+		oregano_error (
+			_("<span weight=\"bold\">Could not create textbox properties dialog.</span>"));
 		return;
 	}
 
 	prop_dialog = g_new0 (TextboxPropDialog, 1);
-	prop_dialog->dialog = GNOME_DIALOG (glade_xml_get_widget (gui, "textbox-properties-dialog"));
-	prop_dialog->entry = glade_xml_get_widget (gui, "entry");
+	prop_dialog->dialog = GTK_DIALOG (
+		glade_xml_get_widget (gui, "textbox-properties-dialog"));
+	prop_dialog->font = GTK_FONT_SELECTION (
+		glade_xml_get_widget (gui, "font_selector"));
+	prop_dialog->entry = GTK_ENTRY (glade_xml_get_widget (gui, "entry"));
 
-	value = textbox_get_text (textbox);
+	value = textbox_get_font (textbox);
+	gtk_font_selection_set_font_name (
+		GTK_FONT_SELECTION (prop_dialog->font), value);
+
+	value = textbox_get_text(textbox);
 	gtk_entry_set_text (GTK_ENTRY (prop_dialog->entry), value);
 
 	sheet = sheet_item_get_sheet (SHEET_ITEM (item));
-	sheet_dialog_set_parent (sheet, prop_dialog->dialog);
+	sheet_dialog_set_parent (sheet, (GtkDialog*) prop_dialog->dialog);
 
-	gnome_dialog_button_connect (prop_dialog->dialog, 0, edit_dialog_ok_cb, item);
-	gnome_dialog_button_connect (prop_dialog->dialog, 1, edit_dialog_cancel_cb, item);
+	gtk_dialog_set_default_response (
+		GTK_DIALOG (prop_dialog->dialog), GTK_RESPONSE_OK);
 
-	gnome_dialog_set_default (prop_dialog->dialog, 0);
-	gnome_dialog_editable_enters (prop_dialog->dialog, GTK_EDITABLE (prop_dialog->entry));
+	response = gtk_dialog_run( GTK_DIALOG (prop_dialog->dialog));
+	if (response == GTK_RESPONSE_OK) {
+		edit_dialog_ok(item);
+	}
 
-	gnome_dialog_run_and_close (prop_dialog->dialog);
+	/* Clean the dialog */
+	gtk_widget_destroy(GTK_WIDGET(prop_dialog->dialog));
+	prop_dialog = NULL;
 }
 
 static void
@@ -682,5 +734,3 @@ edit_cmd (GtkWidget *widget, SchematicView *sv)
 	if ((list != NULL) && IS_TEXTBOX_ITEM (list->data)) 
 		edit_textbox (list->data);
 }
-
-
